@@ -1,8 +1,8 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: aes.c,v 1.21 2005/06/11 05:31:15 arma Exp $ */
-const char aes_c_id[] = "$Id: aes.c,v 1.21 2005/06/11 05:31:15 arma Exp $";
+/* $Id: aes.c,v 1.29 2005/10/03 20:53:27 nickm Exp $ */
+const char aes_c_id[] = "$Id: aes.c,v 1.29 2005/10/03 20:53:27 nickm Exp $";
 
 /**
  * \file aes.c
@@ -14,11 +14,30 @@ const char aes_c_id[] = "$Id: aes.c,v 1.21 2005/06/11 05:31:15 arma Exp $";
  **/
 
 #include "orconfig.h"
+#include <openssl/opensslv.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include "aes.h"
 #include "util.h"
+#include "log.h"
+
+/* Use OpenSSL's AES if we're running 0.9.7 or later.  (The f at the end of
+ * the version below means "release"; see opensslv.h) */
+#if OPENSSL_VERSION_NUMBER >= 0x0090700fl
+#define USE_OPENSSL_AES
+#include <openssl/aes.h>
+#include <openssl/evp.h>
+#endif
+
+/* For now, if OpenSSL supports AES, we always use the EVP_CIPHER_CTX version
+ * of it, so OpenSSL can use an engine instead if available.  If the overhead
+ * turns out to suck, we should maybe switch to use OpenSSL's AES directly
+ * when no engine exists.
+ */
+#ifdef USE_OPENSSL_AES
+#define USE_OPENSSL_EVP
+#endif
 
 /*======================================================================*/
 /* From rijndael-alg-fst.h */
@@ -27,19 +46,27 @@ typedef uint64_t u64;
 typedef uint32_t u32;
 typedef uint8_t u8;
 
+#ifndef USE_OPENSSL_AES
 #define MAXKC   (256/32)
 #define MAXKB   (256/8)
 #define MAXNR   14
 
 static int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits);
 static void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16]);
+#endif
 
 /*======================================================================*/
 /* Interface to AES code, and counter implementation */
 
 struct aes_cnt_cipher {
+#if defined(USE_OPENSSL_EVP)
+  EVP_CIPHER_CTX key;
+#elif defined(USE_OPENSSL_AES)
+  AES_KEY key;
+#else
   u32 rk[4*(MAXNR+1)];
   int nr;
+#endif
   u32 counter1;
   u32 counter0;
   u8 buf[16];
@@ -53,6 +80,12 @@ struct aes_cnt_cipher {
 static void
 _aes_fill_buf(aes_cnt_cipher_t *cipher)
 {
+  /* We don't currently use OpenSSL's counter mode implementation because:
+   *  1) some versions have known bugs
+   *  2) its attitude towards IVs is not our own
+   *  3) changing the counter position was not trivial, last time I looked.
+   * None of these issues are insurmountable in principle.
+   */
   u32 counter0 = cipher->counter0;
   u32 counter1 = cipher->counter1;
   u8 buf[16];
@@ -66,18 +99,25 @@ _aes_fill_buf(aes_cnt_cipher_t *cipher)
   buf[ 9] = (counter1 >> 16) & 0xff;
   buf[ 8] = (counter1 >> 24) & 0xff;
 
+#if defined(USE_OPENSSL_EVP)
+  {
+    int outl=16, inl=16;
+    EVP_EncryptUpdate(&cipher->key, cipher->buf, &outl, buf, inl);
+  }
+#elif defined(USE_OPENSSL_AES)
+  AES_encrypt(buf, cipher->buf, &cipher->key);
+#else
   rijndaelEncrypt(cipher->rk, cipher->nr, buf, cipher->buf);
+#endif
 }
 
 /**
  * Return a newly allocated counter-mode AES128 cipher implementation.
  */
 aes_cnt_cipher_t*
-aes_new_cipher()
+aes_new_cipher(void)
 {
-  aes_cnt_cipher_t* result = tor_malloc(sizeof(aes_cnt_cipher_t));
-  memset(result->rk, 0, 4*(MAXNR+1));
-  memset(result->buf, 0, 16);
+  aes_cnt_cipher_t* result = tor_malloc_zero(sizeof(aes_cnt_cipher_t));
 
   return result;
 }
@@ -89,8 +129,21 @@ aes_new_cipher()
 void
 aes_set_key(aes_cnt_cipher_t *cipher, const char *key, int key_bits)
 {
+#if defined(USE_OPENSSL_EVP)
+  const EVP_CIPHER *c;
+  switch (key_bits) {
+    case 128: c = EVP_aes_128_ecb(); break;
+    case 192: c = EVP_aes_192_ecb(); break;
+    case 256: c = EVP_aes_256_ecb(); break;
+    default: tor_assert(0);
+  }
+  EVP_EncryptInit(&cipher->key, c, (const unsigned char*)key, NULL);
+#elif defined(USE_OPENSSL_AES)
+  AES_set_encrypt_key((const unsigned char *)key, key_bits, &(cipher->key));
+#else
   cipher->nr = rijndaelKeySetupEnc(cipher->rk, (const unsigned char*)key,
                                    key_bits);
+#endif
   cipher->counter0 = 0;
   cipher->counter1 = 0;
   cipher->pos = 0;
@@ -103,8 +156,11 @@ void
 aes_free_cipher(aes_cnt_cipher_t *cipher)
 {
   assert(cipher);
+#ifdef USE_OPENSSL_EVP
+  EVP_CIPHER_CTX_cleanup(&cipher->key);
+#endif
   memset(cipher, 0, sizeof(cipher));
-  free(cipher);
+  tor_free(cipher);
 }
 
 /** Encrypt <b>len</b> bytes from <b>input</b>, storing the result in
@@ -158,6 +214,7 @@ aes_adjust_counter(aes_cnt_cipher_t *cipher, long delta)
   aes_set_counter(cipher, counter);
 }
 
+#ifndef USE_OPENSSL_AES
 /*======================================================================*/
 /* From rijndael-alg-fst.c */
 
@@ -554,7 +611,9 @@ static const u32 rcon[] = {
  *
  * @return      the number of rounds for the given cipher key size.
  */
-int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits) {
+int
+rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits)
+{
         int i = 0;
         u32 temp;
 
@@ -635,7 +694,9 @@ int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBit
         return 0;
 }
 
-void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16]) {
+void
+rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16])
+{
         u32 s0, s1, s2, s3, t0, t1, t2, t3;
 #ifndef FULL_UNROLL
     int r;
@@ -815,4 +876,5 @@ void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 c
                 rk[3];
         PUTU32(ct + 12, s3);
 }
+#endif
 
