@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.66 2005/08/26 18:40:44 arma Exp $ */
-const char compat_c_id[] = "$Id: compat.c,v 1.66 2005/08/26 18:40:44 arma Exp $";
+/* $Id: compat.c,v 1.73 2005/10/07 19:33:47 nickm Exp $ */
+const char compat_c_id[] = "$Id: compat.c,v 1.73 2005/10/07 19:33:47 nickm Exp $";
 
 /**
  * \file compat.c
@@ -80,6 +80,12 @@ const char compat_c_id[] = "$Id: compat.c,v 1.66 2005/08/26 18:40:44 arma Exp $"
 #ifdef HAVE_PTHREAD_H
 #include <pthread.h>
 #endif
+#ifdef HAVE_UTIME_H
+#include <utime.h>
+#endif
+#ifdef HAVE_SYS_UTIME_H
+#include <sys/utime.h>
+#endif
 
 #include "log.h"
 #include "util.h"
@@ -269,6 +275,15 @@ replace_file(const char *from, const char *to)
 #endif
 }
 
+/** Change <b>fname</b>'s modification time to now. */
+int
+touch_file(const char *fname)
+{
+  if (utime(fname, NULL)!=0)
+    return -1;
+  return 0;
+}
+
 /** Turn <b>socket</b> into a nonblocking socket.
  */
 void
@@ -425,7 +440,8 @@ tor_socketpair(int family, int type, int protocol, int fd[2])
  * have a low soft limit.) Make sure we set it to at least
  * <b>limit</b>. Return a new limit if we can, or -1 if we fail. */
 int
-set_max_file_descriptors(unsigned long limit, unsigned long cap) {
+set_max_file_descriptors(unsigned long limit, unsigned long cap)
+{
 #ifndef HAVE_GETRLIMIT
   log_fn(LOG_INFO,"This platform is missing getrlimit(). Proceeding.");
   if (limit > cap) {
@@ -473,7 +489,8 @@ set_max_file_descriptors(unsigned long limit, unsigned long cap) {
  * success.  On failure, log and return -1.
  */
 int
-switch_id(char *user, char *group) {
+switch_id(char *user, char *group)
+{
 #ifndef MS_WINDOWS
   struct passwd *pw = NULL;
   struct group *gr = NULL;
@@ -676,12 +693,13 @@ get_uname(void)
 #endif
       {
 #ifdef MS_WINDOWS
-        OSVERSIONINFO info;
+        OSVERSIONINFOEX info;
         int i;
         const char *plat = NULL;
         static struct {
           int major; int minor; const char *version;
         } win_version_table[] = {
+          { 6, 0, "Windows \"Longhorn\"" },
           { 5, 2, "Windows Server 2003" },
           { 5, 1, "Windows XP" },
           { 5, 0, "Windows 2000" },
@@ -693,7 +711,7 @@ get_uname(void)
           { -1, -1, NULL }
         };
         info.dwOSVersionInfoSize = sizeof(info);
-        GetVersionEx(&info);
+        GetVersionEx((LPOSVERSIONINFO)&info);
         if (info.dwMajorVersion == 4 && info.dwMinorVersion == 0) {
           if (info.dwPlatformId == VER_PLATFORM_WIN32_NT)
             plat = "Windows NT 4.0";
@@ -709,10 +727,11 @@ get_uname(void)
           }
         }
         if (plat) {
-          strlcpy(uname_result, plat, sizeof(uname_result));
+          tor_snprintf(uname_result, sizeof(uname_result), "%s %s",
+                       plat, info.szCSDVersion);
         } else {
-          if (info.dwMajorVersion > 5 ||
-              (info.dwMajorVersion==5 && info.dwMinorVersion>2))
+          if (info.dwMajorVersion > 6 ||
+              (info.dwMajorVersion==6 && info.dwMinorVersion>0))
             tor_snprintf(uname_result, sizeof(uname_result),
                          "Very recent version of Windows [major=%d,minor=%d]",
                          (int)info.dwMajorVersion,(int)info.dwMinorVersion);
@@ -721,6 +740,28 @@ get_uname(void)
                          "Unrecognized version of Windows [major=%d,minor=%d]",
                          (int)info.dwMajorVersion,(int)info.dwMinorVersion);
         }
+        if (info.wProductType == VER_NT_DOMAIN_CONTROLLER) {
+          strlcat(uname_result, " [domain controller]", sizeof(uname_result));
+        } else if (info.wProductType == VER_NT_SERVER) {
+          strlcat(uname_result, " [server]", sizeof(uname_result));
+        } else if (info.wProductType == VER_NT_WORKSTATION) {
+          strlcat(uname_result, " [workstation]", sizeof(uname_result));
+        }
+        if (info.wSuiteMask & VER_SUITE_DATACENTER) {
+          strlcat(uname_result, " {datacenter}", sizeof(uname_result));
+        }
+        if (info.wSuiteMask & VER_SUITE_ENTERPRISE) {
+          strlcat(uname_result, " {enterprise}", sizeof(uname_result));
+        }
+        if (info.wSuiteMask & VER_SUITE_PERSONAL) {
+          strlcat(uname_result, " {personal}", sizeof(uname_result));
+        }
+        if (info.wSuiteMask & VER_SUITE_EMBEDDEDNT) {
+          strlcat(uname_result, " {personal}", sizeof(uname_result));
+        }
+        if (info.wSuiteMask & VER_SUITE_PERSONAL) {
+          strlcat(uname_result, " {personal}", sizeof(uname_result));
+        }
 #else
         strlcpy(uname_result, "Unknown platform", sizeof(uname_result));
 #endif
@@ -735,6 +776,9 @@ get_uname(void)
  */
 
 #if defined(USE_PTHREADS)
+/** Wraps a an int (*)(void*) function and its argument so we can
+ * invoke them in a way pthreads would expect.
+ */
 typedef struct tor_pthread_data_t {
   int (*func)(void *);
   void *data;
@@ -802,7 +846,7 @@ spawn_func(int (*func)(void *), void *data)
 /** End the current thread/process.
  */
 void
-spawn_exit()
+spawn_exit(void)
 {
 #if defined(USE_WIN32_THREADS)
   _endthread();
@@ -920,6 +964,7 @@ tor_gmtime_r(const time_t *timep, struct tm *result)
 #endif
 
 #ifdef USE_WIN32_THREADS
+/** A generic lock structure for multithreaded builds. */
 struct tor_mutex_t {
   HANDLE handle;
 };
@@ -969,6 +1014,7 @@ tor_get_thread_id(void)
   return (unsigned long)GetCurrentThreadId();
 }
 #elif defined(USE_PTHREADS)
+/** A generic lock structure for multithreaded builds. */
 struct tor_mutex_t {
   pthread_mutex_t mutex;
 };
@@ -1001,9 +1047,15 @@ tor_mutex_free(tor_mutex_t *m)
 unsigned long
 tor_get_thread_id(void)
 {
-  return (unsigned long)pthread_self();
+  union {
+    pthread_t thr;
+    unsigned long id;
+  } r;
+  r.thr = pthread_self();
+  return r.id;
 }
 #else
+/** A generic lock structure for multithreaded builds. */
 struct tor_mutex_t {
   int _unused;
 };
@@ -1021,7 +1073,8 @@ struct tor_mutex_t {
  * socket to get the error.
  */
 #ifdef MS_WINDOWS
-int tor_socket_errno(int sock)
+int
+tor_socket_errno(int sock)
 {
   int optval, optvallen=sizeof(optval);
   int err = WSAGetLastError();
