@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine
    Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.c,v 1.38 2005/09/13 15:30:22 nickm Exp $ */
-const char container_c_id[] = "$Id: container.c,v 1.38 2005/09/13 15:30:22 nickm Exp $";
+/* $Id: container.c,v 1.44 2005/10/06 04:33:40 nickm Exp $ */
+const char container_c_id[] = "$Id: container.c,v 1.44 2005/10/06 04:33:40 nickm Exp $";
 
 /**
  * \file container.c
@@ -29,8 +29,9 @@ const char container_c_id[] = "$Id: container.c,v 1.38 2005/09/13 15:30:22 nickm
 #define SMARTLIST_DEFAULT_CAPACITY 32
 
 #ifndef FAST_SMARTLIST
+/** A resizeable list of pointers, with associated helpful functionality. */
 struct smartlist_t {
-  /** <b>list</b> has enough capacity to store exactly <b>capacity</b> elements
+  /* <b>list</b> has enough capacity to store exactly <b>capacity</b> elements
    * before it needs to be resized.  Only the first <b>num_used</b> (\<=
    * capacity) elements point to valid data.
    */
@@ -43,7 +44,8 @@ struct smartlist_t {
 /** Allocate and return an empty smartlist.
  */
 smartlist_t *
-smartlist_create() {
+smartlist_create(void)
+{
   smartlist_t *sl = tor_malloc(sizeof(smartlist_t));
   sl->num_used = 0;
   sl->capacity = SMARTLIST_DEFAULT_CAPACITY;
@@ -55,9 +57,10 @@ smartlist_create() {
  * list's elements.
  */
 void
-smartlist_free(smartlist_t *sl) {
-  free(sl->list);
-  free(sl);
+smartlist_free(smartlist_t *sl)
+{
+  tor_free(sl->list);
+  tor_free(sl);
 }
 
 /** Change the capacity of the smartlist to <b>n</b>, so that we can grow
@@ -66,7 +69,9 @@ smartlist_free(smartlist_t *sl) {
  * currently in the list, reduce the list's capacity as much as
  * possible without losing elements.
  */
-void smartlist_set_capacity(smartlist_t *sl, int n) {
+void
+smartlist_set_capacity(smartlist_t *sl, int n)
+{
   if (n < sl->num_used)
     n = sl->num_used;
   if (sl->capacity != n) {
@@ -78,7 +83,8 @@ void smartlist_set_capacity(smartlist_t *sl, int n) {
 /** Remove all elements from the list.
  */
 void
-smartlist_clear(smartlist_t *sl) {
+smartlist_clear(smartlist_t *sl)
+{
   sl->num_used = 0;
 }
 
@@ -95,7 +101,8 @@ smartlist_truncate(smartlist_t *sl, int len)
 
 /** Append element to the end of the list. */
 void
-smartlist_add(smartlist_t *sl, void *element) {
+smartlist_add(smartlist_t *sl, void *element)
+{
   if (sl->num_used >= sl->capacity) {
     int higher = sl->capacity * 2;
     tor_assert(higher > sl->capacity); /* detect overflow */
@@ -109,7 +116,16 @@ smartlist_add(smartlist_t *sl, void *element) {
 void
 smartlist_add_all(smartlist_t *sl, const smartlist_t *s2)
 {
-  SMARTLIST_FOREACH(s2, void *, element, smartlist_add(sl, element));
+  int n2 = sl->num_used + s2->num_used;
+  if (n2 > sl->capacity) {
+    int higher = sl->capacity * 2;
+    while (n2 > higher)
+      higher *= 2;
+    sl->capacity = higher;
+    sl->list = tor_realloc(sl->list, sizeof(void*)*sl->capacity);
+  }
+  memcpy(sl->list + sl->num_used, s2->list, s2->num_used*sizeof(void*));
+  sl->num_used += s2->num_used;
 }
 
 /** Remove all elements E from sl such that E==element.  Preserve
@@ -469,14 +485,14 @@ smartlist_sort_strings(smartlist_t *sl)
   smartlist_sort(sl, _compare_string_ptrs);
 }
 
-/** Splay-tree implementation of string-to-void* map
- */
+/** A node in a strmap_t string-to-void* map. */
 typedef struct strmap_entry_t {
   SPLAY_ENTRY(strmap_entry_t) node;
   char *key;
   void *val;
 } strmap_entry_t;
 
+/** Splay-tree implementation of string-to-void* map */
 struct strmap_t {
   SPLAY_HEAD(strmap_tree, strmap_entry_t) head;
 };
@@ -535,7 +551,8 @@ strmap_set(strmap_t *map, const char *key, void *val)
 /** Return the current value associated with <b>key</b>, or NULL if no
  * value is set.
  */
-void *strmap_get(strmap_t *map, const char *key)
+void *
+strmap_get(strmap_t *map, const char *key)
 {
   strmap_entry_t *resolve;
   strmap_entry_t search;
