@@ -1,11 +1,11 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.h,v 1.15 2005/09/09 23:12:54 nickm Exp $ */
+/* $Id: container.h,v 1.17 2005/09/12 08:27:01 nickm Exp $ */
 
 #ifndef __CONTAINER_H
 #define __CONTAINER_H
-#define CONTAINER_H_ID "$Id: container.h,v 1.15 2005/09/09 23:12:54 nickm Exp $"
+#define CONTAINER_H_ID "$Id: container.h,v 1.17 2005/09/12 08:27:01 nickm Exp $"
 
 /** Generic resizeable array. */
 typedef struct smartlist_t smartlist_t;
@@ -52,6 +52,7 @@ void smartlist_del_keeporder(smartlist_t *sl, int idx);
 void smartlist_insert(smartlist_t *sl, int idx, void *val);
 void smartlist_sort(smartlist_t *sl,
                     int (*compare)(const void **a, const void **b));
+void smartlist_sort_strings(smartlist_t *sl);
 void *smartlist_bsearch(smartlist_t *sl, const void *key,
                         int (*compare)(const void *key, const void **member));
 
@@ -64,6 +65,22 @@ char *smartlist_join_strings(smartlist_t *sl, const char *join, int terminate,
 char *smartlist_join_strings2(smartlist_t *sl, const char *join,
                               size_t join_len, int terminate, size_t *len_out);
 
+/** Iterate over the items in a smartlist <b>sl</b>, in order.  For each item,
+ * assign it to a new local variable of type <b>type</b> named <b>var</b>, and
+ * execute the statement <b>cmd</b>.  Inside the loop, the loop index can
+ * be accessed as <b>var</b>_sl_idx.
+ *
+ * Example use:
+ * <pre>
+ *   smartlist_t *list = smartlist_split("A:B:C", ":", 0, 0);
+ *   SMARTLIST_FOREACH(list, char *, cp,
+ *   {
+ *     printf("%d: %s\n", cp_sl_idx, cp);
+ *     tor_free(cp);
+ *   });
+ *   smarlitst_free(list);
+ * </pre>
+ */
 #define SMARTLIST_FOREACH(sl, type, var, cmd)                   \
   do {                                                          \
     int var ## _sl_idx, var ## _sl_len=smartlist_len(sl);       \
