@@ -1,8 +1,8 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: crypto.c,v 1.155 2005/09/09 22:07:15 nickm Exp $ */
-const char crypto_c_id[] = "$Id: crypto.c,v 1.155 2005/09/09 22:07:15 nickm Exp $";
+/* $Id: crypto.c,v 1.167 2005/10/07 19:53:57 arma Exp $ */
+const char crypto_c_id[] = "$Id: crypto.c,v 1.167 2005/10/07 19:53:57 arma Exp $";
 
 /**
  * \file crypto.c
@@ -91,18 +91,22 @@ static tor_mutex_t **_openssl_mutexes = NULL;
 static int _n_openssl_mutexes = -1;
 #endif
 
+/** A public key, or a public/private keypair. */
 struct crypto_pk_env_t
 {
   int refs; /* reference counting so we don't have to copy keys */
   RSA *key;
 };
 
+/** Key and stream information for a stream cipher. */
 struct crypto_cipher_env_t
 {
   char key[CIPHER_KEY_LEN];
   aes_cnt_cipher_t *cipher;
 };
 
+/** A structure to hold the first half (x, g^x) of a Diffie-Hellman handshake
+ * while we're waiting for the second.*/
 struct crypto_dh_env_t {
   DH *dh;
 };
@@ -119,7 +123,8 @@ static int tor_check_dh_key(BIGNUM *bn);
 /** Return the number of bytes added by padding method <b>padding</b>.
  */
 static INLINE int
-crypto_get_rsa_padding_overhead(int padding) {
+crypto_get_rsa_padding_overhead(int padding)
+{
   switch (padding)
     {
     case RSA_NO_PADDING: return 0;
@@ -132,7 +137,8 @@ crypto_get_rsa_padding_overhead(int padding) {
 /** Given a padding method <b>padding</b>, return the correct OpenSSL constant.
  */
 static INLINE int
-crypto_get_rsa_padding(int padding) {
+crypto_get_rsa_padding(int padding)
+{
   switch (padding)
     {
     case PK_NO_PADDING: return RSA_NO_PADDING;
@@ -216,8 +222,10 @@ crypto_global_init(int useAccel)
 
 /** Uninitialize the crypto library. Return 0 on success, -1 on failure.
  */
-int crypto_global_cleanup()
+int
+crypto_global_cleanup(void)
 {
+  EVP_cleanup();
   ERR_free_strings();
 #ifndef NO_ENGINES
   ENGINE_cleanup();
@@ -239,7 +247,8 @@ int crypto_global_cleanup()
 }
 
 /** used by tortls.c: wrap an RSA* in a crypto_pk_env_t. */
-crypto_pk_env_t *_crypto_new_pk_env_rsa(RSA *rsa)
+crypto_pk_env_t *
+_crypto_new_pk_env_rsa(RSA *rsa)
 {
   crypto_pk_env_t *env;
   tor_assert(rsa);
@@ -250,14 +259,16 @@ crypto_pk_env_t *_crypto_new_pk_env_rsa(RSA *rsa)
 }
 
 /** used by tortls.c: return the RSA* from a crypto_pk_env_t. */
-RSA *_crypto_pk_env_get_rsa(crypto_pk_env_t *env)
+RSA *
+_crypto_pk_env_get_rsa(crypto_pk_env_t *env)
 {
   return env->key;
 }
 
 /** used by tortls.c: get an equivalent EVP_PKEY* for a crypto_pk_env_t.  Iff
  * private is set, include the private-key portion of the key. */
-EVP_PKEY *_crypto_pk_env_get_evp_pkey(crypto_pk_env_t *env, int private)
+EVP_PKEY *
+_crypto_pk_env_get_evp_pkey(crypto_pk_env_t *env, int private)
 {
   RSA *key = NULL;
   EVP_PKEY *pkey = NULL;
@@ -284,7 +295,8 @@ EVP_PKEY *_crypto_pk_env_get_evp_pkey(crypto_pk_env_t *env, int private)
 
 /** Used by tortls.c: Get the DH* from a crypto_dh_env_t.
  */
-DH *_crypto_dh_env_get_dh(crypto_dh_env_t *dh)
+DH *
+_crypto_dh_env_get_dh(crypto_dh_env_t *dh)
 {
   return dh->dh;
 }
@@ -292,7 +304,8 @@ DH *_crypto_dh_env_get_dh(crypto_dh_env_t *dh)
 /** Allocate and return storage for a public key.  The key itself will not yet
  * be set.
  */
-crypto_pk_env_t *crypto_new_pk_env(void)
+crypto_pk_env_t *
+crypto_new_pk_env(void)
 {
   RSA *rsa;
 
@@ -304,7 +317,8 @@ crypto_pk_env_t *crypto_new_pk_env(void)
 /** Release a reference to an asymmetric key; when all the references
  * are released, free the key.
  */
-void crypto_free_pk_env(crypto_pk_env_t *env)
+void
+crypto_free_pk_env(crypto_pk_env_t *env)
 {
   tor_assert(env);
 
@@ -314,7 +328,7 @@ void crypto_free_pk_env(crypto_pk_env_t *env)
   if (env->key)
     RSA_free(env->key);
 
-  free(env);
+  tor_free(env);
 }
 
 /** Create a new symmetric cipher for a given key and encryption flag
@@ -354,7 +368,8 @@ crypto_create_init_cipher(const char *key, int encrypt_mode)
 
 /** Allocate and return a new symmetric cipher.
  */
-crypto_cipher_env_t *crypto_new_cipher_env()
+crypto_cipher_env_t *
+crypto_new_cipher_env(void)
 {
   crypto_cipher_env_t *env;
 
@@ -365,7 +380,8 @@ crypto_cipher_env_t *crypto_new_cipher_env()
 
 /** Free a symmetric cipher.
  */
-void crypto_free_cipher_env(crypto_cipher_env_t *env)
+void
+crypto_free_cipher_env(crypto_cipher_env_t *env)
 {
   tor_assert(env);
 
@@ -379,7 +395,8 @@ void crypto_free_cipher_env(crypto_cipher_env_t *env)
 /** Generate a new public/private keypair in <b>env</b>.  Return 0 on
  * success, -1 on failure.
  */
-int crypto_pk_generate_key(crypto_pk_env_t *env)
+int
+crypto_pk_generate_key(crypto_pk_env_t *env)
 {
   tor_assert(env);
 
@@ -397,8 +414,9 @@ int crypto_pk_generate_key(crypto_pk_env_t *env)
 /** Read a PEM-encoded private key from the string <b>s</b> into <b>env</b>.
  * Return 0 on success, -1 on failure.
  */
-static int crypto_pk_read_private_key_from_string(crypto_pk_env_t *env,
-                                                  const char *s)
+static int
+crypto_pk_read_private_key_from_string(crypto_pk_env_t *env,
+                                       const char *s)
 {
   BIO *b;
 
@@ -425,7 +443,8 @@ static int crypto_pk_read_private_key_from_string(crypto_pk_env_t *env,
 /** Read a PEM-encoded private key from the file named by
  * <b>keyfile</b> into <b>env</b>.  Return 0 on success, -1 on failure.
  */
-int crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *keyfile)
+int
+crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *keyfile)
 {
   char *contents;
   int r;
@@ -455,7 +474,9 @@ int crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *k
  * string, *<b>len</b> to the string's length, and return 0.  On
  * failure, return -1.
  */
-int crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size_t *len) {
+int
+crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size_t *len)
+{
   BUF_MEM *buf;
   BIO *b;
 
@@ -491,7 +512,9 @@ int crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size
  * <b>src</b>, and store the result in <b>env</b>.  Return 0 on success, -1 on
  * failure.
  */
-int crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src, size_t len) {
+int
+crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src, size_t len)
+{
   BIO *b;
 
   tor_assert(env);
@@ -543,7 +566,7 @@ crypto_pk_write_private_key_to_filename(crypto_pk_env_t *env,
   s[len]='\0';
   r = write_str_to_file(fname, s, 0);
   BIO_free(bio);
-  free(s);
+  tor_free(s);
   return r;
 }
 
@@ -556,7 +579,8 @@ crypto_pk_write_private_key_to_filename(crypto_pk_env_t *env,
  * directories containing "opt keyword\n-----BEGIN OBJECT----" entries
  * in versions of Tor up to 0.0.9pre2.</i>
  */
-int crypto_pk_DER64_encode_public_key(crypto_pk_env_t *env, char **out)
+int
+crypto_pk_DER64_encode_public_key(crypto_pk_env_t *env, char **out)
 {
   int len;
   char buf[PK_BYTES*2]; /* Too long, but hey, stacks are big. */
@@ -585,7 +609,8 @@ int crypto_pk_DER64_encode_public_key(crypto_pk_env_t *env, char **out)
  * directories containing "opt keyword\n-----BEGIN OBJECT----" entries
  * in versions of Tor up to 0.0.9pre2.</i>
  */
-crypto_pk_env_t *crypto_pk_DER64_decode_public_key(const char *in)
+crypto_pk_env_t *
+crypto_pk_DER64_decode_public_key(const char *in)
 {
   char partitioned[PK_BYTES*2 + 16];
   char buf[PK_BYTES*2];
@@ -611,7 +636,8 @@ crypto_pk_env_t *crypto_pk_DER64_decode_public_key(const char *in)
 
 /** Return true iff <b>env</b> has a valid key.
  */
-int crypto_pk_check_key(crypto_pk_env_t *env)
+int
+crypto_pk_check_key(crypto_pk_env_t *env)
 {
   int r;
   tor_assert(env);
@@ -625,7 +651,9 @@ int crypto_pk_check_key(crypto_pk_env_t *env)
 /** Compare the public-key components of a and b.  Return -1 if a\<b, 0
  * if a==b, and 1 if a\>b.
  */
-int crypto_pk_cmp_keys(crypto_pk_env_t *a, crypto_pk_env_t *b) {
+int
+crypto_pk_cmp_keys(crypto_pk_env_t *a, crypto_pk_env_t *b)
+{
   int result;
 
   if (!a || !b)
@@ -643,7 +671,8 @@ int crypto_pk_cmp_keys(crypto_pk_env_t *a, crypto_pk_env_t *b) {
 }
 
 /** Return the size of the public key modulus in <b>env</b>, in bytes. */
-size_t crypto_pk_keysize(crypto_pk_env_t *env)
+size_t
+crypto_pk_keysize(crypto_pk_env_t *env)
 {
   tor_assert(env);
   tor_assert(env->key);
@@ -653,7 +682,9 @@ size_t crypto_pk_keysize(crypto_pk_env_t *env)
 
 /** Increase the reference count of <b>env</b>, and return it.
  */
-crypto_pk_env_t *crypto_pk_dup_key(crypto_pk_env_t *env) {
+crypto_pk_env_t *
+crypto_pk_dup_key(crypto_pk_env_t *env)
+{
   tor_assert(env);
   tor_assert(env->key);
 
@@ -827,11 +858,12 @@ crypto_pk_private_sign_digest(crypto_pk_env_t *env, char *to,
  *     padded and encrypted with the public key; followed by the rest of
  *     the source data encrypted in AES-CTR mode with the symmetric key.
  */
-int crypto_pk_public_hybrid_encrypt(crypto_pk_env_t *env,
-                                    char *to,
-                                    const char *from,
-                                    size_t fromlen,
-                                    int padding, int force)
+int
+crypto_pk_public_hybrid_encrypt(crypto_pk_env_t *env,
+                                char *to,
+                                const char *from,
+                                size_t fromlen,
+                                int padding, int force)
 {
   int overhead, outlen, r, symlen;
   size_t pkeylen;
@@ -890,11 +922,12 @@ int crypto_pk_public_hybrid_encrypt(crypto_pk_env_t *env,
 }
 
 /** Invert crypto_pk_public_hybrid_encrypt. */
-int crypto_pk_private_hybrid_decrypt(crypto_pk_env_t *env,
-                                     char *to,
-                                     const char *from,
-                                     size_t fromlen,
-                                     int padding, int warnOnFailure)
+int
+crypto_pk_private_hybrid_decrypt(crypto_pk_env_t *env,
+                                 char *to,
+                                 const char *from,
+                                 size_t fromlen,
+                                 int padding, int warnOnFailure)
 {
   int overhead, outlen, r;
   size_t pkeylen;
@@ -937,7 +970,8 @@ int crypto_pk_private_hybrid_decrypt(crypto_pk_env_t *env,
 /** ASN.1-encode the public portion of <b>pk</b> into <b>dest</b>.
  * Return -1 on error, or the number of characters used on success.
  */
-int crypto_pk_asn1_encode(crypto_pk_env_t *pk, char *dest, int dest_len)
+int
+crypto_pk_asn1_encode(crypto_pk_env_t *pk, char *dest, int dest_len)
 {
   int len;
   unsigned char *buf, *cp;
@@ -962,7 +996,8 @@ int crypto_pk_asn1_encode(crypto_pk_env_t *pk, char *dest, int dest_len)
 /** Decode an ASN.1-encoded public key from <b>str</b>; return the result on
  * success and NULL on failure.
  */
-crypto_pk_env_t *crypto_pk_asn1_decode(const char *str, size_t len)
+crypto_pk_env_t *
+crypto_pk_asn1_decode(const char *str, size_t len)
 {
   RSA *rsa;
   unsigned char *buf;
@@ -989,7 +1024,8 @@ crypto_pk_env_t *crypto_pk_asn1_decode(const char *str, size_t len)
  * public key into <b>digest_out</b> (must have DIGEST_LEN bytes of space).
  * Return 0 on success, -1 on failure.
  */
-int crypto_pk_get_digest(crypto_pk_env_t *pk, char *digest_out)
+int
+crypto_pk_get_digest(crypto_pk_env_t *pk, char *digest_out)
 {
   unsigned char *buf, *bufp;
   int len;
@@ -1001,14 +1037,14 @@ int crypto_pk_get_digest(crypto_pk_env_t *pk, char *digest_out)
   len = i2d_RSAPublicKey(pk->key, &bufp);
   if (len < 0) {
     crypto_log_errors(LOG_WARN,"encoding public key");
-    free(buf);
+    tor_free(buf);
     return -1;
   }
   if (crypto_digest(digest_out, (char*)buf, len) < 0) {
-    free(buf);
+    tor_free(buf);
     return -1;
   }
-  free(buf);
+  tor_free(buf);
   return 0;
 }
 
@@ -1063,7 +1099,8 @@ crypto_pk_check_fingerprint_syntax(const char *s)
 /** Generate a new random key for the symmetric cipher in <b>env</b>.
  * Return 0 on success, -1 on failure.  Does not initialize the cipher.
  */
-int crypto_cipher_generate_key(crypto_cipher_env_t *env)
+int
+crypto_cipher_generate_key(crypto_cipher_env_t *env)
 {
   tor_assert(env);
 
@@ -1074,7 +1111,8 @@ int crypto_cipher_generate_key(crypto_cipher_env_t *env)
  * CIPHER_KEY_LEN bytes of <b>key</b>. Does not initialize the cipher.
  * Return 0 on success, -1 on failure.
  */
-int crypto_cipher_set_key(crypto_cipher_env_t *env, const char *key)
+int
+crypto_cipher_set_key(crypto_cipher_env_t *env, const char *key)
 {
   tor_assert(env);
   tor_assert(key);
@@ -1089,7 +1127,8 @@ int crypto_cipher_set_key(crypto_cipher_env_t *env, const char *key)
 
 /** Return a pointer to the key set for the cipher in <b>env</b>.
  */
-const char *crypto_cipher_get_key(crypto_cipher_env_t *env)
+const char *
+crypto_cipher_get_key(crypto_cipher_env_t *env)
 {
   return env->key;
 }
@@ -1097,7 +1136,8 @@ const char *crypto_cipher_get_key(crypto_cipher_env_t *env)
 /** Initialize the cipher in <b>env</b> for encryption.  Return 0 on
  * success, -1 on failure.
  */
-int crypto_cipher_encrypt_init_cipher(crypto_cipher_env_t *env)
+int
+crypto_cipher_encrypt_init_cipher(crypto_cipher_env_t *env)
 {
   tor_assert(env);
 
@@ -1108,7 +1148,8 @@ int crypto_cipher_encrypt_init_cipher(crypto_cipher_env_t *env)
 /** Initialize the cipher in <b>env</b> for decryption. Return 0 on
  * success, -1 on failure.
  */
-int crypto_cipher_decrypt_init_cipher(crypto_cipher_env_t *env)
+int
+crypto_cipher_decrypt_init_cipher(crypto_cipher_env_t *env)
 {
   tor_assert(env);
 
@@ -1175,13 +1216,15 @@ crypto_cipher_advance(crypto_cipher_env_t *env, long delta)
  * <b>m</b>.  Write the DIGEST_LEN byte result into <b>digest</b>.
  * Return 0 on success, -1 on failure.
  */
-int crypto_digest(char *digest, const char *m, size_t len)
+int
+crypto_digest(char *digest, const char *m, size_t len)
 {
   tor_assert(m);
   tor_assert(digest);
   return (SHA1((const unsigned char*)m,len,(unsigned char*)digest) == NULL);
 }
 
+/** Intermediate information about the digest of a stream of data. */
 struct crypto_digest_env_t {
   SHA_CTX d;
 };
@@ -1200,7 +1243,8 @@ crypto_new_digest_env(void)
 /** Deallocate a digest object.
  */
 void
-crypto_free_digest_env(crypto_digest_env_t *digest) {
+crypto_free_digest_env(crypto_digest_env_t *digest)
+{
   tor_free(digest);
 }
 
@@ -1224,8 +1268,9 @@ crypto_digest_add_bytes(crypto_digest_env_t *digest, const char *data,
  * object; write the first out_len bytes of the result to <b>out</b>.
  * <b>out_len</b> must be \<= DIGEST_LEN.
  */
-void crypto_digest_get_digest(crypto_digest_env_t *digest,
-                              char *out, size_t out_len)
+void
+crypto_digest_get_digest(crypto_digest_env_t *digest,
+                         char *out, size_t out_len)
 {
   static unsigned char r[DIGEST_LEN];
   SHA_CTX tmpctx;
@@ -1272,7 +1317,9 @@ static BIGNUM *dh_param_g = NULL;
 
 /** Initialize dh_param_p and dh_param_g if they are not already
  * set. */
-static void init_dh_param(void) {
+static void
+init_dh_param(void)
+{
   BIGNUM *p, *g;
   int r;
   if (dh_param_p && dh_param_g)
@@ -1303,7 +1350,8 @@ static void init_dh_param(void) {
 
 /** Allocate and return a new DH object for a key exchange.
  */
-crypto_dh_env_t *crypto_dh_new()
+crypto_dh_env_t *
+crypto_dh_new(void)
 {
   crypto_dh_env_t *res = NULL;
 
@@ -1325,13 +1373,14 @@ crypto_dh_env_t *crypto_dh_new()
  err:
   crypto_log_errors(LOG_WARN, "creating DH object");
   if (res && res->dh) DH_free(res->dh); /* frees p and g too */
-  if (res) free(res);
+  if (res) tor_free(res);
   return NULL;
 }
 
 /** Return the length of the DH key in <b>dh</b>, in bytes.
  */
-int crypto_dh_get_bytes(crypto_dh_env_t *dh)
+int
+crypto_dh_get_bytes(crypto_dh_env_t *dh)
 {
   tor_assert(dh);
   return DH_size(dh->dh);
@@ -1340,7 +1389,8 @@ int crypto_dh_get_bytes(crypto_dh_env_t *dh)
 /** Generate \<x,g^x\> for our part of the key exchange.  Return 0 on
  * success, -1 on failure.
  */
-int crypto_dh_generate_public(crypto_dh_env_t *dh)
+int
+crypto_dh_generate_public(crypto_dh_env_t *dh)
 {
  again:
   if (!DH_generate_key(dh->dh)) {
@@ -1362,7 +1412,8 @@ int crypto_dh_generate_public(crypto_dh_env_t *dh)
  * as a <b>pubkey_len</b>-byte value into <b>pubkey</b>. Return 0 on
  * success, -1 on failure.  <b>pubkey_len</b> must be \>= DH_BYTES.
  */
-int crypto_dh_get_public(crypto_dh_env_t *dh, char *pubkey, size_t pubkey_len)
+int
+crypto_dh_get_public(crypto_dh_env_t *dh, char *pubkey, size_t pubkey_len)
 {
   int bytes;
   tor_assert(dh);
@@ -1454,9 +1505,10 @@ tor_check_dh_key(BIGNUM *bn)
  *         SHA1( g^xy || "\x00" ) || SHA1( g^xy || "\x01" ) || ...
  * where || is concatenation.)
  */
-int crypto_dh_compute_secret(crypto_dh_env_t *dh,
-                             const char *pubkey, size_t pubkey_len,
-                             char *secret_out, size_t secret_bytes_out)
+int
+crypto_dh_compute_secret(crypto_dh_env_t *dh,
+                         const char *pubkey, size_t pubkey_len,
+                         char *secret_out, size_t secret_bytes_out)
 {
   char hash[DIGEST_LEN];
   char *secret_tmp = NULL;
@@ -1512,52 +1564,75 @@ int crypto_dh_compute_secret(crypto_dh_env_t *dh,
 
 /** Free a DH key exchange object.
  */
-void crypto_dh_free(crypto_dh_env_t *dh)
+void
+crypto_dh_free(crypto_dh_env_t *dh)
 {
   tor_assert(dh);
   tor_assert(dh->dh);
   DH_free(dh->dh);
-  free(dh);
+  tor_free(dh);
 }
 
 /* random numbers */
 
-/** Seed OpenSSL's random number generator with DIGEST_LEN bytes from the
+/* This is how much entropy OpenSSL likes to add right now, so maybe it will
+ * work for us too. */
+#define ADD_ENTROPY 32
+
+/* Use RAND_poll if openssl is 0.9.6 release or later.  (The "f" means
+   "release".)  */
+// #define USE_RAND_POLL (OPENSSL_VERSION_NUMBER >= 0x0090600fl)
+#define USE_RAND_POLL 0
+
+/** Seed OpenSSL's random number generator with bytes from the
  * operating system.  Return 0 on success, -1 on failure.
  */
-int crypto_seed_rng(void)
+int
+crypto_seed_rng(void)
 {
+  char buf[ADD_ENTROPY];
+  int rand_poll_status;
+
+  /* local variables */
 #ifdef MS_WINDOWS
   static int provider_set = 0;
   static HCRYPTPROV provider;
-  char buf[DIGEST_LEN+1];
-
-  if (!provider_set) {
-    if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
-      if (GetLastError() != NTE_BAD_KEYSET) {
-        log_fn(LOG_ERR,"Can't get CryptoAPI provider [1]");
-        return -1;
-      }
-    }
-    provider_set = 1;
-  }
-  if (!CryptGenRandom(provider, sizeof(buf), buf)) {
-    log_fn(LOG_ERR,"Can't get entropy from CryptoAPI.");
-    return -1;
-  }
-  RAND_seed(buf, sizeof(buf));
-  /* And add the current screen state to the entropy pool for
-   * good measure. */
-  RAND_screen();
-  return 0;
 #else
   static const char *filenames[] = {
     "/dev/srandom", "/dev/urandom", "/dev/random", NULL
   };
   int fd;
   int i, n;
-  char buf[DIGEST_LEN+1];
+#endif
 
+#if USE_RAND_POLL
+  /* OpenSSL 0.9.6 adds a RAND_poll function that knows about more kinds of
+   * entropy than we do.  We'll try calling that, *and* calling our own entropy
+   * functions.  If one succeeds, we'll accept the RNG as seeded. */
+  rand_poll_status = RAND_poll();
+  if (rand_poll_status == 0)
+    log_fn(LOG_WARN, "RAND_poll() failed.");
+#else
+  rand_poll_status = 0;
+#endif
+
+#ifdef MS_WINDOWS
+  if (!provider_set) {
+    if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)) {
+      if (GetLastError() != NTE_BAD_KEYSET) {
+        log_fn(LOG_WARN, "Can't get CryptoAPI provider [1]");
+        return rand_poll_status ? 0 : -1;
+      }
+    }
+    provider_set = 1;
+  }
+  if (!CryptGenRandom(provider, sizeof(buf), buf)) {
+    log_fn(LOG_WARN, "Can't get entropy from CryptoAPI.");
+    return rand_poll_status ? 0 : -1;
+  }
+  RAND_seed(buf, sizeof(buf));
+  return 0;
+#else
   for (i = 0; filenames[i]; ++i) {
     fd = open(filenames[i], O_RDONLY, 0);
     if (fd<0) continue;
@@ -1573,14 +1648,15 @@ int crypto_seed_rng(void)
   }
 
   log_fn(LOG_WARN, "Cannot seed RNG -- no entropy source found.");
-  return -1;
+  return rand_poll_status ? 0 : -1;
 #endif
 }
 
 /** Write n bytes of strong random data to <b>to</b>. Return 0 on
  * success, -1 on failure.
  */
-int crypto_rand(char *to, size_t n)
+int
+crypto_rand(char *to, size_t n)
 {
   int r;
   tor_assert(to);
@@ -1590,22 +1666,11 @@ int crypto_rand(char *to, size_t n)
   return (r == 1) ? 0 : -1;
 }
 
-/** Write n bytes of pseudorandom data to <b>to</b>. Return 0 on
- * success, -1 on failure.
- */
-void crypto_pseudo_rand(char *to, size_t n)
-{
-  tor_assert(to);
-  if (RAND_pseudo_bytes((unsigned char*)to, n) == -1) {
-    log_fn(LOG_ERR, "RAND_pseudo_bytes failed unexpectedly.");
-    crypto_log_errors(LOG_WARN, "generating random data");
-    exit(1);
-  }
-}
-
 /** Return a pseudorandom integer, chosen uniformly from the values
  * between 0 and max-1. */
-int crypto_pseudo_rand_int(unsigned int max) {
+int
+crypto_rand_int(unsigned int max)
+{
   unsigned int val;
   unsigned int cutoff;
   tor_assert(max < UINT_MAX);
@@ -1617,7 +1682,7 @@ int crypto_pseudo_rand_int(unsigned int max) {
    */
   cutoff = UINT_MAX - (UINT_MAX%max);
   while (1) {
-    crypto_pseudo_rand((char*)&val, sizeof(val));
+    crypto_rand((char*)&val, sizeof(val));
     if (val < cutoff)
       return val % max;
   }
@@ -1625,11 +1690,13 @@ int crypto_pseudo_rand_int(unsigned int max) {
 
 /** Return a randomly chosen element of sl; or NULL if sl is empty.
  */
-void *smartlist_choose(const smartlist_t *sl) {
+void *
+smartlist_choose(const smartlist_t *sl)
+{
   size_t len;
   len = smartlist_len(sl);
   if (len)
-    return smartlist_get(sl,crypto_pseudo_rand_int(len));
+    return smartlist_get(sl,crypto_rand_int(len));
   return NULL; /* no elements to choose from */
 }
 
@@ -1688,6 +1755,31 @@ base64_decode(char *dest, size_t destlen, const char *src, size_t srclen)
   return ret;
 }
 
+int
+digest_to_base64(char *d64, const char *digest)
+{
+  char buf[256];
+  base64_encode(buf, sizeof(buf), digest, DIGEST_LEN);
+  buf[BASE64_DIGEST_LEN] = '\0';
+  memcpy(d64, buf, BASE64_DIGEST_LEN+1);
+  return 0;
+}
+
+int
+digest_from_base64(char *digest, const char *d64)
+{
+  char buf_in[BASE64_DIGEST_LEN+3];
+  char buf[256];
+  if (strlen(d64) != BASE64_DIGEST_LEN)
+    return -1;
+  memcpy(buf_in, d64, BASE64_DIGEST_LEN);
+  memcpy(buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
+  if (base64_decode(buf, sizeof(buf), buf_in, strlen(buf_in)) != DIGEST_LEN)
+    return -1;
+  memcpy(digest, buf, DIGEST_LEN);
+  return 0;
+}
+
 /** Implements base32 encoding as in rfc3548.  Limitation: Requires
  * that srclen*8 is a multiple of 5.
  */
@@ -1768,8 +1860,10 @@ _openssl_locking_cb(int mode, int n, const char *file, int line)
   else
     tor_mutex_release(_openssl_mutexes[n]);
 }
+
 static int
-setup_openssl_threading(void) {
+setup_openssl_threading(void)
+{
   int i;
   int n = CRYPTO_num_locks();
   _n_openssl_mutexes = n;
@@ -1781,6 +1875,10 @@ setup_openssl_threading(void) {
   return 0;
 }
 #else
-static int setup_openssl_threading(void) { return 0; }
+static int
+setup_openssl_threading(void)
+{
+  return 0;
+}
 #endif
 
