@@ -1,8 +1,8 @@
 /* Copyright 2004 Roger Dingledine */
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: torgzip.c,v 1.21 2005/08/29 18:01:38 nickm Exp $ */
-const char torgzip_c_id[] = "$Id: torgzip.c,v 1.21 2005/08/29 18:01:38 nickm Exp $";
+/* $Id: torgzip.c,v 1.22 2005/09/30 01:09:52 nickm Exp $ */
+const char torgzip_c_id[] = "$Id: torgzip.c,v 1.22 2005/09/30 01:09:52 nickm Exp $";
 
 /**
  * \file torgzip.c
@@ -253,7 +253,8 @@ tor_gzip_uncompress(char **out, size_t *out_len,
  * to be compressed or not.  If it is, return the likeliest compression method.
  * Otherwise, return 0.
  */
-int detect_compression_method(const char *in, size_t in_len)
+int
+detect_compression_method(const char *in, size_t in_len)
 {
   if (in_len > 2 && !memcmp(in, "\x1f\x8b", 2)) {
     return GZIP_METHOD;
