@@ -1,7 +1,7 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.h,v 1.139 2005/09/13 06:19:31 nickm Exp $ */
+/* $Id: util.h,v 1.143 2005/10/06 04:33:40 nickm Exp $ */
 
 /**
  * \file util.h
@@ -10,7 +10,7 @@
 
 #ifndef __UTIL_H
 #define __UTIL_H
-#define UTIL_H_ID "$Id: util.h,v 1.139 2005/09/13 06:19:31 nickm Exp $"
+#define UTIL_H_ID "$Id: util.h,v 1.143 2005/10/06 04:33:40 nickm Exp $"
 
 #include "orconfig.h"
 #include "torint.h"
@@ -47,24 +47,43 @@
  } } while (0)
 #endif
 
+#ifdef USE_DMALLOC
+#define DMALLOC_PARAMS , const char *file, const int line
+#define DMALLOC_ARGS , _SHORT_FILE_, __LINE__
+#else
+#define DMALLOC_PARAMS
+#define DMALLOC_ARGS
+#endif
+
 /** Define this if you want Tor to crash when any problem comes up,
  * so you can get a coredump and track things down. */
 // #define tor_fragile_assert() tor_assert(0)
 #define tor_fragile_assert()
 
 /* Memory management */
-void *_tor_malloc(const char *file, const int line, size_t size);
-void *_tor_malloc_zero(const char *file, const int line, size_t size);
-void *_tor_realloc(const char *file, const int line, void *ptr, size_t size);
-char *_tor_strdup(const char *file, const int line, const char *s);
-char *_tor_strndup(const char *file, const int line, const char *s, size_t n);
+void *_tor_malloc(size_t size DMALLOC_PARAMS);
+void *_tor_malloc_zero(size_t size DMALLOC_PARAMS);
+void *_tor_realloc(void *ptr, size_t size DMALLOC_PARAMS);
+char *_tor_strdup(const char *s DMALLOC_PARAMS);
+char *_tor_strndup(const char *s, size_t n DMALLOC_PARAMS);
+#ifdef USE_DMALLOC
+extern int dmalloc_free(const char *file, const int line, void *pnt,
+                        const int func_id);
+#define tor_free(p) do { \
+    if (p) {                                        \
+      dmalloc_free(_SHORT_FILE_, __LINE__, (p), 0); \
+      (p)=NULL;                                     \
+    }                                               \
+  } while (0)
+#else
 #define tor_free(p) do { if (p) {free(p); (p)=NULL;} } while (0)
+#endif
 
-#define tor_malloc(size)       _tor_malloc(_SHORT_FILE_, __LINE__, size)
-#define tor_malloc_zero(size)  _tor_malloc_zero(_SHORT_FILE_, __LINE__, size)
-#define tor_realloc(ptr, size) _tor_realloc(_SHORT_FILE_, __LINE__, ptr, size)
-#define tor_strdup(s)          _tor_strdup(_SHORT_FILE_, __LINE__, s)
-#define tor_strndup(s, n)      _tor_strndup(_SHORT_FILE_, __LINE__, s, n)
+#define tor_malloc(size)       _tor_malloc(size DMALLOC_ARGS)
+#define tor_malloc_zero(size)  _tor_malloc_zero(size DMALLOC_ARGS)
+#define tor_realloc(ptr, size) _tor_realloc(ptr, size DMALLOC_ARGS)
+#define tor_strdup(s)          _tor_strdup(s DMALLOC_ARGS)
+#define tor_strndup(s, n)      _tor_strndup(s, n DMALLOC_ARGS)
 
 /* String manipulation */
 #define HEX_CHARACTERS "0123456789ABCDEFabcdef"
@@ -121,6 +140,8 @@ int check_private_dir(const char *dirname, cpd_check_t check);
 int write_str_to_file(const char *fname, const char *str, int bin);
 int write_bytes_to_file(const char *fname, const char *str, size_t len,
                         int bin);
+/** An ad-hoc type to hold a string of characters and a count; used by
+ * write_chunks_to_file. */
 typedef struct sized_chunk_t {
   const char *bytes;
   size_t len;
