@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: buffers.c,v 1.169 2005/08/30 07:01:30 arma Exp $ */
-const char buffers_c_id[] = "$Id: buffers.c,v 1.169 2005/08/30 07:01:30 arma Exp $";
+/* $Id: buffers.c,v 1.173 2005/10/06 04:33:40 nickm Exp $ */
+const char buffers_c_id[] = "$Id: buffers.c,v 1.173 2005/10/06 04:33:40 nickm Exp $";
 
 /**
  * \file buffers.c
@@ -49,6 +49,7 @@ const char buffers_c_id[] = "$Id: buffers.c,v 1.169 2005/08/30 07:01:30 arma Exp
 #endif
 
 #define BUFFER_MAGIC 0xB0FFF312u
+/** A resizeable buffer, optimized for reading and writing. */
 struct buf_t {
   uint32_t magic; /**< Magic cookie for debugging: Must be set to BUFFER_MAGIC */
   char *mem;      /**< Storage for data in the buffer */
@@ -83,14 +84,15 @@ buf_normalize(buf_t *buf)
   if (buf->cur + buf->datalen <= buf->mem+buf->len) {
     return;
   } else {
-    char *newmem;
+    char *newmem, *oldmem;
     size_t sz = (buf->mem+buf->len)-buf->cur;
     log_fn(LOG_WARN, "Unexpected non-normalized buffer.");
     newmem = GUARDED_MEM(tor_malloc(ALLOC_LEN(buf->len)));
     SET_GUARDS(newmem, buf->len);
     memcpy(newmem, buf->cur, sz);
     memcpy(newmem+sz, buf->mem, buf->datalen-sz);
-    free(RAW_MEM(buf->mem));
+    oldmem = RAW_MEM(buf->mem);
+    tor_free(oldmem); /* Can't use tor_free directly. */
     buf->mem = buf->cur = newmem;
     check();
   }
@@ -196,11 +198,11 @@ buf_resize(buf_t *buf, size_t new_capacity)
       !buf->datalen &&
       buf->len >= 1<<16) {
     /* don't realloc; free and malloc */
-    char *newmem = GUARDED_MEM(tor_malloc(ALLOC_LEN(new_capacity)));
+    char *oldmem, *newmem = GUARDED_MEM(tor_malloc(ALLOC_LEN(new_capacity)));
     SET_GUARDS(newmem, new_capacity);
-    free(RAW_MEM(buf->mem));
+    oldmem = RAW_MEM(buf->mem);
+    tor_free(oldmem);
     buf->mem = buf->cur = newmem;
-
   } else {
     buf->mem = GUARDED_MEM(tor_realloc(RAW_MEM(buf->mem),
                                        ALLOC_LEN(new_capacity)));
@@ -289,7 +291,8 @@ buf_shrink(buf_t *buf)
 
 /** Remove the first <b>n</b> bytes from buf. */
 static INLINE void
-buf_remove_from_front(buf_t *buf, size_t n) {
+buf_remove_from_front(buf_t *buf, size_t n)
+{
   tor_assert(buf->datalen >= n);
   buf->datalen -= n;
   buf_total_used -= n;
@@ -313,7 +316,8 @@ buf_nul_terminate(buf_t *buf)
 
 /** Create and return a new buf with capacity <b>size</b>. */
 buf_t *
-buf_new_with_capacity(size_t size) {
+buf_new_with_capacity(size_t size)
+{
   buf_t *buf;
   buf = tor_malloc_zero(sizeof(buf_t));
   buf->magic = BUFFER_MAGIC;
@@ -369,9 +373,11 @@ _buf_peek_raw_buffer(const buf_t *buf)
 void
 buf_free(buf_t *buf)
 {
+  char *oldmem;
   assert_buf_ok(buf);
   buf->magic = 0xDEADBEEF;
-  free(RAW_MEM(buf->mem));
+  oldmem = RAW_MEM(buf->mem);
+  tor_free(oldmem);
   buf_total_alloc -= buf->len;
   buf_total_used -= buf->datalen;
   tor_free(buf);
@@ -468,7 +474,7 @@ read_to_buf(int s, size_t at_most, buf_t *buf, int *reached_eof)
  * -1 on failure.
  */
 static INLINE int
-read_to_buf_tls_impl(tor_tls *tls, size_t at_most, buf_t *buf, char *next)
+read_to_buf_tls_impl(tor_tls_t *tls, size_t at_most, buf_t *buf, char *next)
 {
   int r;
 
@@ -507,7 +513,7 @@ read_to_buf_tls_impl(tor_tls *tls, size_t at_most, buf_t *buf, char *next)
  * ready to write -- or vice versa.
  */
 int
-read_to_buf_tls(tor_tls *tls, size_t at_most, buf_t *buf)
+read_to_buf_tls(tor_tls_t *tls, size_t at_most, buf_t *buf)
 {
   int r;
   char *next;
@@ -629,7 +635,7 @@ flush_buf(int s, buf_t *buf, size_t *buf_flushlen)
  * Return the number of bytes written on success, -1 on failure.
  */
 static INLINE int
-flush_buf_tls_impl(tor_tls *tls, buf_t *buf, size_t sz, size_t *buf_flushlen)
+flush_buf_tls_impl(tor_tls_t *tls, buf_t *buf, size_t sz, size_t *buf_flushlen)
 {
   int r;
 
@@ -646,7 +652,8 @@ flush_buf_tls_impl(tor_tls *tls, buf_t *buf, size_t sz, size_t *buf_flushlen)
 
 /** As flush_buf(), but writes data to a TLS connection.
  */
-int flush_buf_tls(tor_tls *tls, buf_t *buf, size_t *buf_flushlen)
+int
+flush_buf_tls(tor_tls_t *tls, buf_t *buf, size_t *buf_flushlen)
 {
   int r;
   size_t flushed=0;
@@ -949,7 +956,8 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           strlcpy(req->address,tmpbuf,sizeof(req->address));
           req->port = ntohs(*(uint16_t*)(buf->cur+8));
           buf_remove_from_front(buf, 10);
-          if (!have_warned_about_unsafe_socks) {
+          if (!address_is_in_virtual_range(req->address) &&
+              !have_warned_about_unsafe_socks) {
             log_fn(LOG_WARN,"Your application (using socks5 on port %d) is giving Tor only an IP address. Applications that do DNS resolves themselves may leak information. Consider using Socks4A (e.g. via privoxy or socat) instead.  For more information, please see http://wiki.noreply.org/noreply/TheOnionRouter/TorFAQ#SOCKSAndDNS", req->port);
 //            have_warned_about_unsafe_socks = 1; // (for now, warn every time)
           }
@@ -1019,7 +1027,9 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
       tor_assert(next < buf->cur+buf->datalen);
 
       startaddr = NULL;
-      if (socks4_prot != socks4a && !have_warned_about_unsafe_socks) {
+      if (socks4_prot != socks4a &&
+          !address_is_in_virtual_range(tmpbuf) &&
+          !have_warned_about_unsafe_socks) {
         log_fn(LOG_WARN,"Your application (using socks4 on port %d) is giving Tor only an IP address. Applications that do DNS resolves themselves may leak information. Consider using Socks4A (e.g. via privoxy or socat) instead.", req->port);
 //      have_warned_about_unsafe_socks = 1; // (for now, warn every time)
       }
@@ -1195,7 +1205,8 @@ fetch_from_buf_line(buf_t *buf, char *data_out, size_t *data_len)
 
 /** Log an error and exit if <b>buf</b> is corrupted.
  */
-void assert_buf_ok(buf_t *buf)
+void
+assert_buf_ok(buf_t *buf)
 {
   tor_assert(buf);
   tor_assert(buf->magic == BUFFER_MAGIC);
