@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.142 2005/09/13 21:24:51 arma Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.142 2005/09/13 21:24:51 arma Exp $";
+/* $Id: circuitbuild.c,v 1.148 2005/10/04 22:23:31 nickm Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.148 2005/10/04 22:23:31 nickm Exp $";
 
 /**
  * \file circuitbuild.c
@@ -461,7 +461,7 @@ circuit_deliver_create_cell(circuit_t *circ, uint8_t cell_type, char *payload)
 /** We've decided to start our reachability testing. If all
  * is set, log this to the user. Return 1 if we did, or 0 if
  * we chose not to log anything. */
-int
+static int
 inform_testing_reachability(void)
 {
   char dirbuf[128];
@@ -979,6 +979,8 @@ circuit_all_predicted_ports_handled(time_t now, int *need_uptime,
   uint16_t *port;
   smartlist_t *sl = circuit_get_unhandled_ports(now);
   smartlist_t *LongLivedServices = get_options()->LongLivedPorts;
+  tor_assert(need_uptime);
+  tor_assert(need_capacity);
   enough = (smartlist_len(sl) == 0);
   for (i = 0; i < smartlist_len(sl); ++i) {
     port = smartlist_get(sl, i);
@@ -1051,7 +1053,7 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
   or_options_t *options = get_options();
 
   preferredentries = smartlist_create();
-  add_nickname_list_to_smartlist(preferredentries,options->EntryNodes,1);
+  add_nickname_list_to_smartlist(preferredentries,options->EntryNodes,1,1);
 
   get_connection_array(&carray, &n_connections);
 
@@ -1140,10 +1142,10 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
          n_best_support, best_support, n_pending_connections);
 
   preferredexits = smartlist_create();
-  add_nickname_list_to_smartlist(preferredexits,options->ExitNodes,1);
+  add_nickname_list_to_smartlist(preferredexits,options->ExitNodes,1,1);
 
   excludedexits = smartlist_create();
-  add_nickname_list_to_smartlist(excludedexits,options->ExcludeNodes,0);
+  add_nickname_list_to_smartlist(excludedexits,options->ExcludeNodes,0,1);
 
   sl = smartlist_create();
 
@@ -1380,6 +1382,8 @@ choose_good_middle_server(uint8_t purpose,
   routerinfo_t *r, *choice;
   crypt_path_t *cpath;
   smartlist_t *excluded;
+  tor_assert(_CIRCUIT_PURPOSE_MIN <= purpose &&
+             purpose <= _CIRCUIT_PURPOSE_MAX);
 
   log_fn(LOG_DEBUG, "Contemplating intermediate hop: random choice.");
   excluded = smartlist_create();
@@ -1502,7 +1506,7 @@ onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
          state->desired_path_len);
 
   excludednodes = smartlist_create();
-  add_nickname_list_to_smartlist(excludednodes,get_options()->ExcludeNodes,0);
+  add_nickname_list_to_smartlist(excludednodes,get_options()->ExcludeNodes,0,1);
 
   if (cur_len == state->desired_path_len - 1) { /* Picking last node */
     info = extend_info_dup(state->chosen_exit);
@@ -1659,7 +1663,7 @@ pick_helper_nodes(void)
     helper_nodes_changed();
 }
 
-/** Remove all elements from the list of helper nodes */
+/** Remove all elements from the list of helper nodes. */
 static void
 clear_helper_nodes(void)
 {
@@ -1668,6 +1672,16 @@ clear_helper_nodes(void)
   helper_nodes_changed();
 }
 
+/** Release all storage held by the list of helper nodes. */
+void
+helper_nodes_free_all(void)
+{
+  /* Don't call clear_helper_nodes(); that will flush our state change to disk */
+  SMARTLIST_FOREACH(helper_nodes, helper_node_t *, h, tor_free(h));
+  smartlist_free(helper_nodes);
+  helper_nodes = NULL;
+}
+
 /** How long (in seconds) do we allow a helper node to be nonfunctional before
  * we give up on it? */
 #define HELPER_ALLOW_DOWNTIME 48*60*60
@@ -1740,15 +1754,14 @@ helper_nodes_set_status_from_directory(void)
       routerinfo_t *r = router_get_by_digest(helper->identity);
       if (! r) {
         if (! helper->unlisted_since) {
-        /* Watch out for skew here. XXXX */
-          helper->unlisted_since = routers->published_on;
+          helper->unlisted_since = time(NULL);
           ++changed;
-          log_fn(LOG_WARN,"Helper node '%s' is not published in latest directory",
+          log_fn(LOG_WARN,"Helper node '%s' is not listed by directories",
                  helper->nickname);
         }
       } else {
         if (helper->unlisted_since) {
-          log_fn(LOG_WARN,"Helper node '%s' is listed again in latest directory",
+          log_fn(LOG_WARN,"Helper node '%s' is listed again by directories",
                  helper->nickname);
           ++changed;
         }
@@ -1761,7 +1774,7 @@ helper_nodes_set_status_from_directory(void)
           }
         } else {
           if (helper->down_since) {
-            log_fn(LOG_WARN,"Helper node '%s' is up in latest directory",
+            log_fn(LOG_WARN,"Helper node '%s' is up in latest directories",
                    helper->nickname);
             ++changed;
           }
@@ -1896,6 +1909,7 @@ helper_nodes_parse_state(or_state_t *state, int set, const char **err)
   if (*err || !set) {
     SMARTLIST_FOREACH(helpers, helper_node_t *, h, tor_free(h));
     smartlist_free(helpers);
+    helpers = NULL;
   }
   if (!*err && set) {
     if (helper_nodes) {
