@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.56 2005/07/22 21:12:10 nickm Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.56 2005/07/22 21:12:10 nickm Exp $";
+/* $Id: circuitlist.c,v 1.60 2005/10/06 22:18:01 nickm Exp $ */
+const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.60 2005/10/06 22:18:01 nickm Exp $";
 
 /**
  * \file circuitlist.c
@@ -11,6 +11,10 @@ const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.56 2005/07/22 21:12:10 n
  **/
 
 #include "or.h"
+
+/* Define RB_AUGMENT to avoid warnings about if statements with emtpy bodies.
+ */
+#define RB_AUGMENT(x) do{}while(0)
 #include "tree.h"
 
 /********* START VARIABLES **********/
@@ -174,7 +178,8 @@ _circuit_get_global_list(void)
 
 /** Function to make circ-\>state human-readable */
 const char *
-circuit_state_to_string(int state) {
+circuit_state_to_string(int state)
+{
   static char buf[64];
   switch (state) {
     case CIRCUIT_STATE_BUILDING: return "doing handshakes";
@@ -214,7 +219,7 @@ circuit_new(uint16_t p_circ_id, connection_t *p_conn)
   circ->package_window = CIRCWINDOW_START;
   circ->deliver_window = CIRCWINDOW_START;
 
-  circ->next_stream_id = crypto_pseudo_rand_int(1<<16);
+  circ->next_stream_id = crypto_rand_int(1<<16);
   circ->global_identifier = n_circuits_allocated++;
 
   circuit_add(circ);
@@ -562,6 +567,27 @@ circuit_mark_all_unused_circs(void)
   }
 }
 
+/** Go through the circuitlist; for each circuit that starts at us
+ * and is dirty, frob its timestamp_dirty so we won't use it for any
+ * new streams.
+ *
+ * This is useful for letting the user change pseudonyms, so new
+ * streams will not be linkable to old streams.
+ */
+void
+circuit_expire_all_dirty_circs(void)
+{
+  circuit_t *circ;
+  or_options_t *options = get_options();
+
+  for (circ=global_circuitlist; circ; circ = circ->next) {
+    if (CIRCUIT_IS_ORIGIN(circ) &&
+        !circ->marked_for_close &&
+        circ->timestamp_dirty)
+      circ->timestamp_dirty -= options->MaxCircuitDirtiness;
+  }
+}
+
 /** Mark <b>circ</b> to be closed next time we call
  * circuit_close_all_marked(). Do any cleanup needed:
  *   - If state is onionskin_pending, remove circ from the onion_pending
