@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuituse.c,v 1.80 2005/08/15 03:25:39 arma Exp $ */
-const char circuituse_c_id[] = "$Id: circuituse.c,v 1.80 2005/08/15 03:25:39 arma Exp $";
+/* $Id: circuituse.c,v 1.84 2005/10/05 05:03:52 nickm Exp $ */
+const char circuituse_c_id[] = "$Id: circuituse.c,v 1.84 2005/10/05 05:03:52 nickm Exp $";
 
 /**
  * \file circuituse.c
@@ -585,7 +585,8 @@ circuit_testing_opened(circuit_t *circ)
 
 /** A testing circuit has failed to build. Take whatever stats we want. */
 static void
-circuit_testing_failed(circuit_t *circ, int at_last_hop) {
+circuit_testing_failed(circuit_t *circ, int at_last_hop)
+{
 #if 0
   routerinfo_t *me = router_get_my_routerinfo();
 
@@ -805,7 +806,7 @@ circuit_launch_by_nickname(uint8_t purpose, const char *exit_nickname,
   routerinfo_t *router = NULL;
 
   if (exit_nickname) {
-    router = router_get_by_nickname(exit_nickname);
+    router = router_get_by_nickname(exit_nickname, 1);
     if (!router) {
       log_fn(LOG_WARN, "No such OR as '%s'", exit_nickname);
       return NULL;
@@ -869,7 +870,12 @@ circuit_get_open_circ_or_launch(connection_t *conn,
   if (!has_fetched_directory) {
     if (!connection_get_by_type(CONN_TYPE_DIR)) {
       log(LOG_NOTICE,"Application request when we're believed to be offline. Optimistically trying again.");
-      directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
+      router_reset_status_download_failures();
+      router_reset_descriptor_download_failures();
+      update_networkstatus_downloads(time(NULL));
+
+      /* XXXX011 NM This should be a generic "retry all directory fetches". */
+      directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1); /*XXXX011NM*/
     }
     /* the stream will be dealt with when has_fetched_directory becomes
      * 1, or when all directory attempts fail and directory_all_unreachable()
@@ -922,7 +928,7 @@ circuit_get_open_circ_or_launch(connection_t *conn,
     if (desired_circuit_purpose == CIRCUIT_PURPOSE_C_GENERAL) {
       if (conn->chosen_exit_name) {
         routerinfo_t *r;
-        if (!(r = router_get_by_nickname(conn->chosen_exit_name))) {
+        if (!(r = router_get_by_nickname(conn->chosen_exit_name, 1))) {
           log_fn(LOG_NOTICE,"Requested exit point '%s' is not known. Closing.",
                  conn->chosen_exit_name);
           return -1;
@@ -1096,7 +1102,7 @@ connection_ap_handshake_attach_circuit(connection_t *conn)
     circuit_t *circ=NULL;
 
     if (conn->chosen_exit_name) {
-      routerinfo_t *router = router_get_by_nickname(conn->chosen_exit_name);
+      routerinfo_t *router = router_get_by_nickname(conn->chosen_exit_name, 1);
       if (!router) {
         log_fn(LOG_WARN,"Requested exit point '%s' is not known. Closing.",
                conn->chosen_exit_name);
