@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.399 2005/09/14 02:36:29 nickm Exp $ */
-const char connection_c_id[] = "$Id: connection.c,v 1.399 2005/09/14 02:36:29 nickm Exp $";
+/* $Id: connection.c,v 1.408 2005/10/07 19:49:48 arma Exp $ */
+const char connection_c_id[] = "$Id: connection.c,v 1.408 2005/10/07 19:49:48 arma Exp $";
 
 /**
  * \file connection.c
@@ -176,7 +176,7 @@ connection_new(int type)
     conn->socks_request = tor_malloc_zero(sizeof(socks_request_t));
   }
 
-  conn->next_circ_id = crypto_pseudo_rand_int(1<<15);
+  conn->next_circ_id = crypto_rand_int(1<<15);
 
   conn->timestamp_created = now;
   conn->timestamp_lastread = now;
@@ -522,13 +522,13 @@ connection_create_listener(const char *bindaddress, uint16_t bindport,
 #endif
 
   if (bind(s,(struct sockaddr *)&bindaddr,sizeof(bindaddr)) < 0) {
-    log_fn(LOG_WARN, "Could not bind to port %u: %s", usePort,
+    log_fn(LOG_WARN, "Could not bind to %s:%u: %s", address, usePort,
            tor_socket_strerror(tor_socket_errno(s)));
     goto err;
   }
 
   if (listen(s,SOMAXCONN) < 0) {
-    log_fn(LOG_WARN, "Could not listen on port %u: %s", usePort,
+    log_fn(LOG_WARN, "Could not listen on %s:%u: %s", address, usePort,
            tor_socket_strerror(tor_socket_errno(s)));
     goto err;
   }
@@ -858,12 +858,14 @@ retry_listeners(int type, config_line_t *cfg,
     line = NULL;
     SMARTLIST_FOREACH(launch, config_line_t *, wanted,
       {
-        char *addr;
+        char *address=NULL;
         uint16_t port;
-        if (! parse_addr_port(wanted->value, &addr, NULL, &port)) {
+        if (! parse_addr_port(wanted->value, &address, NULL, &port)) {
+          int addr_matches = !strcasecmp(address, conn->address);
+          tor_free(address);
           if (! port)
             port = port_option;
-          if (port == conn->port && !strcasecmp(addr, conn->address)) {
+          if (port == conn->port && addr_matches) {
             line = wanted;
             break;
           }
@@ -881,9 +883,11 @@ retry_listeners(int type, config_line_t *cfg,
       }
     } else {
       /* It's configured; we don't need to launch it. */
-      log_fn(LOG_INFO, "Already have %s on %s:%d",
+      log_fn(LOG_DEBUG, "Already have %s on %s:%d",
              conn_type_to_string(type), conn->address, conn->port);
       smartlist_remove(launch, line);
+      if (free_launch_elts)
+        config_free_lines(line);
     }
   }
 
@@ -1637,7 +1641,8 @@ connection_get_by_type_state_lastwritten(int type, int state)
  * is non-zero, conn must be of that state too.
  */
 connection_t *
-connection_get_by_type_state_rendquery(int type, int state, const char *rendquery) {
+connection_get_by_type_state_rendquery(int type, int state, const char *rendquery)
+{
   int i, n;
   connection_t *conn;
   connection_t **carray;
@@ -2000,8 +2005,10 @@ assert_connection_ok(connection_t *conn, time_t now)
     tor_assert(!conn->cpath_layer);
     tor_assert(!conn->package_window);
     tor_assert(!conn->deliver_window);
+#if 0
     tor_assert(!conn->done_sending);
     tor_assert(!conn->done_receiving);
+#endif
   } else {
     /* XXX unchecked: package window, deliver window. */
   }
