@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c,v 1.348 2005/09/12 22:05:17 arma Exp $ */
-const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.348 2005/09/12 22:05:17 arma Exp $";
+/* $Id: connection_edge.c,v 1.354 2005/10/06 04:33:40 nickm Exp $ */
+const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.354 2005/10/06 04:33:40 nickm Exp $";
 
 /**
  * \file connection_edge.c
@@ -18,7 +18,6 @@ static addr_policy_t *socks_policy = NULL;
 static smartlist_t *redirect_exit_list = NULL;
 
 static int connection_ap_handshake_process_socks(connection_t *conn);
-static int address_is_in_virtual_range(const char *addr);
 
 /** An AP stream has failed/finished. If it hasn't already sent back
  * a socks reply, send one now (based on endreason). Also set
@@ -452,6 +451,7 @@ typedef struct {
   int num_resolve_failures;
 } addressmap_entry_t;
 
+/** Entry for mapping addresses to which virtual address we mapped them to. */
 typedef struct {
   char *ipv4_address;
   char *hostname_address;
@@ -500,16 +500,16 @@ addressmap_virtaddress_ent_free(void *_ent)
 
 /** Free storage held by a virtaddress_entry_t* entry in <b>ent</b> */
 static void
-addressmap_virtaddress_remove(const char *addr, addressmap_entry_t *ent)
+addressmap_virtaddress_remove(const char *address, addressmap_entry_t *ent)
 {
   if (ent && ent->new_address && address_is_in_virtual_range(ent->new_address)) {
     virtaddress_entry_t *ve =
       strmap_get(virtaddress_reversemap, ent->new_address);
     /*log_fn(LOG_NOTICE,"remove reverse mapping for %s",ent->new_address);*/
     if (ve) {
-      if (!strcmp(addr, ve->ipv4_address))
+      if (!strcmp(address, ve->ipv4_address))
         tor_free(ve->ipv4_address);
-      if (!strcmp(addr, ve->hostname_address))
+      if (!strcmp(address, ve->hostname_address))
         tor_free(ve->hostname_address);
       if (!ve->ipv4_address && !ve->hostname_address) {
         tor_free(ve);
@@ -521,9 +521,9 @@ addressmap_virtaddress_remove(const char *addr, addressmap_entry_t *ent)
 
 /* DOCDOC */
 static void
-addressmap_ent_remove(const char *addr, addressmap_entry_t *ent)
+addressmap_ent_remove(const char *address, addressmap_entry_t *ent)
 {
-  addressmap_virtaddress_remove(addr, ent);
+  addressmap_virtaddress_remove(address, ent);
   addressmap_ent_free(ent);
 }
 
@@ -735,16 +735,16 @@ client_dns_set_addressmap(const char *address, uint32_t val, const char *exitnam
  * Return true iff <b>addr</b> is likely to have been returned by
  * client_dns_get_unused_address.
  **/
-static int
-address_is_in_virtual_range(const char *addr)
+int
+address_is_in_virtual_range(const char *address)
 {
   struct in_addr in;
-  tor_assert(addr);
-  if (!strcasecmpend(addr, ".virtual")) {
+  tor_assert(address);
+  if (!strcasecmpend(address, ".virtual")) {
     return 1;
-  } else if (tor_inet_aton(addr, &in)) {
-    uint32_t a = ntohl(in.s_addr);
-    if (a >= MIN_UNUSED_IPV4 && a <= MAX_UNUSED_IPV4)
+  } else if (tor_inet_aton(address, &in)) {
+    uint32_t addr = ntohl(in.s_addr);
+    if (addr >= MIN_UNUSED_IPV4 && addr <= MAX_UNUSED_IPV4)
       return 1;
   }
   return 0;
@@ -952,6 +952,8 @@ connection_ap_handshake_process_socks(connection_t *conn)
   } /* else socks handshake is done, continue processing */
 
   tor_strlower(socks->address); /* normalize it */
+  log_fn(LOG_DEBUG,"Client asked for %s:%d", safe_str(socks->address),
+         socks->port);
 
   /* For address map controls, remap the address */
   addressmap_rewrite(socks->address, sizeof(socks->address));
@@ -995,7 +997,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
       }
     } else {
       struct in_addr in;
-      routerinfo_t *r = router_get_by_nickname(socks->address);
+      routerinfo_t *r = router_get_by_nickname(socks->address, 1);
       if (r) {
         conn->chosen_exit_name = tor_strdup(socks->address);
         /* XXXX Should this use server->address instead? */
@@ -1692,15 +1694,17 @@ connection_ap_can_use_exit(connection_t *conn, routerinfo_t *exit)
   tor_assert(conn->socks_request);
   tor_assert(exit);
 
+#if 0
   log_fn(LOG_DEBUG,"considering nickname %s, for address %s / port %d:",
          exit->nickname, safe_str(conn->socks_request->address),
          conn->socks_request->port);
+#endif
 
   /* If a particular exit node has been requested for the new connection,
    * make sure the exit node of the existing circuit matches exactly.
    */
   if (conn->chosen_exit_name) {
-    if (router_get_by_nickname(conn->chosen_exit_name) != exit) {
+    if (router_get_by_nickname(conn->chosen_exit_name, 1) != exit) {
       /* doesn't match */
       log_fn(LOG_DEBUG,"Requested node '%s', considering node '%s'. No.",
              conn->chosen_exit_name, exit->nickname);
@@ -1725,7 +1729,7 @@ connection_ap_can_use_exit(connection_t *conn, routerinfo_t *exit)
 /** A helper function for socks_policy_permits_address() below.
  *
  * Parse options->SocksPolicy in the same way that the exit policy
- * is parsed, and put the processed version in &socks_policy.
+ * is parsed, and put the processed version in socks_policy.
  * Ignore port specifiers.
  */
 void
@@ -1785,10 +1789,10 @@ set_exit_redirects(smartlist_t *lst)
 }
 
 /** If address is of the form "y.onion" with a well-formed handle y:
- *     Put a \code{'\0'} after y, lower-case it, and return ONION_HOSTNAME.
+ *     Put a NUL after y, lower-case it, and return ONION_HOSTNAME.
  *
  * If address is of the form "y.exit":
- *     Put a \code{'\0'} after y and return EXIT_HOSTNAME.
+ *     Put a NUL after y and return EXIT_HOSTNAME.
  *
  * Otherwise:
  *     Return NORMAL_HOSTNAME and change nothing.
