@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.275 2005/09/14 03:49:17 arma Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.275 2005/09/14 03:49:17 arma Exp $";
+/* $Id: directory.c,v 1.303 2005/10/05 20:45:18 nickm Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.303 2005/10/05 20:45:18 nickm Exp $";
 
 #include "or.h"
 
@@ -29,7 +29,6 @@ const char directory_c_id[] = "$Id: directory.c,v 1.275 2005/09/14 03:49:17 arma
  * - connection_dir_finished_connecting(), called from
  *   connection_finished_connecting() in connection.c
  */
-
 static void
 directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
                                       uint8_t purpose, int private_connection,
@@ -52,7 +51,9 @@ static int purpose_is_private(uint8_t purpose);
 static char *http_get_header(const char *headers, const char *which);
 static char *http_get_origin(const char *headers, connection_t *conn);
 static void connection_dir_download_networkstatus_failed(connection_t *conn);
+static void connection_dir_download_routerdesc_failed(connection_t *conn);
 static void dir_networkstatus_download_failed(smartlist_t *failed);
+static void dir_routerdesc_download_failed(smartlist_t *failed);
 
 /********* START VARIABLES **********/
 
@@ -147,7 +148,8 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
         if (!fascist_firewall_allows_address(ds->addr,ds->dir_port))
           continue;
       }
-      directory_initiate_command_trusted_dir(ds, purpose, purpose_is_private(purpose),
+      directory_initiate_command_trusted_dir(ds, purpose,
+                                             purpose_is_private(purpose),
                                              NULL, payload, payload_len);
     });
 }
@@ -215,9 +217,11 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
   }
 
   if (r)
-    directory_initiate_command_router(r, purpose, !directconn, resource, NULL, 0);
+    directory_initiate_command_router(r, purpose, !directconn,
+                                      resource, NULL, 0);
   else if (ds)
-    directory_initiate_command_trusted_dir(ds, purpose, !directconn, resource, NULL, 0);
+    directory_initiate_command_trusted_dir(ds, purpose, !directconn,
+                                           resource, NULL, 0);
   else {
     log_fn(LOG_NOTICE,"No running dirservers known. Will try again later. (purpose %d)",
            purpose);
@@ -258,8 +262,9 @@ directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
                                        const char *resource,
                                        const char *payload, size_t payload_len)
 {
-  directory_initiate_command(dirserv->address, dirserv->addr,dirserv->dir_port,
-               NULL, dirserv->digest, purpose, private_connection, resource,
+  directory_initiate_command(dirserv->address, dirserv->addr,
+               dirserv->dir_port, NULL, dirserv->digest, purpose,
+               private_connection, resource,
                payload, payload_len);
 }
 
@@ -282,15 +287,26 @@ connection_dir_request_failed(connection_t *conn)
     log_fn(LOG_INFO, "Giving up on directory server at '%s'; retrying",
            conn->address);
     connection_dir_download_networkstatus_failed(conn);
+  } else if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC) {
+    log_fn(LOG_INFO, "Giving up on directory server at '%s'; retrying",
+           conn->address);
+    connection_dir_download_routerdesc_failed(conn);
   }
 }
 
-/** Called when an attempt to download one or network status documents
- * on connection <b>conn</b> failed.
+/** Called when an attempt to download one or more network status
+ * documents on connection <b>conn</b> failed. Decide whether to
+ * retry the fetch now, later, or never.
  */
 static void
 connection_dir_download_networkstatus_failed(connection_t *conn)
 {
+  if (!conn->requested_resource) {
+    /* We never reached directory_send_command, which means that we never
+     * opened a network connection.  Either we're out of sockets, or the
+     * network is down.  Either way, retrying would be pointless. */
+    return;
+  }
   if (!strcmpstart(conn->requested_resource, "all")) {
     /* We're a non-authoritative directory cache; try again. */
     directory_get_from_dirserver(conn->purpose, "all.z",
@@ -299,14 +315,9 @@ connection_dir_download_networkstatus_failed(connection_t *conn)
     /* We were trying to download by fingerprint; mark them all has having
      * failed, and possibly retry them later.*/
     smartlist_t *failed = smartlist_create();
-    /* XXXX NM this splitting logic is duplicated someplace. Fix that. */
-    smartlist_split_string(failed, conn->requested_resource+3, "+", 0, 0);
+    dir_split_resource_into_fingerprints(conn->requested_resource+3,
+                                         failed, NULL);
     if (smartlist_len(failed)) {
-      char *last = smartlist_get(failed,smartlist_len(failed)-1);
-      size_t last_len = strlen(last);
-      if (!strcmp(last+last_len-2, ".z"))
-        last[last_len-2] = '\0';
-
       dir_networkstatus_download_failed(failed);
       SMARTLIST_FOREACH(failed, char *, cp, tor_free(cp));
     }
@@ -314,6 +325,17 @@ connection_dir_download_networkstatus_failed(connection_t *conn)
   }
 }
 
+/** Called when an attempt to download one or more router descriptors
+ * on connection <b>conn</b> failed.
+ */
+static void
+connection_dir_download_routerdesc_failed(connection_t *conn)
+{
+  /* Try again. No need to increment the failure count for routerdescs, since
+   * it's not their fault.*/
+  /* update_router_descriptor_downloads(time(NULL)); */
+}
+
 /** Helper for directory_initiate_command_(router|trusted_dir): send the
  * command to a server whose address is <b>address</b>, whose IP is
  * <b>addr</b>, whose directory port is <b>dir_port</b>, whose tor version is
@@ -892,19 +914,12 @@ connection_dir_client_reached_eof(connection_t *conn)
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
-    if (router_load_routerlist_from_directory(body, NULL, !skewed, 0) < 0) {
+    if (router_parse_directory(body) < 0) {
       log_fn(LOG_NOTICE,"I failed to parse the directory I fetched from '%s:%d'. Ignoring.", conn->address, conn->port);
-      tor_free(body); tor_free(headers); tor_free(reason);
-      return -1;
     }
-    log_fn(LOG_INFO,"updated routers.");
-    /* do things we've been waiting to do */
-    directory_has_arrived(time(NULL), conn->identity_digest);
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
-    running_routers_t *rrs;
-    routerlist_t *rl;
     /* just update our list of running routers, if this list is new info */
     log_fn(LOG_INFO,"Received running-routers list (size %d)", (int)body_len);
     if (status_code != 200) {
@@ -913,19 +928,12 @@ connection_dir_client_reached_eof(connection_t *conn)
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
-    if (!(rrs = router_parse_runningrouters(body, 1))) {
-      log_fn(LOG_WARN, "Can't parse runningrouters list (server '%s:%d')",
+    if (router_parse_runningrouters(body)<0) {
+      log_fn(LOG_WARN,"Bad running-routers from server '%s:%d'. I'll try again soon.",
              conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
-    router_get_routerlist(&rl);
-    if (rl) {
-      routerlist_set_runningrouters(rl,rrs);
-      helper_nodes_set_status_from_directory();
-    } else {
-      running_routers_free(rrs);
-    }
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS) {
@@ -942,18 +950,16 @@ connection_dir_client_reached_eof(connection_t *conn)
     }
     if (conn->requested_resource &&
         !strcmpstart(conn->requested_resource,"fp/")) {
-      int n;
       which = smartlist_create();
-      smartlist_split_string(which, conn->requested_resource+3, "+", 0, -1);
-      n = smartlist_len(which);
-      if (n && strlen(smartlist_get(which,n-1))==HEX_DIGEST_LEN+2)
-        ((char*)smartlist_get(which,n-1))[HEX_DIGEST_LEN] = '\0';
+      dir_split_resource_into_fingerprints(conn->requested_resource+3,
+                                           which, NULL);
     }
     cp = body;
     while (*cp) {
       char *next = strstr(cp, "\nnetwork-status-version");
       if (next)
         next[1] = '\0';
+      /* learn from it, and then remove it from 'which' */
       if (router_set_networkstatus(cp, time(NULL), NS_FROM_DIR, which)<0)
         break;
       if (next) {
@@ -963,6 +969,8 @@ connection_dir_client_reached_eof(connection_t *conn)
       else
         break;
     }
+    routers_update_all_from_networkstatus(); /*launches router downloads*/
+    directory_info_has_arrived(time(NULL), 0);
     if (which) {
       if (smartlist_len(which)) {
         dir_networkstatus_download_failed(which);
@@ -973,8 +981,67 @@ connection_dir_client_reached_eof(connection_t *conn)
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC) {
-    /* XXXX NM implement this. */
-    log_fn(LOG_WARN, "Somehow, we requested some individual server descriptors. Skipping.");
+    smartlist_t *which = NULL;
+    int n_asked_for = 0;
+    log_fn(LOG_INFO,"Received server info (size %d) from server '%s:%d'",
+           (int)body_len, conn->address, conn->port);
+    if (conn->requested_resource &&
+        !strcmpstart(conn->requested_resource,"fp/")) {
+      which = smartlist_create();
+      dir_split_resource_into_fingerprints(conn->requested_resource+3,
+                                           which, NULL);
+      n_asked_for = smartlist_len(which);
+    }
+    if (status_code != 200) {
+      int no_warn = status_code == 404 ||
+        (status_code == 400 && !strcmp(reason, "Servers unavailable."));
+      /* 404 means that it didn't have them; no big deal.
+       * Older (pre-0.1.1.8) servers said 400 Servers unavailable instead. */
+      log_fn(no_warn ? LOG_INFO : LOG_WARN,
+             "Received http status code %d (\"%s\") from server '%s:%d' while fetching \"/tor/server/%s\". I'll try again soon.",
+             status_code, reason, conn->address, conn->port,
+             conn->requested_resource);
+      if (!which) {
+        connection_dir_download_routerdesc_failed(conn);
+      } else {
+        dir_routerdesc_download_failed(which);
+        SMARTLIST_FOREACH(which, char *, cp, tor_free(cp));
+        smartlist_free(which);
+      }
+      tor_free(body); tor_free(headers); tor_free(reason);
+      return -1;
+    }
+    /* Learn the routers, assuming we requested by fingerprint or "all".
+     * Right now, we only use "authority" to fetch ourself, so we don't want
+     * to risk replacing ourself with a router running at the addr:port we
+     * think we have.
+     */
+    if (which || (conn->requested_resource &&
+                  !strcmpstart(conn->requested_resource, "all"))) {
+      /* as we learn from them, we remove them from 'which' */
+      router_load_routers_from_string(body, 0, which);
+      directory_info_has_arrived(time(NULL), 0);
+    }
+    if (which) { /* mark remaining ones as failed */
+      log_fn(LOG_INFO, "Received %d/%d routers requested from %s:%d",
+             n_asked_for-smartlist_len(which), n_asked_for,
+             conn->address, (int)conn->port);
+      if (smartlist_len(which)) {
+        dir_routerdesc_download_failed(which);
+      }
+      SMARTLIST_FOREACH(which, char *, cp, tor_free(cp));
+      smartlist_free(which);
+    }
+    if (conn->requested_resource &&
+        !strcmpstart(conn->requested_resource,"authority")) {
+      /* this might have been a dirport reachability test. see if it is. */
+      routerinfo_t *me = router_get_my_routerinfo();
+      if (me &&
+          !memcmp(me->identity_digest, conn->identity_digest, DIGEST_LEN) &&
+          me->addr == conn->addr &&
+          me->dir_port == conn->port)
+        router_dirport_found_reachable();
+    }
   }
 
   if (conn->purpose == DIR_PURPOSE_UPLOAD_DIR) {
@@ -1048,6 +1115,13 @@ connection_dir_reached_eof(connection_t *conn)
   int retval;
   if (conn->state != DIR_CONN_STATE_CLIENT_READING) {
     log_fn(LOG_INFO,"conn reached eof, not reading. Closing.");
+    /* This check is temporary; it's to let us know whether we should consider
+     * parsing partial serverdesc responses. */
+    if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
+        buf_datalen(conn->inbuf)>=(24*1024)) {
+      log_fn(LOG_NOTICE, "Directory connection closed early after downloading %d bytes of descriptors.  If this happens often, please file a bug report.",
+             (int)buf_datalen(conn->inbuf));
+    }
     connection_close_immediate(conn); /* it was an error; give up on flushing */
     connection_mark_for_close(conn);
     return -1;
@@ -1063,7 +1137,8 @@ connection_dir_reached_eof(connection_t *conn)
 /** Read handler for directory connections.  (That's connections <em>to</em>
  * directory servers and connections <em>at</em> directory servers.)
  */
-int connection_dir_process_inbuf(connection_t *conn)
+int
+connection_dir_process_inbuf(connection_t *conn)
 {
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_DIR);
@@ -1171,9 +1246,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
     log_fn(LOG_DEBUG,"Dumping %sdirectory to client.",
            deflated?"deflated ":"");
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\nContent-Encoding: %s\r\n\r\n",
+    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
              date,
              (int)dlen,
+             deflated?"application/octet-stream":"text/plain",
              deflated?"deflate":"identity");
     connection_write_to_buf(tmp, strlen(tmp), conn);
     connection_write_to_buf(cp, dlen, conn);
@@ -1194,9 +1270,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
     }
 
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\nContent-Encoding: %s\r\n\r\n",
+    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
                  date,
                  (int)dlen,
+                 deflated?"application/octet-stream":"text/plain",
                  deflated?"deflate":"identity");
     connection_write_to_buf(tmp, strlen(tmp), conn);
     connection_write_to_buf(cp, strlen(cp), conn);
@@ -1225,9 +1302,10 @@ directory_handle_command_get(connection_t *conn, char *headers,
     SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
                       dlen += deflated?d->dir_z_len:d->dir_len);
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\nContent-Encoding: %s\r\n\r\n",
+    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
                  date,
                  (int)dlen,
+                 deflated?"application/octet-stream":"text/plain",
                  deflated?"deflate":"identity");
     connection_write_to_buf(tmp, strlen(tmp), conn);
     SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
@@ -1250,7 +1328,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
     dirserv_get_routerdescs(descs, url);
     tor_free(url);
     if (!smartlist_len(descs)) {
-      write_http_status_line(conn, 400, "Servers unavailable.");
+      write_http_status_line(conn, 404, "Servers unavailable.");
     } else {
       size_t len = 0;
       format_rfc1123_time(date, time(NULL));
@@ -1269,7 +1347,8 @@ directory_handle_command_get(connection_t *conn, char *headers,
              *cp++ = '\n';
            });
         *cp = '\0';
-        /* XXXX NM This could be way more efficiently handled. */
+        /* XXXX This could be way more efficiently handled; let's see if it
+         * shows up under oprofile. */
         if (tor_gzip_compress(&compressed, &compressed_len,
                               inp, cp-inp, ZLIB_METHOD)<0) {
           tor_free(inp);
@@ -1284,7 +1363,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
         connection_write_to_buf(compressed, compressed_len, conn);
         tor_free(compressed);
       } else {
-        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: application/octet-stream\r\n\r\n",
+        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\n\r\n",
                      date,
                      (int)len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
@@ -1501,7 +1580,7 @@ connection_dir_finished_connecting(connection_t *conn)
 }
 
 /** Called when one or more networkstatus fetches have failed (with uppercase
- * fingerprints listed in <b>fp</>).  Mark those fingerprints has having
+ * fingerprints listed in <b>failed</>).  Mark those fingerprints has having
  * failed once. */
 static void
 dir_networkstatus_download_failed(smartlist_t *failed)
@@ -1517,3 +1596,77 @@ dir_networkstatus_download_failed(smartlist_t *failed)
   });
 }
 
+/** Called when one or more networkstatus fetches have failed (with uppercase
+ * fingerprints listed in <b>failed</>). */
+static void
+dir_routerdesc_download_failed(smartlist_t *failed)
+{
+  char digest[DIGEST_LEN];
+  local_routerstatus_t *rs;
+  time_t now = time(NULL);
+  int server = server_mode(get_options()) && get_options()->DirPort;
+  SMARTLIST_FOREACH(failed, const char *, cp,
+  {
+    base16_decode(digest, DIGEST_LEN, cp, strlen(cp));
+    rs = router_get_combined_status_by_digest(digest);
+    if (!rs || rs->n_download_failures >= MAX_ROUTERDESC_DOWNLOAD_FAILURES)
+      continue;
+    ++rs->n_download_failures;
+    if (server) {
+      switch (rs->n_download_failures) {
+        case 1: rs->next_attempt_at = 0; break;
+        case 2: rs->next_attempt_at = 0; break;
+        case 3: rs->next_attempt_at = now+60; break;
+        case 4: rs->next_attempt_at = now+60; break;
+        case 5: rs->next_attempt_at = now+60*2; break;
+        case 6: rs->next_attempt_at = now+60*5; break;
+        case 7: rs->next_attempt_at = now+60*15; break;
+        default: rs->next_attempt_at = TIME_MAX; break;
+      }
+    } else {
+      switch (rs->n_download_failures) {
+        case 1: rs->next_attempt_at = 0; break;
+        case 2: rs->next_attempt_at = now+60; break;
+        case 3: rs->next_attempt_at = now+60*5; break;
+        case 4: rs->next_attempt_at = now+60*10; break;
+        default: rs->next_attempt_at = TIME_MAX; break;
+      }
+    }
+    if (rs->next_attempt_at == 0)
+      log_fn(LOG_DEBUG, "%s failed %d time(s); I'll try again immediately.",
+             cp, (int)rs->n_download_failures);
+    else if (rs->next_attempt_at < TIME_MAX)
+      log_fn(LOG_DEBUG, "%s failed %d time(s); I'll try again in %d seconds.",
+             cp, (int)rs->n_download_failures, (int)(rs->next_attempt_at-now));
+    else
+      log_fn(LOG_DEBUG, "%s failed %d time(s); Giving up for a while.",
+             cp, (int)rs->n_download_failures);
+  });
+
+  /* update_router_descriptor_downloads(time(NULL)); */
+}
+
+/* Given a directory <b>resource</b> request generated by us, containing zero
+ * or more strings separated by plus signs, followed optionally by ".z", store
+ * the strings, in order, into <b>fp_out</b>.  If <b>compressed_out</b> is
+ * non-NULL, set it to 1 if the resource ends in ".z", else set it to 0.
+ */
+int
+dir_split_resource_into_fingerprints(const char *resource,
+                                     smartlist_t *fp_out, int *compressed_out)
+{
+  smartlist_split_string(fp_out, resource, "+", 0, 0);
+  if (compressed_out)
+    *compressed_out = 0;
+  if (smartlist_len(fp_out)) {
+    char *last = smartlist_get(fp_out,smartlist_len(fp_out)-1);
+    size_t last_len = strlen(last);
+    if (last_len > 2 && !strcmp(last+last_len-2, ".z")) {
+      last[last_len-2] = '\0';
+      if (compressed_out)
+        *compressed_out = 1;
+    }
+  }
+  return 0;
+}
+
