@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.227 2005/09/13 21:14:55 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.227 2005/09/13 21:14:55 nickm Exp $";
+/* $Id: dirserv.c,v 1.246 2005/10/06 04:33:40 nickm Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.246 2005/10/06 04:33:40 nickm Exp $";
 
 #include "or.h"
 
@@ -19,6 +19,13 @@ const char dirserv_c_id[] = "$Id: dirserv.c,v 1.227 2005/09/13 21:14:55 nickm Ex
 
 extern long stats_n_seconds_working;
 
+typedef enum {
+  FP_NAMED, /**< Listed in fingerprint file. */
+  FP_VALID, /**< Unlisted but believed valid. */
+  FP_INVALID, /**< Believed invalid. */
+  FP_REJECT, /**< We will not publish this router. */
+} router_status_t;
+
 /** Do we need to regenerate the directory when someone asks for it? */
 static int the_directory_is_dirty = 1;
 static int runningrouters_is_dirty = 1;
@@ -30,16 +37,60 @@ static char *format_versions_list(config_line_t *ln);
 /* Should be static; exposed for testing */
 int add_fingerprint_to_dir(const char *nickname, const char *fp, smartlist_t *list);
 static int router_is_general_exit(routerinfo_t *ri);
+static router_status_t dirserv_router_get_status(const routerinfo_t *router,
+                                                 const char **msg);
+static int dirserv_thinks_router_is_reachable(routerinfo_t *router,
+                                              time_t now);
 
 /************** Fingerprint handling code ************/
 
+static addr_policy_t *authdir_reject_policy = NULL;
+static addr_policy_t *authdir_invalid_policy = NULL;
+
+/** Parse authdir policy strings from the configuration.
+ */
+void
+parse_authdir_policy(void)
+{
+  addr_policy_t *n;
+  if (authdir_reject_policy) {
+    addr_policy_free(authdir_reject_policy);
+    authdir_reject_policy = NULL;
+  }
+  config_parse_addr_policy(get_options()->AuthDirReject,
+                           &authdir_reject_policy, ADDR_POLICY_REJECT);
+  /* ports aren't used. */
+  for (n=authdir_reject_policy; n; n = n->next) {
+    n->prt_min = 1;
+    n->prt_max = 65535;
+  }
+
+  if (authdir_invalid_policy) {
+    addr_policy_free(authdir_invalid_policy);
+    authdir_invalid_policy = NULL;
+  }
+  config_parse_addr_policy(get_options()->AuthDirInvalid,
+                           &authdir_invalid_policy, ADDR_POLICY_REJECT);
+  /* ports aren't used. */
+  for (n=authdir_invalid_policy; n; n = n->next) {
+    n->prt_min = 1;
+    n->prt_max = 65535;
+  }
+}
+
+/** A member of fingerprint_list: maps a name to a fingerprint.
+ **/
 typedef struct fingerprint_entry_t {
-  char *nickname;
+  char *nickname; /**< The name of a router (if this fingerprint is bound to a
+                   * name); the string "!reject" (if this fingerprint should
+                   * always be rejected); or the string "!invalid" (if this
+                   * fingerprint should be accepted but never marked as
+                   * valid. */
   char *fingerprint; /**< Stored as HEX_DIGEST_LEN characters, followed by a NUL */
 } fingerprint_entry_t;
 
 /** List of nickname-\>identity fingerprint mappings for all the routers
- * that we recognize. Used to prevent Sybil attacks. */
+ * that we name.  Used to prevent router impersonation. */
 /* Should be static; exposed for testing */
 smartlist_t *fingerprint_list = NULL;
 
@@ -60,12 +111,14 @@ add_fingerprint_to_dir(const char *nickname, const char *fp, smartlist_t *list)
   fingerprint = tor_strdup(fp);
   tor_strstrip(fingerprint, " ");
 
-  for (i = 0; i < smartlist_len(list); ++i) {
-    ent = smartlist_get(list, i);
-    if (!strcasecmp(ent->nickname,nickname)) {
-      tor_free(ent->fingerprint);
-      ent->fingerprint = fingerprint;
-      return 1;
+  if (nickname[0] != '!') {
+    for (i = 0; i < smartlist_len(list); ++i) {
+      ent = smartlist_get(list, i);
+      if (!strcasecmp(ent->nickname,nickname)) {
+        tor_free(ent->fingerprint);
+        ent->fingerprint = fingerprint;
+        return 1;
+      }
     }
   }
   ent = tor_malloc(sizeof(fingerprint_entry_t));
@@ -126,6 +179,12 @@ dirserv_parse_fingerprint_file(const char *fname)
       log(LOG_NOTICE, "Nickname '%s' too long in fingerprint file. Skipping.", nickname);
       continue;
     }
+    if (!is_legal_nickname(nickname) &&
+        strcasecmp(nickname, "!reject") &&
+        strcasecmp(nickname, "!invalid")) {
+      log(LOG_NOTICE, "Invalid nickname '%s' in fingerprint file. Skipping.", nickname);
+      continue;
+    }
     if (strlen(fingerprint) != FINGERPRINT_LEN ||
         !crypto_pk_check_fingerprint_syntax(fingerprint)) {
       log_fn(LOG_NOTICE, "Invalid fingerprint (nickname '%s', fingerprint %s). Skipping.",
@@ -153,44 +212,83 @@ dirserv_parse_fingerprint_file(const char *fname)
 }
 
 /** Check whether <b>router</b> has a nickname/identity key combination that
- * we recognize from the fingerprint list.  Return 1 if router's
- * identity and nickname match, -1 if we recognize the nickname but
- * the identity key is wrong, and 0 if the nickname is not known. */
-int
-dirserv_router_fingerprint_is_known(const routerinfo_t *router)
+ * we recognize from the fingerprint list, or an IP we automatically act on
+ * according to our configuration.  Return the appropriate router status.
+ *
+ * If the status is 'FP_REJECT' and <b>msg</b> is provided, set
+ * *<b>msg</b> to an explanation of why.
+ */
+static router_status_t
+dirserv_router_get_status(const routerinfo_t *router, const char **msg)
 {
-  int i, found=0;
-  fingerprint_entry_t *ent =NULL;
+  fingerprint_entry_t *nn_ent = NULL, *fp_ent = NULL;
   char fp[FINGERPRINT_LEN+1];
 
   if (!fingerprint_list)
     fingerprint_list = smartlist_create();
 
+  if (crypto_pk_get_fingerprint(router->identity_pkey, fp, 0)) {
+    log_fn(LOG_WARN,"Error computing fingerprint");
+    return -1;
+  }
+
   log_fn(LOG_DEBUG, "%d fingerprints known.", smartlist_len(fingerprint_list));
-  for (i=0;i<smartlist_len(fingerprint_list);++i) {
-    ent = smartlist_get(fingerprint_list, i);
-    log_fn(LOG_DEBUG,"%s vs %s", router->nickname, ent->nickname);
-    if (!strcasecmp(router->nickname,ent->nickname)) {
-      found = 1;
-      break;
+  SMARTLIST_FOREACH(fingerprint_list, fingerprint_entry_t *, ent,
+  {
+    if (!strcasecmp(fp,ent->fingerprint))
+      fp_ent = ent;
+    if (!strcasecmp(router->nickname,ent->nickname))
+      nn_ent = ent;
+  });
+
+  if (fp_ent) {
+    if (!strcasecmp(fp_ent->nickname, "!reject")) {
+      if (msg)
+        *msg = "Fingerprint is marked rejected";
+      return FP_REJECT;
+    } else if (!strcasecmp(fp_ent->nickname, "!invalid")) {
+      if (msg)
+        *msg = "Fingerprint is marged invalid";
+      return FP_INVALID;
     }
   }
 
-  if (!found) { /* No such server known */
-    log_fn(LOG_INFO,"no fingerprint found for '%s'",router->nickname);
+  if (!nn_ent) { /* No such server known with that nickname */
+    addr_policy_result_t rej = router_compare_addr_to_addr_policy(
+                       router->addr, router->or_port, authdir_reject_policy);
+    addr_policy_result_t inv = router_compare_addr_to_addr_policy(
+                       router->addr, router->or_port, authdir_invalid_policy);
+
+    if (rej == ADDR_POLICY_PROBABLY_REJECTED || rej == ADDR_POLICY_REJECTED) {
+      log_fn(LOG_INFO, "Rejecting '%s' because of address %s",
+             router->nickname, router->address);
+      if (msg)
+        *msg = "Authdir is rejecting routers in this range.";
+      return FP_REJECT;
+    }
+    if (inv == ADDR_POLICY_PROBABLY_REJECTED || inv == ADDR_POLICY_REJECTED) {
+      log_fn(LOG_INFO, "Not marking '%s' valid because of address %s",
+             router->nickname, router->address);
+      return FP_INVALID;
+    }
+    if (tor_version_as_new_as(router->platform,"0.1.0.2-rc"))
+      return FP_VALID;
+    else
+      return FP_INVALID;
+    log_fn(LOG_INFO,"No fingerprint found for '%s'",router->nickname);
     return 0;
   }
-  if (crypto_pk_get_fingerprint(router->identity_pkey, fp, 0)) {
-    log_fn(LOG_WARN,"error computing fingerprint");
-    return -1;
-  }
-  if (0==strcasecmp(ent->fingerprint, fp)) {
-    log_fn(LOG_DEBUG,"good fingerprint for '%s'",router->nickname);
-    return 1; /* Right fingerprint. */
+  if (0==strcasecmp(nn_ent->fingerprint, fp)) {
+    log_fn(LOG_DEBUG,"Good fingerprint for '%s'",router->nickname);
+    return FP_NAMED; /* Right fingerprint. */
   } else {
-    log_fn(LOG_WARN,"mismatched fingerprint for '%s': expected '%s' got '%s'",
-           router->nickname, ent->fingerprint, fp);
-    return -1; /* Wrong fingerprint. */
+    log_fn(LOG_WARN,"Mismatched fingerprint for '%s': expected '%s' got '%s'. ContactInfo '%s', platform '%s'.)",
+           router->nickname, nn_ent->fingerprint, fp,
+           router->contact_info ? router->contact_info : "",
+           router->platform ? router->platform : "");
+    if (msg)
+      *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
+    return FP_REJECT; /* Wrong fingerprint. */
   }
 }
 
@@ -214,7 +312,7 @@ dirserv_get_nickname_by_digest(const char *digest)
 
 /** Clear the current fingerprint list. */
 void
-dirserv_free_fingerprint_list()
+dirserv_free_fingerprint_list(void)
 {
   int i;
   fingerprint_entry_t *ent;
@@ -266,33 +364,22 @@ dirserv_router_has_valid_address(routerinfo_t *ri)
   return 0;
 }
 
-/** DOCDOC */
+/** Check whether we, as a directory server, want to accept <b>ri</b>.  If so,
+ * return 0, and set its is_valid,named,running fields.  Otherwise, return -1.
+ *
+ * If the router is rejected, set *<b>msg</b> to an explanation of why.
+ */
 int
-dirserv_wants_to_reject_router(routerinfo_t *ri, int *verified,
+authdir_wants_to_reject_router(routerinfo_t *ri,
                                const char **msg)
 {
   /* Okay.  Now check whether the fingerprint is recognized. */
-  int r = dirserv_router_fingerprint_is_known(ri);
+  router_status_t status = dirserv_router_get_status(ri, msg);
   time_t now;
-  if (r==-1) {
-    log_fn(LOG_WARN, "Known nickname '%s', wrong fingerprint. Not adding (ContactInfo '%s', platform '%s').",
-           ri->nickname, ri->contact_info ? ri->contact_info : "",
-           ri->platform ? ri->platform : "");
-    *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
-    return -1;
-  } else if (r==0) {
-    char fp[FINGERPRINT_LEN+1];
-    log_fn(LOG_INFO, "Unknown nickname '%s' (%s:%d). Will try to add.",
-           ri->nickname, ri->address, ri->or_port);
-    if (crypto_pk_get_fingerprint(ri->identity_pkey, fp, 1) < 0) {
-      log_fn(LOG_WARN, "Error computing fingerprint for '%s'", ri->nickname);
-    } else {
-      log_fn(LOG_INFO, "Fingerprint line: %s %s", ri->nickname, fp);
-    }
-    *verified = 0;
-  } else {
-    *verified = 1;
-  }
+  tor_assert(msg);
+  if (status == FP_REJECT)
+    return -1; /* msg is already set. */
+
   /* Is there too much clock skew? */
   now = time(NULL);
   if (ri->published_on > now+ROUTER_ALLOW_SKEW) {
@@ -319,6 +406,22 @@ dirserv_wants_to_reject_router(routerinfo_t *ri, int *verified,
     *msg = "Rejected: Address is not an IP, or IP is a private address.";
     return -1;
   }
+  /* Okay, looks like we're willing to accept this one. */
+  switch (status) {
+    case FP_NAMED:
+      ri->is_named = ri->is_verified = 1;
+      break;
+    case FP_VALID:
+      ri->is_named = 0;
+      ri->is_verified = 1;
+      break;
+    case FP_INVALID:
+      ri->is_named = ri->is_verified = 0;
+      break;
+    default:
+      tor_assert(0);
+  }
+
   return 0;
 }
 
@@ -331,7 +434,6 @@ dirserv_wants_to_reject_router(routerinfo_t *ri, int *verified,
  *  0 if well-formed but redundant with one we already have;
  * -1 if it looks vaguely like a router descriptor but rejected;
  * -2 if we can't find a router descriptor in <b>desc</b>.
- *
  */
 int
 dirserv_add_descriptor(const char *desc, const char **msg)
@@ -348,7 +450,7 @@ dirserv_add_descriptor(const char *desc, const char **msg)
     *msg = "Rejected: Couldn't parse server descriptor.";
     return -2;
   }
-  if ((r = router_add_to_routerlist(ri, msg))<0) {
+  if ((r = router_add_to_routerlist(ri, msg, 0))<0) {
     return r == -1 ? 0 : -1;
   } else {
     smartlist_t *changed = smartlist_create();
@@ -378,22 +480,39 @@ directory_remove_invalid(void)
     return;
 
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
+    const char *msg;
     routerinfo_t *ent = smartlist_get(descriptor_list, i);
-    int r = dirserv_router_fingerprint_is_known(ent);
-    if (r<0) {
-      log(LOG_INFO, "Router '%s' is now verified with a key; removing old router with same name and different key.",
-          ent->nickname);
-      routerinfo_free(ent);
-      smartlist_del(descriptor_list, i--);
-      changed = 1;
-    } else if (r>0 && !ent->is_verified) {
-      log(LOG_INFO, "Router '%s' is now approved.", ent->nickname);
-      ent->is_verified = ent->is_named = 1;
-      changed = 1;
-    } else if (r==0 && ent->is_verified) {
-      log(LOG_INFO, "Router '%s' is no longer approved.", ent->nickname);
-      ent->is_verified = ent->is_named = 0;
-      changed = 1;
+    router_status_t r = dirserv_router_get_status(ent, &msg);
+    switch (r) {
+      case FP_REJECT:
+        log(LOG_INFO, "Router '%s' is now rejected: %s",
+            ent->nickname, msg?msg:"");
+        routerinfo_free(ent);
+        smartlist_del(descriptor_list, i--);
+        changed = 1;
+        break;
+      case FP_NAMED:
+        if (!ent->is_verified || !ent->is_named) {
+          log(LOG_INFO, "Router '%s' is now verified and named.", ent->nickname);
+          ent->is_verified = ent->is_named = 1;
+          changed = 1;
+        }
+        break;
+      case FP_VALID:
+        if (!ent->is_verified || ent->is_named) {
+          log(LOG_INFO, "Router '%s' is now verified.", ent->nickname);
+          ent->is_verified = 1;
+          ent->is_named = 0;
+          changed = 1;
+        }
+        break;
+      case FP_INVALID:
+        if (ent->is_verified || ent->is_named) {
+          log(LOG_INFO, "Router '%s' is no longer verified.", ent->nickname);
+          ent->is_verified = ent->is_named = 0;
+          changed = 1;
+        }
+        break;
     }
   }
   if (changed)
@@ -407,7 +526,8 @@ directory_remove_invalid(void)
 char *
 dirserver_getinfo_unregistered(const char *question)
 {
-  int i, r;
+  int i;
+  router_status_t r;
   smartlist_t *answerlist;
   char buf[1024];
   char *answer;
@@ -421,10 +541,10 @@ dirserver_getinfo_unregistered(const char *question)
   answerlist = smartlist_create();
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
     ent = smartlist_get(descriptor_list, i);
-    r = dirserv_router_fingerprint_is_known(ent);
+    r = dirserv_router_get_status(ent, NULL);
     if (ent->bandwidthcapacity >= (size_t)min_bw &&
         ent->bandwidthrate >= (size_t)min_bw &&
-        r == 0) {
+        r != FP_NAMED) {
       /* then log this one */
       tor_snprintf(buf, sizeof(buf),
                    "%s: BW %d on '%s'.",
@@ -444,7 +564,7 @@ dirserver_getinfo_unregistered(const char *question)
  * generated one.
  */
 void
-directory_set_dirty()
+directory_set_dirty(void)
 {
   time_t now = time(NULL);
 
@@ -452,6 +572,8 @@ directory_set_dirty()
     the_directory_is_dirty = now;
   if (!runningrouters_is_dirty)
     runningrouters_is_dirty = now;
+  if (!the_v2_networkstatus_is_dirty)
+    the_v2_networkstatus_is_dirty = now;
 }
 
 /**
@@ -554,7 +676,10 @@ list_server_status(smartlist_t *routers, char **router_status_out)
   return 0;
 }
 
-int
+/** Helper: Given pointers to two strings describing tor versions, return -1
+ * if _a precedes _b, 1 if _b preceeds _a, and 0 if they are equivalent.
+ * Used to sort a list of versions. */
+static int
 _compare_tor_version_str_ptr(const void **_a, const void **_b)
 {
   const char *a = *_a, *b = *_b;
@@ -578,7 +703,8 @@ _compare_tor_version_str_ptr(const void **_a, const void **_b)
 /* Given a (possibly empty) list of config_line_t, each line of which contains
  * a list of comma-separated version numbers surrounded by optional space,
  * allocate and return a new string containing the version numbers, in order,
- * separated by commas.  Used to generate Recommended(Client|Server)?Versions */
+ * separated by commas.  Used to generate Recommended(Client|Server)?Versions
+ */
 static char *
 format_versions_list(config_line_t *ln)
 {
@@ -695,7 +821,7 @@ dirserv_dump_directory_to_string(char **dir_out,
 static cached_dir_t the_directory = { NULL, NULL, 0, 0, 0 };
 
 /* Used only by non-auth dirservers: The directory and runningrouters we'll
-* serve when requested. */
+ * serve when requested. */
 static cached_dir_t cached_directory = { NULL, NULL, 0, 0, 0 };
 static cached_dir_t cached_runningrouters = { NULL, NULL, 0, 0, 0 };
 
@@ -762,13 +888,6 @@ dirserv_set_cached_directory(const char *directory, time_t published,
   cached_dir_t *d;
   d = is_running_routers ? &cached_runningrouters : &cached_directory;
   set_cached_dir(d, tor_strdup(directory), published);
-  if (!is_running_routers) {
-    char filename[512];
-    tor_snprintf(filename,sizeof(filename),"%s/cached-directory", get_options()->DataDirectory);
-    if (write_str_to_file(filename,cached_directory.dir,0) < 0) {
-      log_fn(LOG_NOTICE, "Couldn't write cached directory to disk. Ignoring.");
-    }
-  }
 }
 
 /** We've just received a v2 network-status for an authoritative directory
@@ -985,7 +1104,7 @@ static int
 router_is_general_exit(routerinfo_t *ri)
 {
   static const int ports[] = { 80, 443, 6667 };
-  int n_allowed = 3;
+  int n_allowed = 0;
   int i;
   for (i = 0; i < 3; ++i) {
     struct addr_policy_t *policy = ri->exit_policy;
@@ -997,8 +1116,8 @@ router_is_general_exit(routerinfo_t *ri)
       if ((policy->addr & 0xff000000ul) == 0x7f000000ul)
         continue; /* 127.x */
       /* We have a match that is at least a /8. */
-      if (policy->policy_type != ADDR_POLICY_ACCEPT)
-        --n_allowed;
+      if (policy->policy_type == ADDR_POLICY_ACCEPT)
+        ++n_allowed;
       break;
     }
   }
@@ -1014,7 +1133,6 @@ static cached_dir_t the_v2_networkstatus = { NULL, NULL, 0, 0, 0 };
 static int
 generate_v2_networkstatus(void)
 {
-#define BASE64_DIGEST_LEN 27
 #define LONGEST_STATUS_FLAG_NAME_LEN 7
 #define N_STATUS_FLAGS 6
 #define RS_ENTRY_LEN                                                    \
@@ -1038,6 +1156,9 @@ generate_v2_networkstatus(void)
   uint32_t addr;
   crypto_pk_env_t *private_key = get_identity_key();
   smartlist_t *descriptor_list = get_descriptor_list();
+  time_t now = time(NULL);
+  int naming = options->NamingAuthoritativeDir;
+  int versioning = options->VersioningAuthoritativeDir;
   const char *contact;
 
   if (!descriptor_list) {
@@ -1082,7 +1203,7 @@ generate_v2_networkstatus(void)
                "fingerprint %s\n"
                "contact %s\n"
                "published %s\n"
-               "dir-options %s\n"
+               "dir-options%s%s\n"
                "client-versions %s\n"
                "server-versions %s\n"
                "dir-signing-key\n%s\n",
@@ -1090,7 +1211,8 @@ generate_v2_networkstatus(void)
                fingerprint,
                contact,
                published,
-               "Names",
+               naming ? " Names" : "",
+               versioning ? " Versions" : "",
                client_versions,
                server_versions,
                identity_pkey);
@@ -1102,43 +1224,27 @@ generate_v2_networkstatus(void)
       int f_stable = !router_is_unreliable(ri, 1, 0);
       int f_fast = !router_is_unreliable(ri, 0, 1);
       int f_running;
-      int f_named = ri->is_verified;
-      int f_valid = f_named;
-      char identity64[128];
-      char digest64[128];
-
+      int f_authority = router_digest_is_trusted_dir(ri->identity_digest);
+      int f_named = naming && ri->is_named;
+      int f_valid = ri->is_verified;
+      char identity64[BASE64_DIGEST_LEN+1];
+      char digest64[BASE64_DIGEST_LEN+1];
       if (options->AuthoritativeDir) {
-        connection_t *conn = connection_get_by_identity_digest(
-                                         ri->identity_digest, CONN_TYPE_OR);
-        f_running = (router_is_me(ri) && !we_are_hibernating()) ||
-          (conn && conn->state == OR_CONN_STATE_OPEN);
-        /* Update router status in routerinfo_t. */
-        ri->is_running = f_running;
-      } else {
-        f_running = ri->is_running;
+        ri->is_running = dirserv_thinks_router_is_reachable(ri, now);
       }
+      f_running = ri->is_running;
 
       format_iso_time(published, ri->published_on);
 
-      if (base64_encode(identity64, sizeof(identity64),
-                        ri->identity_digest, DIGEST_LEN)<0) {
-        log_fn(LOG_WARN, "Unable to encode router identity digest.");
-        goto done;
-      }
-      if (base64_encode(digest64, sizeof(digest64),
-                        ri->signed_descriptor_digest, DIGEST_LEN)<0) {
-        log_fn(LOG_WARN, "Unable to encode router descriptor digest.");
-        goto done;
-      }
-      identity64[BASE64_DIGEST_LEN] = '\0';
-      digest64[BASE64_DIGEST_LEN] = '\0';
+      digest_to_base64(identity64, ri->identity_digest);
+      digest_to_base64(digest64, ri->signed_descriptor_digest);
 
       in.s_addr = htonl(ri->addr);
       tor_inet_ntoa(&in, ipaddr, sizeof(ipaddr));
 
       if (tor_snprintf(outp, endp-outp,
                        "r %s %s %s %s %s %d %d\n"
-                       "s%s%s%s%s%s%s\n",
+                       "s%s%s%s%s%s%s%s\n",
                        ri->nickname,
                        identity64,
                        digest64,
@@ -1146,15 +1252,16 @@ generate_v2_networkstatus(void)
                        ipaddr,
                        ri->or_port,
                        ri->dir_port,
+                       f_authority?" Authority":"",
                        f_exit?" Exit":"",
-                       f_stable?" Stable":"",
                        f_fast?" Fast":"",
-                       f_running?" Running":"",
                        f_named?" Named":"",
+                       f_stable?" Stable":"",
+                       f_running?" Running":"",
                        f_valid?" Valid":"")<0) {
         log_fn(LOG_WARN, "Unable to print router status.");
         goto done;
-      }
+        }
       outp += strlen(outp);
     });
 
@@ -1232,7 +1339,7 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
       log_fn(LOG_WARN, "Client requested 'all' network status objects; we have none.");
   } else if (!strcmpstart(key, "fp/")) {
     smartlist_t *hexdigests = smartlist_create();
-    smartlist_split_string(hexdigests, key+3, "+", 0, 0);
+    dir_split_resource_into_fingerprints(key+3, hexdigests, NULL);
     SMARTLIST_FOREACH(hexdigests, char *, cp,
         {
           cached_dir_t *cached;
@@ -1278,7 +1385,7 @@ dirserv_get_routerdescs(smartlist_t *descs_out, const char *key)
     smartlist_t *hexdigests = smartlist_create();
     smartlist_t *digests = smartlist_create();
     key += strlen("/tor/server/fp/");
-    smartlist_split_string(hexdigests, key, "+", 0, 0);
+    dir_split_resource_into_fingerprints(key, hexdigests, NULL);
     SMARTLIST_FOREACH(hexdigests, char *, cp,
                       {
                         char *d;
@@ -1374,6 +1481,10 @@ dirserv_free_all(void)
     smartlist_free(fingerprint_list);
     fingerprint_list = NULL;
   }
+  if (authdir_reject_policy)
+    addr_policy_free(authdir_reject_policy);
+  if (authdir_invalid_policy)
+    addr_policy_free(authdir_invalid_policy);
   clear_cached_dir(&the_directory);
   clear_cached_dir(&the_runningrouters);
   clear_cached_dir(&cached_directory);
