@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: hibernate.c,v 1.59 2005/08/08 21:59:48 nickm Exp $ */
-const char hibernate_c_id[] = "$Id: hibernate.c,v 1.59 2005/08/08 21:59:48 nickm Exp $";
+/* $Id: hibernate.c,v 1.60 2005/09/13 21:24:51 arma Exp $ */
+const char hibernate_c_id[] = "$Id: hibernate.c,v 1.60 2005/09/13 21:24:51 arma Exp $";
 
 /**
  * \file hibernate.c
@@ -413,10 +413,10 @@ accounting_run_housekeeping(time_t now)
   }
   if (time_to_record_bandwidth_usage(now)) {
     if (accounting_record_bandwidth_usage(now)) {
-      log_fn(LOG_ERR, "Couldn't record bandwidth usage; exiting.");
-      /* XXX this can fail when you're out of fd's, causing a crash.
-       * Perhaps the better answer is to hold the file open all the
-       * time? */
+      log_fn(LOG_ERR, "Couldn't record bandwidth usage to disk; exiting.");
+      /* This can fail when we're out of fd's, causing a crash.
+       * The current answer is to reserve 32 more than we need, in
+       * set_max_file_descriptors(). */
       exit(1);
     }
   }
