@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.549 2005/09/14 02:36:29 nickm Exp $ */
-const char main_c_id[] = "$Id: main.c,v 1.549 2005/09/14 02:36:29 nickm Exp $";
+/* $Id: main.c,v 1.575 2005/10/06 22:22:22 nickm Exp $ */
+const char main_c_id[] = "$Id: main.c,v 1.575 2005/10/06 22:22:22 nickm Exp $";
 
 /**
  * \file main.c
@@ -95,7 +95,10 @@ static char* nt_strerror(uint32_t errnum);
 #define FORCE_REGENERATE_DESCRIPTOR_INTERVAL 18*60*60 /* 18 hours */
 #define CHECK_DESCRIPTOR_INTERVAL 60 /* one minute */
 #define BUF_SHRINK_INTERVAL 60 /* one minute */
+#define DESCRIPTOR_RETRY_INTERVAL 10
+#define DESCRIPTOR_FAILURE_RESET_INTERVAL 60*60
 #define TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT (20*60) /* 20 minutes */
+#define ENTROPY_INTERVAL 60*60
 
 /********* END VARIABLES ************/
 
@@ -480,6 +483,7 @@ void
 directory_all_unreachable(time_t now)
 {
   connection_t *conn;
+  /* XXXX011 NM Update this to reflect new directories? */
 
   has_fetched_directory=0;
   stats_n_seconds_working=0; /* reset it */
@@ -526,40 +530,29 @@ get_status_fetch_period(or_options_t *options)
     return 30*60;
 }
 
-/** This function is called whenever we successfully pull down a directory.
- * If <b>identity_digest</b> is defined, it contains the digest of the
- * router that just gave us this directory. */
+/** This function is called whenever we successfully pull down some new
+ * network statuses or server descriptors. */
 void
-directory_has_arrived(time_t now, char *identity_digest)
+directory_info_has_arrived(time_t now, int from_cache)
 {
   or_options_t *options = get_options();
 
-  log_fn(LOG_INFO, "A directory has arrived.");
+  if (!router_have_minimum_dir_info()) {
+    log_fn(LOG_NOTICE, "I learned some more directory information, but not enough to build a circuit.");
+    return;
+  }
+
+  if (!has_fetched_directory) {
+    log_fn(LOG_NOTICE, "We have enough directory information to build circuits.");
+  }
 
   has_fetched_directory=1;
-  /* Don't try to upload or download anything for a while
-   * after the directory we had when we started.
-   */
-  if (!time_to_fetch_directory)
-    time_to_fetch_directory = now + get_dir_fetch_period(options);
-
-  if (!time_to_fetch_running_routers)
-    time_to_fetch_running_routers = now + get_status_fetch_period(options);
-
-  if (identity_digest) /* if it's fresh */
-    helper_nodes_set_status_from_directory();
-
-  if (server_mode(options) && identity_digest) {
-    /* if this is us, then our dirport is reachable */
-    if (router_digest_is_me(identity_digest))
-      router_dirport_found_reachable();
-  }
 
   if (server_mode(options) &&
       !we_are_hibernating()) { /* connect to the appropriate routers */
     if (!authdir_mode(options))
       router_retry_connections(0);
-    if (identity_digest) /* we got a fresh directory */
+    if (!from_cache)
       consider_testing_reachability();
   }
 }
@@ -583,6 +576,13 @@ run_connection_housekeeping(int i, time_t now)
       conn->timestamp_lastwritten + 5*60 < now) {
     log_fn(LOG_INFO,"Expiring wedged directory conn (fd %d, purpose %d)",
            conn->s, conn->purpose);
+    /* This check is temporary; it's to let us know whether we should consider
+     * parsing partial serverdesc responses. */
+    if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
+        buf_datalen(conn->inbuf)>=(24*1024)) {
+      log_fn(LOG_NOTICE, "Expired a wedged directory connection that had already downloaded %d bytes of descriptors.  If this happens often, please file a bug report.",
+             (int)buf_datalen(conn->inbuf));
+    }
     connection_mark_for_close(conn);
     return;
   }
@@ -638,6 +638,9 @@ run_scheduled_events(time_t now)
   static time_t time_to_check_listeners = 0;
   static time_t time_to_check_descriptor = 0;
   static time_t time_to_shrink_buffers = 0;
+  static time_t time_to_try_getting_descriptors = 0;
+  static time_t time_to_reset_descriptor_failures = 0;
+  static time_t time_to_add_entropy = 0;
   or_options_t *options = get_options();
   int i;
 
@@ -663,6 +666,16 @@ run_scheduled_events(time_t now)
       router_upload_dir_desc_to_dirservers(0);
   }
 
+  if (time_to_try_getting_descriptors < now) {
+    update_router_descriptor_downloads(now);
+    time_to_try_getting_descriptors = now + DESCRIPTOR_RETRY_INTERVAL;
+  }
+
+  if (time_to_reset_descriptor_failures < now) {
+    router_reset_descriptor_download_failures();
+    time_to_reset_descriptor_failures = now + DESCRIPTOR_FAILURE_RESET_INTERVAL;
+  }
+
   /** 1b. Every MAX_SSL_KEY_LIFETIME seconds, we change our TLS context. */
   if (!last_rotated_certificate)
     last_rotated_certificate = now;
@@ -678,6 +691,14 @@ run_scheduled_events(time_t now)
      *      them at all. */
   }
 
+  if (time_to_add_entropy == 0)
+    time_to_add_entropy = now + ENTROPY_INTERVAL;
+  if (time_to_add_entropy < now) {
+    /* We already seeded once, so don't die on failure. */
+    crypto_seed_rng();
+    time_to_add_entropy = now + ENTROPY_INTERVAL;
+  }
+
   /** 1c. If we have to change the accounting interval or record
    * bandwidth used in this accounting interval, do so. */
   if (accounting_is_enabled(options))
@@ -687,9 +708,9 @@ run_scheduled_events(time_t now)
    * new running-routers list, and/or force-uploading our descriptor
    * (if we've passed our internal checks). */
   if (time_to_fetch_directory < now) {
-    time_t next_status_fetch;
     /* purge obsolete entries */
     routerlist_remove_old_routers(ROUTER_MAX_AGE);
+    networkstatus_list_clean(now);
 
     if (authdir_mode(options)) {
       if (!we_are_hibernating()) { /* try to determine reachability */
@@ -697,20 +718,21 @@ run_scheduled_events(time_t now)
       }
     }
 
-    directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
-    time_to_fetch_directory = now + get_dir_fetch_period(options);
-    next_status_fetch = now + get_status_fetch_period(options);
-    if (time_to_fetch_running_routers < next_status_fetch) {
-      time_to_fetch_running_routers = next_status_fetch;
+    /* Only caches actually need to fetch directories now. */
+    if (options->DirPort && !options->V1AuthoritativeDir) {
+      directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
     }
 
+    time_to_fetch_directory = now + get_dir_fetch_period(options);
+
     /* Also, take this chance to remove old information from rephist
      * and the rend cache. */
     rep_history_clean(now - options->RephistTrackTime);
     rend_cache_clean();
   }
 
-  if (time_to_fetch_running_routers < now) {
+  /* Caches need to fetch running_routers; directory clients don't. */
+  if (options->DirPort && time_to_fetch_running_routers < now) {
     if (!authdir_mode(options) || !options->V1AuthoritativeDir) {
       directory_get_from_dirserver(DIR_PURPOSE_FETCH_RUNNING_LIST, NULL, 1);
     }
@@ -732,12 +754,15 @@ run_scheduled_events(time_t now)
         !we_are_hibernating())
       consider_testing_reachability();
 
-    if (server_mode(options) && options->DirPort)
-      update_networkstatus_cache_downloads(now);
-    /* XXXX Disabled until 0.1.1.6 is out: only servers need networkstatus.
-    else
-      update_networkstatus_client_downloads(now);
-    */
+    /* If any networkstatus documents are no longer recent, we need to
+     * update all the descriptors' running status. */
+    networkstatus_list_update_recent(now);
+    routers_update_all_from_networkstatus();
+
+    /* Also, once per minute, check whether we want to download any
+     * networkstatus documents.
+     */
+    update_networkstatus_downloads(now);
   }
 
   /** 3a. Every second, we examine pending circuits and prune the
@@ -911,6 +936,8 @@ do_hup(void)
   if (accounting_is_enabled(options))
     accounting_record_bandwidth_usage(time(NULL));
 
+  router_reset_warnings();
+  routerlist_reset_warnings();
   addressmap_clear_transient();
   /* first, reload config variables, in case they've changed */
   /* no need to provide argc/v, they've been cached inside init_from_config */
@@ -927,8 +954,20 @@ do_hup(void)
       log_fn(LOG_NOTICE, "Error reloading fingerprints. Continuing with old list.");
     }
   }
-  /* Fetch a new directory. Even authdirservers do this. */
-  directory_get_from_dirserver(DIR_PURPOSE_FETCH_DIR, NULL, 1);
+
+  /* Rotate away from the old dirty circuits. This has to be done
+   * after we've read the new options, but before we start using
+   * circuits for directory fetches. */
+  circuit_expire_all_dirty_circs();
+
+  /* retry appropriate downloads */
+  router_reset_status_download_failures();
+  router_reset_descriptor_download_failures();
+  update_networkstatus_downloads(time(NULL));
+
+  /* We'll retry routerstatus downloads in about 10 seconds; no need to
+   * force a retry there. */
+
   if (server_mode(options)) {
     const char *descriptor;
     /* Restart cpuworker and dnsworker processes, so they get up-to-date
@@ -980,9 +1019,13 @@ do_main_loop(void)
   if (router_reload_router_list()) {
     return -1;
   }
+  /* load the networkstatuses. (This launches a download for new routers as
+   * appropriate.)
+   */
   if (router_reload_networkstatus()) {
     return -1;
   }
+  directory_info_has_arrived(time(NULL),1);
 
   if (authdir_mode(get_options())) {
     /* the directory is already here, run startup things */
@@ -1288,7 +1331,8 @@ static int
 tor_init(int argc, char *argv[])
 {
   time_of_process_start = time(NULL);
-  closeable_connection_lst = smartlist_create();
+  if (!closeable_connection_lst)
+    closeable_connection_lst = smartlist_create();
   /* Initialize the history structures. */
   rep_hist_init();
   /* Initialize the service cache. */
@@ -1333,8 +1377,7 @@ tor_init(int argc, char *argv[])
 void
 tor_free_all(int postfork)
 {
-  routerlist_free_current();
-  free_trusted_dir_servers();
+  routerlist_free_all();
   addressmap_free_all();
   set_exit_redirects(NULL); /* free the registered exit redirects */
   free_socks_policy();
@@ -1346,10 +1389,11 @@ tor_free_all(int postfork)
   dns_free_all();
   clear_pending_onions();
   circuit_free_all();
+  helper_nodes_free_all();
   connection_free_all();
   if (!postfork) {
     config_free_all();
-    router_free_all_keys();
+    router_free_all();
   }
   tor_tls_free_all();
   /* stuff in main.c */
@@ -1363,7 +1407,8 @@ tor_free_all(int postfork)
 
 /** Do whatever cleanup is necessary before shutting Tor down. */
 void
-tor_cleanup(void) {
+tor_cleanup(void)
+{
   or_options_t *options = get_options();
   /* Remove our pid file. We don't care if there was an error when we
    * unlink, nothing we could do about it anyways. */
@@ -1428,7 +1473,8 @@ do_hash_password(void)
 /** Checks if torrc is present in the same directory
  *  as the service executable.
  *  Return 1 if it is, 0 if it is not present. */
-static int nt_torrc_is_present()
+static int
+nt_torrc_is_present()
 {
   HANDLE hFile;
   TCHAR szPath[_MAX_PATH];
@@ -1448,7 +1494,7 @@ static int nt_torrc_is_present()
   path_to_torrc = tor_malloc(len);
   if (tor_snprintf(path_to_torrc, len, "%s%s%s", szDrive,  szDir, torrc)<0) {
     printf("Failed: tor_snprinf()\n");
-    free(path_to_torrc);
+    tor_free(path_to_torrc);
     return 0;
   }
 
@@ -1756,7 +1802,7 @@ nt_service_install(void)
   }
 
   if ((hSCManager = nt_service_open_scm()) == NULL) {
-    free(command);
+    tor_free(command);
     return 0;
   }
 
@@ -1773,7 +1819,7 @@ nt_service_install(void)
     printf("CreateService() failed : %s\n", errmsg);
     CloseServiceHandle(hSCManager);
     LocalFree(errmsg);
-    free(command);
+    tor_free(command);
     return 0;
   }
 
@@ -1889,10 +1935,23 @@ nt_strerror(uint32_t errnum)
 }
 #endif
 
+#ifdef USE_DMALLOC
+#include <openssl/crypto.h>
+static void
+_tor_dmalloc_free(void *p)
+{
+  tor_free(p);
+}
+#endif
+
 /** DOCDOC */
 int
 tor_main(int argc, char *argv[])
 {
+#ifdef USE_DMALLOC
+    int r = CRYPTO_set_mem_ex_functions(_tor_malloc, _tor_realloc, _tor_dmalloc_free);
+    log_fn(LOG_NOTICE, "r = %d", r);
+#endif
 #ifdef MS_WINDOWS_SERVICE
   backup_argv = argv;
   backup_argc = argc;
