@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.680 2005/09/14 02:36:29 nickm Exp $ */
+/* $Id: or.h,v 1.710 2005/10/07 19:25:01 arma Exp $ */
 
 /**
  * \file or.h
@@ -11,7 +11,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.680 2005/09/14 02:36:29 nickm Exp $"
+#define OR_H_ID "$Id: or.h,v 1.710 2005/10/07 19:25:01 arma Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -591,24 +591,30 @@ struct connection_t {
   uint8_t type; /**< What kind of connection is this? */
   uint8_t state; /**< Current state of this connection. */
   uint8_t purpose; /**< Only used for DIR types currently. */
-  uint8_t wants_to_read; /**< Boolean: should we start reading again once
-                          * the bandwidth throttler allows it?
-                          */
-  uint8_t wants_to_write; /**< Boolean: should we start writing again once
-                           * the bandwidth throttler allows reads?
-                           */
+  unsigned wants_to_read:1; /**< Boolean: should we start reading again once
+                            * the bandwidth throttler allows it?
+                            */
+  unsigned wants_to_write:1; /**< Boolean: should we start writing again once
+                             * the bandwidth throttler allows reads?
+                             */
+  unsigned hold_open_until_flushed:1; /**< Despite this connection's being
+                                      * marked for close, do we flush it
+                                      * before closing it?
+                                      */
+  unsigned has_sent_end:1; /**< For debugging; only used on edge connections.
+                         * Set once we've set the stream end,
+                         * and check in circuit_about_to_close_connection(). */
+
   int s; /**< Our socket; -1 if this connection is closed. */
   int poll_index; /* XXXX rename. */
   struct event *read_event; /**< libevent event structure. */
   struct event *write_event; /**< libevent event structure. */
-  int marked_for_close; /**< Boolean: should we close this conn on the next
-                         * iteration of the main loop?
+  int marked_for_close; /**< Should we close this conn on the next iteration
+                         * of the main loop? (If true, holds the line number
+                         * where this connection was marked.)
                          */
   const char *marked_for_close_file; /**< For debugging: in which file were
                                       * we marked for close? */
-  int hold_open_until_flushed; /**< Despite this connection's being marked
-                                * for close, do we flush it before closing it?
-                                */
 
   buf_t *inbuf; /**< Buffer holding data read over this connection. */
   int inbuf_reached_eof; /**< Boolean: did read() return 0 on this conn? */
@@ -638,7 +644,7 @@ struct connection_t {
   char *chosen_exit_name;
 
 /* Used only by OR connections: */
-  tor_tls *tls; /**< TLS connection state (OR only.) */
+  tor_tls_t *tls; /**< TLS connection state (OR only.) */
   uint16_t next_circ_id; /**< Which circ_id do we try to use next on
                           * this connection?  This is always in the
                           * range 0..1<<15-1. (OR only.)*/
@@ -668,10 +674,10 @@ struct connection_t {
   int deliver_window; /**< How many more relay cells can end at me? (Edge
                        * only.) */
 
+#if 0
   int done_sending; /**< For half-open connections; not used currently. */
   int done_receiving; /**< For half-open connections; not used currently. */
-  char has_sent_end; /**< For debugging: set once we've set the stream end,
-                        and check in circuit_about_to_close_connection(). */
+#endif
   struct circuit_t *on_circuit; /**< The circuit (if any) that this edge
                                  * connection is using. */
 
@@ -769,10 +775,8 @@ typedef struct {
   unsigned int is_verified:1; /**< Has a trusted dirserver validated this OR?
                                *  (For Authdir: Have we validated this OR?)
                                */
-  /*XXXX Make this get used once we think we do naming right. NM */
-  unsigned int is_named:1; /* Do we believe the nickname that this OR gives us? */
-
-  time_t status_set_at; /**< When did we last update is_running? */
+  unsigned int is_named:1; /**< Do we believe the nickname that this OR gives
+                            * us? */
 
   /* The below items are used only by authdirservers for
    * reachability testing. */
@@ -785,14 +789,7 @@ typedef struct {
   int num_unreachable_notifications;
 } routerinfo_t;
 
-/** Contents of a running-routers list */
-typedef struct running_routers_t {
-  time_t published_on; /**< When was the list marked as published? */
-  /** Which ORs are on the list?  Entries may be prefixed with ! and $. */
-  smartlist_t *running_routers;
-} running_routers_t;
-
-/** Contents of a single per-router entry in a network status object.
+/** Contents of a single router entry in a network status object.
  */
 typedef struct routerstatus_t {
   time_t published_on; /**< When was this router published? */
@@ -811,13 +808,32 @@ typedef struct routerstatus_t {
   unsigned int is_valid:1; /**< True iff this router is validated. */
 } routerstatus_t;
 
+/** DOCDOC */
+typedef struct local_routerstatus_t {
+  routerstatus_t status;
+  time_t next_attempt_at; /**< When should we try this descriptor again? */
+  uint8_t n_download_failures; /**< Number of failures trying to download the
+                                * most recent descriptor. */
+  unsigned int should_download:1; /**< DOCDOC */
+  unsigned int name_lookup_warned:1; /**< Have we warned the user for referring
+                                      * to this (unnamed) router by nickname?
+                                      */
+} local_routerstatus_t;
+
+/*XXXX011 make this configurable? */
+#define MAX_ROUTERDESC_DOWNLOAD_FAILURES 8
+
 /** Contents of a (v2 or later) network status object. */
 typedef struct networkstatus_t {
   /** When did we receive the network-status document? */
   time_t received_on;
+
   /** What was the digest of the document? */
   char networkstatus_digest[DIGEST_LEN];
 
+  unsigned int is_recent; /**< Is this recent enough to influence running
+                           * status? */
+
   /* These fields come from the actual network-status document.*/
   time_t published_on; /**< Declared publication date. */
 
@@ -833,7 +849,10 @@ typedef struct networkstatus_t {
   char *server_versions; /**< comma-separated list of recommended server
                           * versions. */
 
-  int binds_names:1; /**< True iff this directory server binds names. */
+  unsigned int binds_names:1; /**< True iff this directory server binds names. */
+  unsigned int recommends_versions:1; /**< True iff this directory server
+                                       * recommends client and server software
+                                       * versions. */
 
   smartlist_t *entries; /**< List of router_status_t*.   This list is kept
                          * sorted by identity_digest. */
@@ -843,14 +862,6 @@ typedef struct networkstatus_t {
 typedef struct {
   /** List of routerinfo_t. */
   smartlist_t *routers;
-  /** Which versions of tor are recommended by this directory? */
-  char *software_versions;
-  /** When was the most recent directory that contributed to this list
-   * published? */
-  time_t published_on;
-  time_t running_routers_updated_on;
-  /** What is the most recently received running_routers structure? */
-  running_routers_t *running_routers;
 } routerlist_t;
 
 /** Information on router used when extending a circuit.  (We don't need a
@@ -972,7 +983,8 @@ struct circuit_t {
   uint32_t magic; /**< For memory debugging: must equal CIRCUIT_MAGIC. */
 
   int marked_for_close; /**< Should we close this circuit at the end of the
-                         * main loop? */
+                         * main loop? (If true, holds the line number where
+                         * this circuit was marked.) */
   const char *marked_for_close_file; /**< For debugging: in which file was this
                                       * circuit marked for close? */
 
@@ -1088,17 +1100,20 @@ typedef struct circuit_t circuit_t;
 #define ALLOW_UNVERIFIED_RENDEZVOUS   8
 #define ALLOW_UNVERIFIED_INTRODUCTION 16
 
+/** An entry specifying a set of addresses and ports that should be remapped
+ * to another address and port before exiting this exit node. */
 typedef struct exit_redirect_t {
   uint32_t addr;
   uint32_t mask;
   uint16_t port_min;
   uint16_t port_max;
 
-  int is_redirect;
   uint32_t addr_dest;
   uint16_t port_dest;
+  unsigned is_redirect:1;
 } exit_redirect_t;
 
+/** A linked list of lines in a config file. */
 typedef struct config_line_t {
   char *key;
   char *value;
@@ -1174,7 +1189,12 @@ typedef struct {
   int DirPort; /**< Port to listen on for directory connections. */
   int AssumeReachable; /**< Whether to publish our descriptor regardless. */
   int AuthoritativeDir; /**< Boolean: is this an authoritative directory? */
-  int V1AuthoritativeDir; /**< Boolean: is this an authoritative directory? */
+  int V1AuthoritativeDir; /**< Boolean: is this an authoritative directory?
+                           * for version 1 directories? */
+  int NamingAuthoritativeDir; /**< Boolean: is this an authoritative directory
+                               * that's willing to bind names? */
+  int VersioningAuthoritativeDir; /**< Boolean: is this an authoritative directory
+                                   * that's willing to recommend versions? */
   int ClientOnly; /**< Boolean: should we never evolve into a server role? */
   int NoPublish; /**< Boolean: should we never publish a descriptor? */
   int ConnLimit; /**< Requested maximum number of simultaneous connections. */
@@ -1239,6 +1259,10 @@ typedef struct {
   config_line_t *RedirectExit; /**< List of config lines for simple
                                        * addr/port redirection */
   smartlist_t *RedirectExitList; /**< List of exit_redirect_t */
+  config_line_t *AuthDirReject; /**< Address policy for descriptors to
+                                 * reject. */
+  config_line_t *AuthDirInvalid; /**< Address policy for descriptors to
+                                  * never mark as valid. */
   char *AccountingStart; /**< How long is the accounting interval, and when
                           * does it start? */
   uint64_t AccountingMax; /**< How many bytes do we allow per accounting
@@ -1320,10 +1344,10 @@ size_t buf_capacity(const buf_t *buf);
 const char *_buf_peek_raw_buffer(const buf_t *buf);
 
 int read_to_buf(int s, size_t at_most, buf_t *buf, int *reached_eof);
-int read_to_buf_tls(tor_tls *tls, size_t at_most, buf_t *buf);
+int read_to_buf_tls(tor_tls_t *tls, size_t at_most, buf_t *buf);
 
 int flush_buf(int s, buf_t *buf, size_t *buf_flushlen);
-int flush_buf_tls(tor_tls *tls, buf_t *buf, size_t *buf_flushlen);
+int flush_buf_tls(tor_tls_t *tls, buf_t *buf, size_t *buf_flushlen);
 
 int write_to_buf(const char *string, size_t string_len, buf_t *buf);
 int fetch_from_buf(char *string, size_t string_len, buf_t *buf);
@@ -1373,6 +1397,7 @@ void helper_nodes_set_status_from_directory(void);
 int helper_nodes_update_state(or_state_t *state);
 int helper_nodes_parse_state(or_state_t *state, int set, const char **err);
 int helper_nodes_getinfo_helper(const char *question, char **answer);
+void helper_nodes_free_all(void);
 
 /********************************* circuitlist.c ***********************/
 
@@ -1395,6 +1420,7 @@ circuit_t *circuit_get_rendezvous(const char *cookie);
 circuit_t *circuit_get_clean_open(uint8_t purpose, int need_uptime,
                                   int need_capacity, int internal);
 void circuit_mark_all_unused_circs(void);
+void circuit_expire_all_dirty_circs(void);
 void _circuit_mark_for_close(circuit_t *circ, int line, const char *file);
 
 #define circuit_mark_for_close(c) \
@@ -1583,6 +1609,7 @@ void addressmap_register(const char *address, char *new_address, time_t expires)
 int client_dns_incr_failures(const char *address);
 void client_dns_clear_failures(const char *address);
 void client_dns_set_addressmap(const char *address, uint32_t val, const char *exitname, int ttl);
+int address_is_in_virtual_range(const char *addr);
 const char *addressmap_register_virtual_address(int type, char *new_address);
 void addressmap_get_mappings(smartlist_t *sl, time_t min_expires, time_t max_expires);
 
@@ -1645,6 +1672,12 @@ void control_adjust_event_log_severity(void);
 void disable_control_logging(void);
 void enable_control_logging(void);
 
+/** Execute the statement <b>stmt</b>, which may log events concerning the
+ * connection <b>conn</b>.  To prevent infinite loops, disable log messages
+ * being stent to controllers if <b>conn</b> is a control connection.
+ *
+ * Stmt must not contain any return or goto statements.
+ */
 #define CONN_LOG_PROTECT(conn, stmt)                                    \
   do {                                                                  \
     int _log_conn_is_control = (conn && conn->type == CONN_TYPE_CONTROL); \
@@ -1655,6 +1688,11 @@ void enable_control_logging(void);
       enable_control_logging();                                         \
   } while (0)
 
+/** Log information about the connection <b>conn</b>, protecting it as with
+ * CONN_LOG_PROTECT. Example:
+ *
+ * LOG_FN_CONN(conn, (LOG_DEBUG, "Socket %d wants to write", conn->s));
+ **/
 #define LOG_FN_CONN(conn, args)                 \
   CONN_LOG_PROTECT(conn, log_fn args)
 
@@ -1704,12 +1742,14 @@ int connection_dir_finished_connecting(connection_t *conn);
 void connection_dir_request_failed(connection_t *conn);
 void parse_dir_policy(void);
 void free_dir_policy(void);
+int dir_split_resource_into_fingerprints(const char *resource,
+                               smartlist_t *fp_out, int *compresseed_out);
 
 /********************************* dirserv.c ***************************/
 
+void parse_authdir_policy(void);
 int dirserv_add_own_fingerprint(const char *nickname, crypto_pk_env_t *pk);
 int dirserv_parse_fingerprint_file(const char *fname);
-int dirserv_router_fingerprint_is_known(const routerinfo_t *router);
 void dirserv_free_fingerprint_list(void);
 const char *dirserv_get_nickname_by_digest(const char *digest);
 int dirserv_add_descriptor(const char *desc, const char **msg);
@@ -1734,7 +1774,7 @@ void dirserv_orconn_tls_done(const char *address,
                              const char *digest_rcvd,
                              const char *nickname,
                              int as_advertised);
-int dirserv_wants_to_reject_router(routerinfo_t *ri, int *verified,
+int authdir_wants_to_reject_router(routerinfo_t *ri,
                                    const char **msg);
 void dirserv_free_all(void);
 
@@ -1786,7 +1826,7 @@ void connection_stop_writing(connection_t *conn);
 void connection_start_writing(connection_t *conn);
 
 void directory_all_unreachable(time_t now);
-void directory_has_arrived(time_t now, char *identity_digest);
+void directory_info_has_arrived(time_t now, int from_cache);
 
 int control_signal_act(int the_signal);
 void handle_signals(int is_parent);
@@ -1928,10 +1968,11 @@ int rend_encode_service_descriptor(rend_service_descriptor_t *desc,
 rend_service_descriptor_t *rend_parse_service_descriptor(const char *str, size_t len);
 int rend_get_service_id(crypto_pk_env_t *pk, char *out);
 
+/** A cached rendezvous descriptor. */
 typedef struct rend_cache_entry_t {
-  size_t len; /* Length of desc */
-  time_t received; /* When was the descriptor received? */
-  char *desc; /* Service descriptor */
+  size_t len; /** Length of <b>desc</b> */
+  time_t received; /** When was the descriptor received? */
+  char *desc; /** Service descriptor */
   rend_service_descriptor_t *parsed; /* Parsed value of 'desc' */
 } rend_cache_entry_t;
 
@@ -2018,12 +2059,15 @@ int router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
                                  crypto_pk_env_t *ident_key);
 int is_legal_nickname(const char *s);
 int is_legal_nickname_or_hexdigest(const char *s);
-void router_free_all_keys(void);
+void router_reset_warnings(void);
+void router_free_all(void);
 
 /********************************* routerlist.c ***************************/
 
 /** Represents information about a single trusted directory server. */
 typedef struct trusted_dir_server_t {
+  char *description;
+  char *nickname;
   char *address; /**< Hostname */
   uint32_t addr; /**< IPv4 address */
   uint16_t dir_port; /**< Directory port */
@@ -2051,7 +2095,7 @@ trusted_dir_server_t *router_get_trusteddirserver_by_digest(
      const char *digest);
 int all_trusted_directory_servers_down(void);
 void routerlist_add_family(smartlist_t *sl, routerinfo_t *router);
-void add_nickname_list_to_smartlist(smartlist_t *sl, const char *list, int warn_if_down);
+void add_nickname_list_to_smartlist(smartlist_t *sl, const char *list, int warn_if_down, int warn_if_unnamed);
 routerinfo_t *routerlist_find_my_routerinfo(void);
 int exit_policy_implicitly_allows_local_networks(addr_policy_t *policy,
                                                  int warn);
@@ -2070,25 +2114,27 @@ routerinfo_t *router_choose_random_node(const char *preferred,
                                         smartlist_t *excludedsmartlist,
                                         int need_uptime, int need_bandwidth,
                                         int allow_unverified, int strict);
-routerinfo_t *router_get_by_nickname(const char *nickname);
+routerinfo_t *router_get_by_nickname(const char *nickname,
+                                     int warn_if_unnamed);
 routerinfo_t *router_get_by_hexdigest(const char *hexdigest);
 routerinfo_t *router_get_by_digest(const char *digest);
 int router_digest_is_trusted_dir(const char *digest);
 void router_get_routerlist(routerlist_t **prouterlist);
-time_t routerlist_get_published_time(void);
+void routerlist_reset_warnings(void);
 void routerlist_free(routerlist_t *routerlist);
 void routerinfo_free(routerinfo_t *router);
 void routerstatus_free(routerstatus_t *routerstatus);
 void networkstatus_free(networkstatus_t *networkstatus);
-void routerlist_free_current(void);
-void free_trusted_dir_servers(void);
+void routerlist_free_all(void);
 routerinfo_t *routerinfo_copy(const routerinfo_t *router);
 void router_mark_as_down(const char *digest);
 void routerlist_remove_old_routers(int age);
-int router_add_to_routerlist(routerinfo_t *router, const char **msg);
+void networkstatus_list_clean(time_t now);
+int router_add_to_routerlist(routerinfo_t *router, const char **msg,
+                             int from_cache);
 int router_load_single_router(const char *s, const char **msg);
-int router_load_routerlist_from_directory(const char *s,crypto_pk_env_t *pkey,
-                                        int dir_is_recent, int dir_is_cached);
+void router_load_routers_from_string(const char *s, int from_cache,
+                                     smartlist_t *requested_fingerprints);
 typedef enum { NS_FROM_CACHE, NS_FROM_DIR, NS_GENERATED} networkstatus_source_t;
 int router_set_networkstatus(const char *s, time_t arrived_at,
                              networkstatus_source_t source,
@@ -2100,22 +2146,22 @@ int router_exit_policy_all_routers_reject(uint32_t addr, uint16_t port,
                                           int need_uptime);
 
 int router_exit_policy_rejects_all(routerinfo_t *router);
-void running_routers_free(running_routers_t *rr);
-void routerlist_set_runningrouters(routerlist_t *list, running_routers_t *rr);
-int routers_update_status_from_entry(smartlist_t *routers,
-                                        time_t list_time,
-                                        const char *s);
-int router_update_status_from_smartlist(routerinfo_t *r,
-                                        time_t list_time,
-                                        smartlist_t *running_list);
-void add_trusted_dir_server(const char *addr, uint16_t port,
+void add_trusted_dir_server(const char *nickname,
+                            const char *address, uint16_t port,
                             const char *digest, int supports_v1);
 void clear_trusted_dir_servers(void);
 networkstatus_t *networkstatus_get_by_digest(const char *digest);
-void update_networkstatus_cache_downloads(time_t now);
-void update_networkstatus_client_downloads(time_t now);
-void routers_update_status_from_networkstatus(smartlist_t *routers);
+local_routerstatus_t *router_get_combined_status_by_digest(const char *digest);
+void update_networkstatus_downloads(time_t now);
+void update_router_descriptor_downloads(time_t now);
+void routers_update_all_from_networkstatus(void);
+void routers_update_status_from_networkstatus(smartlist_t *routers,
+                                              int reset_failures);
 smartlist_t *router_list_superseded(void);
+int router_have_minimum_dir_info(void);
+void networkstatus_list_update_recent(time_t now);
+void router_reset_descriptor_download_failures(void);
+void router_reset_status_download_failures(void);
 
 /********************************* routerparse.c ************************/
 
@@ -2139,6 +2185,15 @@ typedef struct tor_version_t {
   char status_tag[MAX_STATUS_TAG_LEN];
 } tor_version_t;
 
+typedef enum version_status_t {
+  VS_RECOMMENDED=0, /**< This version is listed as recommended. */
+  VS_OLD=1, /**< This version is older than any recommended version. */
+  VS_NEW=2, /**< This version is newer than any recommended version. */
+  VS_NEW_IN_SERIES=3, /**< This version is newer than any recommended version
+                       * in its series, and such recommended versions exist. */
+  VS_UNRECOMMENDED=4 /**< This version is not recommended (general case) */
+} version_status_t;
+
 int router_get_router_hash(const char *s, char *digest);
 int router_get_dir_hash(const char *s, char *digest);
 int router_get_runningrouters_hash(const char *s, char *digest);
@@ -2146,26 +2201,26 @@ int router_get_networkstatus_v2_hash(const char *s, char *digest);
 int router_append_dirobj_signature(char *buf, size_t buf_len, const char *digest,
                                    crypto_pk_env_t *private_key);
 int router_parse_list_from_string(const char **s,
-                                  routerlist_t **dest,
-                                  smartlist_t *good_nickname_list,
-                                  int rr_format,
-                                  time_t published);
+                                  smartlist_t *dest);
 int router_parse_routerlist_from_directory(const char *s,
                                            routerlist_t **dest,
                                            crypto_pk_env_t *pkey,
                                            int check_version,
                                            int write_to_cache);
-running_routers_t *router_parse_runningrouters(const char *str,
-                                               int write_to_cache);
+int router_parse_runningrouters(const char *str);
+int router_parse_directory(const char *str);
 routerinfo_t *router_parse_entry_from_string(const char *s, const char *end);
 int router_add_exit_policy_from_string(routerinfo_t *router, const char *s);
 addr_policy_t *router_parse_addr_policy_from_string(const char *s,
                                                     int assume_action);
-int check_software_version_against_directory(const char *directory);
+version_status_t tor_version_is_obsolete(const char *myversion,
+                                         const char *versionlist);
+version_status_t version_status_join(version_status_t a, version_status_t b);
 int tor_version_parse(const char *s, tor_version_t *out);
 int tor_version_as_new_as(const char *platform, const char *cutoff);
 int tor_version_compare(tor_version_t *a, tor_version_t *b);
 void assert_addr_policy_ok(addr_policy_t *t);
+void sort_routerstatus_entries(smartlist_t *sl);
 
 networkstatus_t *networkstatus_parse_from_string(const char *s);
 
