@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendclient.c,v 1.92 2005/09/09 19:54:28 arma Exp $ */
-const char rendclient_c_id[] = "$Id: rendclient.c,v 1.92 2005/09/09 19:54:28 arma Exp $";
+/* $Id: rendclient.c,v 1.94 2005/10/06 22:18:01 nickm Exp $ */
+const char rendclient_c_id[] = "$Id: rendclient.c,v 1.94 2005/10/06 22:18:01 nickm Exp $";
 
 /**
  * \file rendclient.c
@@ -460,14 +460,14 @@ rend_client_get_random_intro(const char *query)
   if (!entry->parsed->n_intro_points)
     return NULL;
 
-  i = crypto_pseudo_rand_int(entry->parsed->n_intro_points);
+  i = crypto_rand_int(entry->parsed->n_intro_points);
 
   if (entry->parsed->intro_point_extend_info) {
     return extend_info_dup(entry->parsed->intro_point_extend_info[i]);
   } else {
     /* add the intro point nicknames */
     char *choice = entry->parsed->intro_points[i];
-    routerinfo_t *router = router_get_by_nickname(choice);
+    routerinfo_t *router = router_get_by_nickname(choice, 0);
     if (!router) {
       log_fn(LOG_INFO, "Unknown router with nickname '%s'; trying another.",choice);
       tor_free(choice);
