@@ -1,7 +1,7 @@
 /* Copyright 2004 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendcommon.c,v 1.55 2005/09/09 20:58:44 arma Exp $ */
-const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.55 2005/09/09 20:58:44 arma Exp $";
+/* $Id: rendcommon.c,v 1.56 2005/09/30 01:09:52 nickm Exp $ */
+const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.56 2005/09/30 01:09:52 nickm Exp $";
 
 /**
  * \file rendcommon.c
@@ -12,13 +12,16 @@ const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.55 2005/09/09 20:58:44 arm
 #include "or.h"
 
 /** Return 0 if one and two are the same service ids, else -1 or 1 */
-int rend_cmp_service_ids(const char *one, const char *two) {
+int
+rend_cmp_service_ids(const char *one, const char *two)
+{
   return strcasecmp(one,two);
 }
 
 /** Free the storage held by the service descriptor <b>desc</b>.
  */
-void rend_service_descriptor_free(rend_service_descriptor_t *desc)
+void
+rend_service_descriptor_free(rend_service_descriptor_t *desc)
 {
   int i;
   if (desc->pk)
