@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendservice.c,v 1.138 2005/09/09 19:54:28 arma Exp $ */
-const char rendservice_c_id[] = "$Id: rendservice.c,v 1.138 2005/09/09 19:54:28 arma Exp $";
+/* $Id: rendservice.c,v 1.143 2005/10/07 21:38:44 nickm Exp $ */
+const char rendservice_c_id[] = "$Id: rendservice.c,v 1.143 2005/10/07 21:38:44 nickm Exp $";
 
 /**
  * \file rendservice.c
@@ -18,7 +18,7 @@ static circuit_t *find_intro_circuit(routerinfo_t *router, const char *pk_digest
 typedef struct rend_service_port_config_t {
   uint16_t virtual_port;
   uint16_t real_port;
-  uint32_t real_address;
+  uint32_t real_addr;
 } rend_service_port_config_t;
 
 /** Try to maintain this many intro points per service if possible. */
@@ -62,7 +62,9 @@ typedef struct rend_service_t {
 static smartlist_t *rend_service_list = NULL;
 
 /** Return the number of rendezvous services we have configured. */
-int num_rend_services(void) {
+int
+num_rend_services(void)
+{
   if (!rend_service_list)
     return 0;
   return smartlist_len(rend_service_list);
@@ -126,7 +128,7 @@ add_service(rend_service_t *service)
     for (i = 0; i < smartlist_len(service->ports); ++i) {
       char addrbuf[INET_NTOA_BUF_LEN];
       p = smartlist_get(service->ports, i);
-      addr.s_addr = htonl(p->real_address);
+      addr.s_addr = htonl(p->real_addr);
       tor_inet_ntoa(&addr, addrbuf, sizeof(addrbuf));
       log_fn(LOG_DEBUG,"Service maps port %d to %s:%d",
              p->virtual_port, addrbuf, p->real_port);
@@ -189,7 +191,7 @@ parse_port_config(const char *string)
   result = tor_malloc(sizeof(rend_service_port_config_t));
   result->virtual_port = virtport;
   result->real_port = realport;
-  result->real_address = addr;
+  result->real_addr = addr;
  err:
   SMARTLIST_FOREACH(sl, char *, c, tor_free(c));
   smartlist_free(sl);
@@ -290,7 +292,7 @@ rend_service_update_descriptor(rend_service_t *service)
   d->intro_point_extend_info = tor_malloc_zero(sizeof(extend_info_t*)*n);
   d->protocols = (1<<2) | (1<<0); /* We support protocol 2 and protocol 0. */
   for (i=0; i < n; ++i) {
-    router = router_get_by_nickname(smartlist_get(service->intro_nodes, i));
+    router = router_get_by_nickname(smartlist_get(service->intro_nodes, i),1);
     if (!router) {
       log_fn(LOG_INFO,"Router '%s' not found. Skipping.",
              (char*)smartlist_get(service->intro_nodes, i));
@@ -374,7 +376,8 @@ rend_service_get_by_pk_digest(const char* digest)
  * to have good uptime. Else return 0.
  */
 static int
-rend_service_requires_uptime(rend_service_t *service) {
+rend_service_requires_uptime(rend_service_t *service)
+{
   int i;
   rend_service_port_config_t *p;
 
@@ -509,7 +512,7 @@ rend_service_introduce(circuit_t *circuit, const char *request, size_t request_l
     ptr = rp_nickname+nickname_field_len;
     len -= nickname_field_len;
     len -= rp_nickname - buf; /* also remove header space used by version, if any */
-    router = router_get_by_nickname(rp_nickname);
+    router = router_get_by_nickname(rp_nickname, 0);
     if (!router) {
       log_fn(LOG_INFO, "Couldn't find router '%s' named in rendezvous cell.",
              rp_nickname);
@@ -931,7 +934,7 @@ rend_services_introduce(void)
     /* Find out which introduction points we have in progress for this service. */
     for (j=0; j < smartlist_len(service->intro_nodes); ++j) {
       intro = smartlist_get(service->intro_nodes, j);
-      router = router_get_by_nickname(intro);
+      router = router_get_by_nickname(intro, 0);
       if (!router || !find_intro_circuit(router,service->pk_digest)) {
         log_fn(LOG_INFO,"Giving up on %s as intro point for %s.",
                 intro, service->service_id);
@@ -960,6 +963,7 @@ rend_services_introduce(void)
     smartlist_add_all(exclude_routers, intro_routers);
     /* The directory is now here. Pick three ORs as intro points. */
     for (j=prev_intro_nodes; j < NUM_INTRO_POINTS; ++j) {
+      char *hex_digest;
       router = router_choose_random_node(service->intro_prefer_nodes,
                service->intro_exclude_nodes, exclude_routers, 1, 0,
                get_options()->_AllowUnverified & ALLOW_UNVERIFIED_INTRODUCTION, 0);
@@ -969,10 +973,13 @@ rend_services_introduce(void)
         break;
       }
       changed = 1;
+      hex_digest = tor_malloc_zero(HEX_DIGEST_LEN+2);
+      hex_digest[0] = '$';
+      base16_encode(hex_digest+1, HEX_DIGEST_LEN+1, router->identity_digest,
+                    DIGEST_LEN);
       smartlist_add(intro_routers, router);
       smartlist_add(exclude_routers, router);
-/*XXX009 should strdup the hexdigest, not nickname */
-      smartlist_add(service->intro_nodes, tor_strdup(router->nickname));
+      smartlist_add(service->intro_nodes, hex_digest);
       log_fn(LOG_INFO,"Picked router %s as an intro point for %s.", router->nickname,
              service->service_id);
     }
@@ -1017,7 +1024,7 @@ rend_consider_services_upload(time_t now)
     service = smartlist_get(rend_service_list, i);
     if (!service->next_upload_time) { /* never been uploaded yet */
       service->next_upload_time =
-        now + crypto_pseudo_rand_int(2*rendpostperiod);
+        now + crypto_rand_int(2*rendpostperiod);
     }
     if (service->next_upload_time < now ||
         (service->desc_is_dirty &&
@@ -1049,7 +1056,7 @@ rend_service_dump_stats(int severity)
     log(severity, "Service configured in \"%s\":", service->directory);
     for (j=0; j < smartlist_len(service->intro_nodes); ++j) {
       nickname = smartlist_get(service->intro_nodes, j);
-      router = router_get_by_nickname(smartlist_get(service->intro_nodes,j));
+      router = router_get_by_nickname(smartlist_get(service->intro_nodes,j),1);
       if (!router) {
         log(severity, "  Intro point at %s: unrecognized router",nickname);
         continue;
@@ -1091,7 +1098,7 @@ rend_service_set_connection_addr_port(connection_t *conn, circuit_t *circ)
   for (i = 0; i < smartlist_len(service->ports); ++i) {
     p = smartlist_get(service->ports, i);
     if (conn->port == p->virtual_port) {
-      conn->addr = p->real_address;
+      conn->addr = p->real_addr;
       conn->port = p->real_port;
       return 0;
     }
