@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.207 2005/09/13 21:39:42 arma Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.207 2005/09/13 21:39:42 arma Exp $";
+/* $Id: router.c,v 1.218 2005/10/07 18:56:21 arma Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.218 2005/10/07 18:56:21 arma Exp $";
 
 #include "or.h"
 
@@ -95,6 +95,8 @@ get_onion_key_set_at(void)
 void
 set_identity_key(crypto_pk_env_t *k)
 {
+  if (identitykey)
+    crypto_free_pk_env(identitykey);
   identitykey = k;
 }
 
@@ -368,7 +370,8 @@ init_keys(void)
   /* 6b. [authdirserver only] add own key to approved directories. */
   crypto_pk_get_digest(get_identity_key(), digest);
   if (!router_digest_is_trusted_dir(digest)) {
-    add_trusted_dir_server(NULL, (uint16_t)options->DirPort, digest,
+    add_trusted_dir_server(options->Nickname, NULL,
+                           (uint16_t)options->DirPort, digest,
                            options->V1AuthoritativeDir);
   }
   /* success */
@@ -404,7 +407,45 @@ check_whether_dirport_reachable(void)
          can_reach_dir_port;
 }
 
-/**DOCDOC*/
+/** Look at a variety of factors, and return 0 if we don't want to
+ * advertise the fact that we have a DirPort open. Else return the
+ * DirPort we want to advertise. */
+static int
+decide_to_advertise_dirport(or_options_t *options, routerinfo_t *router)
+{
+  if (!router->dir_port) /* short circuit the rest of the function */
+    return 0;
+  if (authdir_mode(options)) /* always publish */
+    return router->dir_port;
+  if (we_are_hibernating())
+    return 0;
+  if (!check_whether_dirport_reachable())
+    return 0;
+  if (router->bandwidthcapacity >= router->bandwidthrate) {
+    /* check if we might potentially hibernate. */
+    if (options->AccountingMax != 0)
+      return 0;
+    /* also check if we're advertising a small amount, and have
+       a "boring" DirPort. */
+    if (router->bandwidthrate < 50000 && router->dir_port > 1024)
+      return 0;
+  }
+
+  /* Sounds like a great idea. Let's publish it. */
+  return router->dir_port;
+}
+
+/** Some time has passed, or we just got new directory information.
+ * See if we currently believe our ORPort or DirPort to be
+ * unreachable. If so, launch a new test for it.
+ *
+ * For ORPort, we simply try making a circuit that ends at ourselves.
+ * Success is noticed in onionskin_answer().
+ *
+ * For DirPort, we make a connection via Tor to our DirPort and ask
+ * for our own server descriptor.
+ * Success is noticed in connection_dir_client_reached_eof().
+ */
 void
 consider_testing_reachability(void)
 {
@@ -419,11 +460,9 @@ consider_testing_reachability(void)
   }
 
   if (!check_whether_dirport_reachable()) {
-    if (me) {
-      directory_initiate_command_router(me, DIR_PURPOSE_FETCH_DIR, 1, NULL, NULL, 0);
-    } else {
-      log(LOG_NOTICE,"Delaying checking DirPort reachability; can't build descriptor.");
-    }
+    /* ask myself, via tor, for my server descriptor. */
+    directory_initiate_command_router(me, DIR_PURPOSE_FETCH_SERVERDESC,
+                                      1, "authority", NULL, 0);
   }
 }
 
@@ -717,6 +756,9 @@ router_get_my_descriptor(void)
   return desc_routerinfo->signed_descriptor;
 }
 
+/*DOCDOC*/
+static smartlist_t *warned_nonexistent_family = NULL;
+
 /** If <b>force</b> is true, or our descriptor is out-of-date, rebuild
  * a fresh routerinfo and signed server descriptor for this OR.
  * Return 0 on success, -1 on error.
@@ -743,8 +785,7 @@ router_rebuild_descriptor(int force)
   ri->nickname = tor_strdup(options->Nickname);
   ri->addr = addr;
   ri->or_port = options->ORPort;
-  ri->dir_port = hibernating ?
-                 0 : options->DirPort;
+  ri->dir_port = options->DirPort;
   ri->published_on = time(NULL);
   ri->onion_pkey = crypto_pk_dup_key(get_onion_key()); /* must invoke from main thread */
   ri->identity_pkey = crypto_pk_dup_key(get_identity_key());
@@ -772,14 +813,46 @@ router_rebuild_descriptor(int force)
   if (authdir_mode(options))
     ri->is_verified = ri->is_named = 1; /* believe in yourself */
   if (options->MyFamily) {
+    smartlist_t *family;
+    if (!warned_nonexistent_family)
+      warned_nonexistent_family = smartlist_create();
+    family = smartlist_create();
     ri->declared_family = smartlist_create();
-    smartlist_split_string(ri->declared_family, options->MyFamily, ",",
+    smartlist_split_string(family, options->MyFamily, ",",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
+    SMARTLIST_FOREACH(family, char *, name,
+     {
+       routerinfo_t *member;
+       if (!strcasecmp(name, options->Nickname))
+         member = ri;
+       else
+         member = router_get_by_nickname(name, 1);
+       if (!member) {
+         if (!smartlist_string_isin(warned_nonexistent_family, name)) {
+           log_fn(LOG_WARN, "I have no descriptor for the router named \"%s\" "
+                  "in my declared family; I'll use the nickname as is, but "
+                  "this may confuse clients.", name);
+           smartlist_add(warned_nonexistent_family, tor_strdup(name));
+         }
+         smartlist_add(ri->declared_family, name);
+         name = NULL;
+       } else {
+         char *fp = tor_malloc(HEX_DIGEST_LEN+2);
+         fp[0] = '$';
+         base16_encode(fp+1,HEX_DIGEST_LEN+1,
+                       member->identity_digest, DIGEST_LEN);
+         smartlist_add(ri->declared_family, fp);
+         if (smartlist_string_isin(warned_nonexistent_family, name))
+           smartlist_string_remove(warned_nonexistent_family, name);
+       }
+       tor_free(name);
+     });
+    smartlist_free(family);
   }
   ri->signed_descriptor = tor_malloc(8192);
   if (router_dump_router_to_string(ri->signed_descriptor, 8192,
                                    ri, get_identity_key())<0) {
-    log_fn(LOG_WARN, "Couldn't dump router to string.");
+    log_fn(LOG_WARN, "Couldn't allocate string for descriptor.");
     return -1;
   }
   ri->signed_descriptor_len = strlen(ri->signed_descriptor);
@@ -938,8 +1011,7 @@ router_dump_router_to_string(char *s, size_t maxlen, routerinfo_t *router,
     router->nickname,
     router->address,
     router->or_port,
-    (authdir_mode(options) || check_whether_dirport_reachable()) ?
-      router->dir_port : 0,
+    decide_to_advertise_dirport(options, router),
     router->platform,
     published,
     fingerprint,
@@ -1074,9 +1146,20 @@ is_legal_nickname_or_hexdigest(const char *s)
   return len == HEX_DIGEST_LEN+1 && strspn(s+1,HEX_CHARACTERS)==len-1;
 }
 
-/** Release all resources held in router keys. */
+/** Forget that we have issued any router-related warnings, so that we'll
+ * warn again if we see the same errors. */
 void
-router_free_all_keys(void)
+router_reset_warnings(void)
+{
+  if (warned_nonexistent_family) {
+    SMARTLIST_FOREACH(warned_nonexistent_family, char *, cp, tor_free(cp));
+    smartlist_clear(warned_nonexistent_family);
+  }
+}
+
+/** Release all static resources held in router.c */
+void
+router_free_all(void)
 {
   if (onionkey)
     crypto_free_pk_env(onionkey);
@@ -1088,5 +1171,9 @@ router_free_all_keys(void)
     tor_mutex_free(key_lock);
   if (desc_routerinfo)
     routerinfo_free(desc_routerinfo);
+  if (warned_nonexistent_family) {
+    SMARTLIST_FOREACH(warned_nonexistent_family, char *, cp, tor_free(cp));
+    smartlist_free(warned_nonexistent_family);
+  }
 }
 
