@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.137 2005/09/13 21:14:55 nickm Exp $ */
-const char routerparse_c_id[] = "$Id: routerparse.c,v 1.137 2005/09/13 21:14:55 nickm Exp $";
+/* $Id: routerparse.c,v 1.152 2005/10/07 18:33:30 nickm Exp $ */
+const char routerparse_c_id[] = "$Id: routerparse.c,v 1.152 2005/10/07 18:33:30 nickm Exp $";
 
 /**
  * \file routerparse.c
@@ -83,15 +83,15 @@ typedef struct directory_token_t {
 
 /** Rules for how many arguments a keyword can take. */
 typedef enum {
-  NO_ARGS,     /**< No arguments, ever */
-  ARGS,        /**< A list of arguments separated by spaces */
+  NO_ARGS,     /**< No arguments, ever. */
+  ARGS,        /**< A list of arguments separated by spaces. */
   CONCAT_ARGS, /**< The rest of the line, treated as a single argument. */
 } arg_syntax;
 
 /** Rules for whether the keyword needs an object. */
 typedef enum {
-  NO_OBJ,      /**< No object, ever */
-  NEED_OBJ,    /**< Object is required */
+  NO_OBJ,      /**< No object, ever. */
+  NEED_OBJ,    /**< Object is required. */
   NEED_KEY,    /**< Object is required, and must be a public key. */
   OBJ_OK,      /**< Object is optional. */
 } obj_syntax;
@@ -99,11 +99,11 @@ typedef enum {
 /** Rules for where a keyword can appear. */
 typedef enum {
   DIR = 1,        /**< Appears only in directory. */
-  RTR = 2,        /**< Appears only in router descriptor or runningrouters */
+  RTR = 2,        /**< Appears only in router descriptor or runningrouters. */
   NETSTATUS = 4,  /**< v2 or later ("versioned") network status. */
   ANYSIGNED = 7,  /**< Any "full" document (that is, not a router status.) */
   RTRSTATUS = 8,  /**< Router-status portion of a versioned network status. */
-  ANY = 15,       /**< Appears in any document type */
+  ANY = 15,       /**< Appears in any document type. */
 } where_syntax;
 
 /** Table mapping keywords to token value and to argument rules. */
@@ -166,8 +166,6 @@ static int check_directory_signature(const char *digest,
                                      crypto_pk_env_t *declared_key,
                                      int check_authority);
 static crypto_pk_env_t *find_dir_signing_key(const char *str);
-/* static */ int is_obsolete_version(const char *myversion,
-                                     const char *versionlist);
 static int tor_version_same_series(tor_version_t *a, tor_version_t *b);
 
 /** Set <b>digest</b> to the SHA-1 digest of the hash of the directory in
@@ -247,57 +245,33 @@ router_append_dirobj_signature(char *buf, size_t buf_len, const char *digest,
   return -1;
 }
 
-/**
- * Find the first instance of "recommended-software ...\n" at the start of
- * a line; return a newly allocated string containing the "..." portion.
- * Return NULL if no such instance was found.
- */
-static char *
-get_recommended_software_from_directory(const char *str)
-{
-#define REC "recommended-software "
-  const char *cp = str, *eol;
-  size_t len = strlen(REC);
-  cp = str;
-  if (strcmpstart(str, REC)==0) {
-    cp += len;
-  } else {
-    cp = strstr(str, "\n"REC);
-    if (!cp)
-      return NULL;
-    cp += len+1;
-  }
-  eol = strchr(cp, '\n');
-  if (!eol)
-    return NULL;
-  return tor_strndup(cp, eol-cp);
-#undef REC
-}
-
-/** Return 1 if <b>myversion</b> is not in <b>versionlist</b>, and if at least
- * one version of Tor on <b>versionlist</b> is newer than <b>myversion</b>.
-
- * Return 1 if no version from the same series as <b>myversion</b> is
- * in <b>versionlist</b> (and <b>myversion</b> is not the newest
- * version), or if a newer version from the same series is in
- * <b>versionlist</b>.
+/** Return VS_RECOMMENDED if <b>myversion</b> is contained in
+ * <b>versionlist</b>.  Else, return VS_OLD if every member of
+ * <b>versionlist</b> is newer than <b>myversion</b>.  Else, return
+ * VS_NEW_IN_SERIES if there is at least one member of <b>versionlist</b> in
+ * the same series (major.minor.micro) as <b>myversion</b>, but no such member
+ * is newer than <b>myversion.</b>.  Else, return VS_NEW if every memeber of
+ * <b>versionlist</b> is older than <b>myversion</b>.  Else, return
+ * VS_UNRECOMMENDED.
  *
- * Otherwise return 0.
  * (versionlist is a comma-separated list of version strings,
  * optionally prefixed with "Tor".  Versions that can't be parsed are
- * ignored.) */
-/* static */ int is_obsolete_version(const char *myversion,
-                           const char *versionlist) {
+ * ignored.)
+ */
+version_status_t
+tor_version_is_obsolete(const char *myversion, const char *versionlist)
+{
   const char *vl;
   tor_version_t mine, other;
-  int found_newer = 0, found_newer_in_series = 0, found_any_in_series = 0,
-    r, ret, same;
-  static int warned_too_new=0;
+  int found_newer = 0, found_older = 0, found_newer_in_series = 0,
+    found_any_in_series = 0, r, same;
+  version_status_t ret = VS_UNRECOMMENDED;
   smartlist_t *version_sl;
 
   vl = versionlist;
 
-  log_fn(LOG_DEBUG,"Checking whether version '%s' is in '%s'", myversion, versionlist);
+  log_fn(LOG_DEBUG,"Checking whether version '%s' is in '%s'",
+         myversion, versionlist);
 
   if (tor_version_parse(myversion, &mine)) {
     log_fn(LOG_ERR, "I couldn't parse my own version (%s)", myversion);
@@ -318,54 +292,28 @@ get_recommended_software_from_directory(const char *str)
         found_any_in_series = 1;
       r = tor_version_compare(&mine, &other);
       if (r==0) {
-        ret = 0;
+        ret = VS_RECOMMENDED;
         goto done;
       } else if (r<0) {
         found_newer = 1;
         if (same)
           found_newer_in_series = 1;
+      } else if (r>0) {
+        found_older = 1;
       }
     }
   });
 
   /* We didn't find the listed version. Is it new or old? */
-
-  if (found_any_in_series) {
-    if (!found_newer_in_series) {
-      /* We belong to a series with recommended members, and we are newer than
-       * any recommended member. We're probably okay. */
-      if (!warned_too_new) {
-        log(LOG_WARN, "This version of Tor (%s) is newer than any in the same series on the recommended list (%s)",
-            myversion, versionlist);
-        warned_too_new = 1;
-      }
-      ret = 0;
-    } else {
-      /* We found a newer one in the same series; we're obsolete. */
-      ret = 1;
-    }
+  if (found_any_in_series && !found_newer_in_series) {
+    ret = VS_NEW_IN_SERIES;
+  } else if (found_newer && !found_older) {
+    ret = VS_OLD;
+  } else if (found_older && !found_newer) {
+    ret = VS_NEW;
   } else {
-    if (found_newer) {
-      /* We belong to a series with no recommended members, and
-       * a newer series is recommended. We're obsolete. */
-      ret = 1;
-    } else {
-      /* We belong to a series with no recommended members, and it's
-       * newer than any recommended series. We're probably okay. */
-      if (!warned_too_new) {
-        log(LOG_WARN, "This version of Tor (%s) is newer than any on the recommended list (%s)",
-            myversion, versionlist);
-        warned_too_new = 1;
-      }
-      ret = 0;
-    }
+    ret = VS_UNRECOMMENDED;
   }
-  /*
-  log_fn(LOG_DEBUG,
-         "Decided that %s is %sobsolete relative to %s: %d, %d, %d\n",
-         myversion, ret?"":"not ", versionlist, found_newer,
-         found_any_in_series, found_newer_in_series);
-  */
 
  done:
   SMARTLIST_FOREACH(version_sl, char *, version, tor_free(version));
@@ -373,60 +321,54 @@ get_recommended_software_from_directory(const char *str)
   return ret;
 }
 
-/* Return 0 if myversion is supported; else warn and return -1. */
-int
-check_software_version_against_directory(const char *directory)
+/** Return the combined status of the current version, given that we know of
+ * one set of networkstatuses that give us status <b>a</b>, and another that
+ * gives us status <b>b</b>.
+ *
+ * For example, if one authority thinks that we're NEW, and another thinks
+ * we're OLD, we're simply UNRECOMMENDED.
+ *
+ * This function does not handle calculating whether we're RECOMMENDED; that
+ * follows a simple majority rule.  This function simply calculates *why*
+ * we're not recommended (if we're not).
+ */
+version_status_t
+version_status_join(version_status_t a, version_status_t b)
 {
-  char *v;
-  v = get_recommended_software_from_directory(directory);
-  if (!v) {
-    log_fn(LOG_WARN, "No recommended-versions string found in directory");
-    return -1;
-  }
-  if (!is_obsolete_version(VERSION, v)) {
-    tor_free(v);
-    return 0;
-  }
-  log(LOG_WARN,
-     "You are running Tor version %s, which will not work with this network.\n"
-     "Please use %s%s.",
-      VERSION, strchr(v,',') ? "one of " : "", v);
-  tor_free(v);
-  return -1;
+  if (a == b)
+    return a;
+  else if (a == VS_UNRECOMMENDED || b == VS_UNRECOMMENDED)
+    return VS_UNRECOMMENDED;
+  else if (a == VS_RECOMMENDED)
+    return b;
+  else if (b == VS_RECOMMENDED)
+    return a;
+  /* Okay.  Neither is 'recommended' or 'unrecommended', and they differ. */
+  else if (a == VS_OLD || b == VS_OLD)
+    return VS_UNRECOMMENDED;
+  /* One is VS_NEW, the other is VS_NEW_IN_SERIES */
+  else
+    return VS_NEW_IN_SERIES;
 }
 
-/** Parse a directory from <b>str</b> and, when done, store the
- * resulting routerlist in *<b>dest</b>, freeing the old value if
- * necessary.
- *
- * If <b>pkey</b> is provided, we check the directory signature with pkey.
- *
- * If <b>check_version</b> is non-zero, then examine the
- * Recommended-versions * line in the directory, and warn or quit
- * as needed.
- *
- * If <b>write_to_cache</b> is non-zero, then store this directory in
- * memory and/or disk as well.
+/** Read a signed directory from <b>str</b>.  If it's well-formed, return 0.
+ * Otherwise, return -1.  If we're a directory cache, cache it.
  */
-int /* Should be static; exposed for unit tests */
-router_parse_routerlist_from_directory(const char *str,
-                                       routerlist_t **dest,
-                                       crypto_pk_env_t *pkey,
-                                       int check_version,
-                                       int write_to_cache)
+int
+router_parse_directory(const char *str)
 {
   directory_token_t *tok;
   char digest[DIGEST_LEN];
-  routerlist_t *new_dir = NULL;
-  char *versions = NULL;
-  smartlist_t *good_nickname_list = NULL;
   time_t published_on;
-  int i, r;
+  int r;
   const char *end, *cp;
   smartlist_t *tokens = NULL;
-  char dirnickname[MAX_NICKNAME_LEN+1];
   crypto_pk_env_t *declared_key = NULL;
 
+  /* XXXX This could be simplified a lot, but it will all go away
+   * once pre-0.1.1.8 is obsolete, and for now it's better not to
+   * tuoch it. */
+
   if (router_get_dir_hash(str, digest)) {
     log_fn(LOG_WARN, "Unable to compute digest of directory");
     goto err;
@@ -453,24 +395,13 @@ router_parse_routerlist_from_directory(const char *str,
     log_fn(LOG_WARN,"Expected a single directory signature"); goto err;
   }
   declared_key = find_dir_signing_key(str);
-  if (check_directory_signature(digest, tok, pkey, declared_key, 1)<0)
+  if (check_directory_signature(digest, tok, NULL, declared_key, 1)<0)
     goto err;
 
-  /* now we know tok->n_args == 1, so it's safe to access tok->args[0] */
-  if (!is_legal_nickname(tok->args[0])) {
-    log_fn(LOG_WARN, "Directory nickname '%s' is misformed", tok->args[0]);
-    goto err;
-  }
-  strlcpy(dirnickname, tok->args[0], sizeof(dirnickname));
-
   SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
   smartlist_free(tokens);
   tokens = NULL;
 
-  /* Now that we know the signature is okay, check the version. */
-  if (check_version)
-    check_software_version_against_directory(str);
-
   /* Now try to parse the first part of the directory. */
   if ((end = strstr(str,"\nrouter "))) {
     ++end;
@@ -484,20 +415,6 @@ router_parse_routerlist_from_directory(const char *str,
   if (tokenize_string(str,end,tokens,DIR)) {
     log_fn(LOG_WARN, "Error tokenizing directory"); goto err;
   }
-  if (smartlist_len(tokens) < 1) {
-    log_fn(LOG_WARN, "Impossibly short directory header"); goto err;
-  }
-  if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
-    log_fn(LOG_WARN, "Unrecognized keyword \"%s\" in directory header; can't parse directory.",
-           tok->args[0]);
-    goto err;
-  }
-
-  tok = smartlist_get(tokens,0);
-  if (tok->tp != K_SIGNED_DIRECTORY) {
-    log_fn(LOG_WARN, "Directory doesn't start with signed-directory.");
-    goto err;
-  }
 
   if (!(tok = find_first_by_keyword(tokens, K_PUBLISHED))) {
     log_fn(LOG_WARN, "Missing published time on directory.");
@@ -511,82 +428,13 @@ router_parse_routerlist_from_directory(const char *str,
 
   /* Now that we know the signature is okay, and we have a
    * publication time, cache the directory. */
-  if (!get_options()->AuthoritativeDir && write_to_cache)
+  if (get_options()->DirPort && !get_options()->V1AuthoritativeDir)
     dirserv_set_cached_directory(str, published_on, 0);
 
-  if (!(tok = find_first_by_keyword(tokens, K_RECOMMENDED_SOFTWARE))) {
-    log_fn(LOG_WARN, "Missing recommended-software line from directory.");
-    goto err;
-  }
-  if (tok->n_args > 1) {
-    log_fn(LOG_WARN, "Invalid recommended-software line");
-    goto err;
-  }
-  versions = tok->n_args ? tor_strdup(tok->args[0]) : tor_strdup("");
-
-  /* Prefer router-status, then running-routers. */
-  if (!(tok = find_first_by_keyword(tokens, K_ROUTER_STATUS))) {
-    log_fn(LOG_WARN,
-           "Missing router-status line from directory.");
-    goto err;
-  }
-
-  good_nickname_list = smartlist_create();
-  for (i=0; i<tok->n_args; ++i) {
-    smartlist_add(good_nickname_list, tok->args[i]);
-  }
-  tok->n_args = 0; /* Don't free the strings in good_nickname_list yet. */
-
-  /* Read the router list from s, advancing s up past the end of the last
-   * router. */
-  str = end;
-  if (router_parse_list_from_string(&str, &new_dir,
-                                    good_nickname_list,
-                                    tok->tp==K_RUNNING_ROUTERS,
-                                    published_on)) {
-    log_fn(LOG_WARN, "Error reading routers from directory");
-    goto err;
-  }
-
-  /* Determine if my routerinfo is considered verified. */
-  {
-    static int have_warned_about_unverified_status = 0;
-    routerinfo_t *me = router_get_my_routerinfo();
-    if (me) {
-      if (router_update_status_from_smartlist(me,
-            published_on, good_nickname_list)==1 &&
-          me->is_verified == 0 && !have_warned_about_unverified_status) {
-        log_fn(LOG_WARN,"Dirserver '%s' lists your server as unverified. Please consider sending your identity fingerprint to the tor-ops.", dirnickname);
-        have_warned_about_unverified_status = 1;
-      }
-    }
-  }
-
-  new_dir->software_versions = versions; versions = NULL;
-  new_dir->published_on = published_on;
-  new_dir->running_routers = tor_malloc_zero(sizeof(running_routers_t));
-  new_dir->running_routers->published_on = published_on;
-  new_dir->running_routers->running_routers = good_nickname_list;
-
-  SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
-  smartlist_free(tokens);
-  tokens = NULL;
-
-  if (*dest)
-    routerlist_free(*dest);
-  *dest = new_dir;
-
   r = 0;
   goto done;
  err:
   r = -1;
-  if (new_dir)
-    routerlist_free(new_dir);
-  tor_free(versions);
-  if (good_nickname_list) {
-    SMARTLIST_FOREACH(good_nickname_list, char *, n, tor_free(n));
-    smartlist_free(good_nickname_list);
-  }
  done:
   if (declared_key) crypto_free_pk_env(declared_key);
   if (tokens) {
@@ -596,17 +444,16 @@ router_parse_routerlist_from_directory(const char *str,
   return r;
 }
 
-/** Read a signed router status statement from <b>str</b>.  On
- * success, return it, and cache the original string if
- * <b>write_to_cache</b> is set.  Otherwise, return NULL.  */
-running_routers_t *
-router_parse_runningrouters(const char *str, int write_to_cache)
+/** Read a signed router status statement from <b>str</b>.  If it's
+ * well-formed, return 0.  Otherwise, return -1.  If we're a directory cache,
+ * cache it.*/
+int
+router_parse_runningrouters(const char *str)
 {
   char digest[DIGEST_LEN];
-  running_routers_t *new_list = NULL;
   directory_token_t *tok;
   time_t published_on;
-  int i;
+  int r = -1;
   crypto_pk_env_t *declared_key = NULL;
   smartlist_t *tokens = NULL;
 
@@ -637,28 +484,6 @@ router_parse_runningrouters(const char *str, int write_to_cache)
   if (parse_iso_time(tok->args[0], &published_on) < 0) {
      goto err;
   }
-
-  /* Now that we know the signature is okay, and we have a
-   * publication time, cache the list. */
-  if (!get_options()->AuthoritativeDir && write_to_cache)
-    dirserv_set_cached_directory(str, published_on, 1);
-
-  if (!(tok = find_first_by_keyword(tokens, K_ROUTER_STATUS))) {
-    if (!(tok = find_first_by_keyword(tokens, K_RUNNING_ROUTERS))) {
-      log_fn(LOG_WARN,
-             "Missing running-routers/router-status line from directory.");
-      goto err;
-    }
-  }
-
-  new_list = tor_malloc_zero(sizeof(running_routers_t));
-  new_list->published_on = published_on;
-  new_list->running_routers = smartlist_create();
-  for (i=0;i<tok->n_args;++i) {
-    smartlist_add(new_list->running_routers, tok->args[i]);
-  }
-  tok->n_args = 0; /* Don't free the elements of tok->args. */
-
   if (!(tok = find_first_by_keyword(tokens, K_DIRECTORY_SIGNATURE))) {
     log_fn(LOG_WARN, "Missing signature on running-routers");
     goto err;
@@ -667,19 +492,19 @@ router_parse_runningrouters(const char *str, int write_to_cache)
   if (check_directory_signature(digest, tok, NULL, declared_key, 1) < 0)
     goto err;
 
-  goto done;
+  /* Now that we know the signature is okay, and we have a
+   * publication time, cache the list. */
+  if (get_options()->DirPort && !get_options()->V1AuthoritativeDir)
+    dirserv_set_cached_directory(str, published_on, 1);
+
+  r = 0;
  err:
-  if (new_list) {
-    running_routers_free(new_list);
-    new_list = NULL;
-  }
- done:
   if (declared_key) crypto_free_pk_env(declared_key);
   if (tokens) {
     SMARTLIST_FOREACH(tokens, directory_token_t *, tok, token_free(tok));
     smartlist_free(tokens);
   }
-  return new_list;
+  return r;
 }
 
 /** Given a directory or running-routers string in <b>str</b>, try to
@@ -809,26 +634,19 @@ check_directory_signature(const char *digest,
 }
 
 /** Given a string *<b>s</b> containing a concatenated sequence of router
- * descriptors, parses them and stores the result in *<b>dest</b>.  If
- * good_nickname_list is provided, then routers are marked as
- * running/nonrunning and verified/unverified based on their status in the
- * list.  Otherwise, all routers are marked running and verified.  Advances
- * *s to a point immediately following the last router entry.  Returns 0 on
- * success and -1 on failure.
+ * descriptors, parses them and stores the result in <b>dest</b>.  All routers
+ * are marked running and verified.  Advances *s to a point immediately
+ * following the last router entry.  Returns 0 on success and -1 on failure.
  */
 int
-router_parse_list_from_string(const char **s, routerlist_t **dest,
-                              smartlist_t *good_nickname_list,
-                              int rr_format, time_t published_on)
+router_parse_list_from_string(const char **s, smartlist_t *dest)
 {
   routerinfo_t *router;
-  smartlist_t *routers;
   const char *end;
 
   tor_assert(s);
   tor_assert(*s);
-
-  routers = smartlist_create();
+  tor_assert(dest);
 
   while (1) {
     *s = eat_whitespace(*s);
@@ -850,25 +668,9 @@ router_parse_list_from_string(const char **s, routerlist_t **dest,
       continue;
     }
 
-    if (!good_nickname_list) {
-      router->is_running = 1; /* start out assuming all dirservers are up */
-      router->is_verified = router->is_named = 1;
-      router->status_set_at = time(NULL);
-    }
-    smartlist_add(routers, router);
-//    log_fn(LOG_DEBUG,"just added router #%d.",smartlist_len(routers));
+    smartlist_add(dest, router);
   }
 
-  if (good_nickname_list) {
-    SMARTLIST_FOREACH(good_nickname_list, const char *, cp,
-                  routers_update_status_from_entry(routers, published_on, cp));
-  }
-
-  if (*dest)
-    routerlist_free(*dest);
-  *dest = tor_malloc_zero(sizeof(routerlist_t));
-  (*dest)->routers = routers;
-
   return 0;
 }
 
@@ -892,6 +694,10 @@ router_parse_entry_from_string(const char *s, const char *end)
     end = s + strlen(s);
   }
 
+  /* point 'end' to a point immediately after the final newline. */
+  while (end > s+2 && *(end-1) == '\n' && *(end-2) == '\n')
+    --end;
+
   if (router_get_router_hash(s, digest) < 0) {
     log_fn(LOG_WARN, "Couldn't compute router hash.");
     return NULL;
@@ -921,7 +727,7 @@ router_parse_entry_from_string(const char *s, const char *end)
   router = tor_malloc_zero(sizeof(routerinfo_t));
   router->signed_descriptor = tor_strndup(s, end-s);
   router->signed_descriptor_len = end-s;
-  crypto_digest(router->signed_descriptor_digest, s, end-s);
+  memcpy(router->signed_descriptor_digest, digest, DIGEST_LEN);
   ports_set = bw_set = 0;
 
   if (tok->n_args == 2 || tok->n_args == 5 || tok->n_args == 6) {
@@ -1130,12 +936,9 @@ find_start_of_next_routerstatus(const char *s)
 static routerstatus_t *
 routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
 {
-#define BASE64_DIGEST_LEN 27
   const char *eos;
   routerstatus_t *rs = NULL;
   directory_token_t *tok;
-  char base64buf_in[BASE64_DIGEST_LEN+3];
-  char base64buf_out[256];
   char timebuf[ISO_TIME_LEN+1];
   struct in_addr in;
 
@@ -1173,33 +976,15 @@ routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
   }
   strlcpy(rs->nickname, tok->args[0], sizeof(rs->nickname));
 
-  if (strlen(tok->args[1]) != BASE64_DIGEST_LEN) {
-    log_fn(LOG_WARN, "Digest '%s' is wrong length in router status; skipping.",
-           tok->args[1]);
-    goto err;
-  }
-  memcpy(base64buf_in, tok->args[1], BASE64_DIGEST_LEN);
-  memcpy(base64buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
-  if (base64_decode(base64buf_out, sizeof(base64buf_out),
-                    base64buf_in, sizeof(base64buf_in)-1) != DIGEST_LEN) {
+  if (digest_from_base64(rs->identity_digest, tok->args[1])) {
     log_fn(LOG_WARN, "Error decoding digest '%s'", tok->args[1]);
     goto err;
   }
-  memcpy(rs->identity_digest, base64buf_out, DIGEST_LEN);
 
-  if (strlen(tok->args[2]) != BASE64_DIGEST_LEN) {
-    log_fn(LOG_WARN, "Digest '%s' is wrong length in router status; skipping.",
-           tok->args[2]);
-    goto err;
-  }
-  memcpy(base64buf_in, tok->args[2], BASE64_DIGEST_LEN);
-  memcpy(base64buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
-  if (base64_decode(base64buf_out, sizeof(base64buf_out),
-                    base64buf_in, sizeof(base64buf_in)-1) != DIGEST_LEN) {
+  if (digest_from_base64(rs->descriptor_digest, tok->args[2])) {
     log_fn(LOG_WARN, "Error decoding digest '%s'", tok->args[2]);
     goto err;
   }
-  memcpy(rs->descriptor_digest, base64buf_out, DIGEST_LEN);
 
   if (tor_snprintf(timebuf, sizeof(timebuf), "%s %s",
                    tok->args[3], tok->args[4]) < 0 ||
@@ -1256,6 +1041,12 @@ _compare_routerstatus_entries(const void **_a, const void **_b)
   return memcmp(a->identity_digest, b->identity_digest, DIGEST_LEN);
 }
 
+void
+sort_routerstatus_entries(smartlist_t *sl)
+{
+  smartlist_sort(sl, _compare_routerstatus_entries);
+}
+
 /** Given a versioned (v2 or later) network-status object in <b>s</b>, try to
  * parse it and return the result.  Return NULL on failure.  Check the
  * signature of the network status, but do not (yet) check the signing key for
@@ -1271,6 +1062,7 @@ networkstatus_parse_from_string(const char *s)
   char tmp_digest[DIGEST_LEN];
   struct in_addr in;
   directory_token_t *tok;
+  int i;
 
   if (router_get_networkstatus_v2_hash(s, ns_digest)) {
     log_fn(LOG_WARN, "Unable to compute digest of network-status");
@@ -1294,7 +1086,6 @@ networkstatus_parse_from_string(const char *s)
     log_fn(LOG_WARN, "Couldn't find network-status-version keyword");
     goto err;
   }
-  /* XXXX do something with the version? NM */
 
   if (!(tok = find_first_by_keyword(tokens, K_DIR_SOURCE))) {
     log_fn(LOG_WARN, "Couldn't find dir-source keyword");
@@ -1352,25 +1143,31 @@ networkstatus_parse_from_string(const char *s)
     goto err;
   }
 
-  if (!(tok = find_first_by_keyword(tokens, K_CLIENT_VERSIONS)) || tok->n_args<1) {
-    log_fn(LOG_WARN, "Missing client-versions");
-    goto err;
+  if ((tok = find_first_by_keyword(tokens, K_DIR_OPTIONS))) {
+    for (i=0; i < tok->n_args; ++i) {
+      if (!strcmp(tok->args[i], "Names"))
+        ns->binds_names = 1;
+      if (!strcmp(tok->args[i], "Versions"))
+        ns->recommends_versions = 1;
+    }
   }
-  ns->client_versions = tok->args[0];
 
-  if (!(tok = find_first_by_keyword(tokens, K_CLIENT_VERSIONS)) || tok->n_args<1) {
-    log_fn(LOG_WARN, "Missing client-versions");
-    goto err;
-  }
-  ns->client_versions = tok->args[0];
-  tok->args[0] = NULL;
+  if (ns->recommends_versions) {
+    if (!(tok = find_first_by_keyword(tokens, K_CLIENT_VERSIONS)) ||
+        tok->n_args<1) {
+      log_fn(LOG_WARN, "Missing client-versions");
+    }
+    ns->client_versions = tok->args[0];
+    tok->args[0] = NULL;
 
-  if (!(tok = find_first_by_keyword(tokens, K_SERVER_VERSIONS)) || tok->n_args<1) {
-    log_fn(LOG_WARN, "Missing server-versions");
-    goto err;
+    if (!(tok = find_first_by_keyword(tokens, K_SERVER_VERSIONS)) ||
+        tok->n_args<1) {
+      log_fn(LOG_WARN, "Missing server-versions on versioning directory");
+      goto err;
+    }
+    ns->server_versions = tok->args[0];
+    tok->args[0] = NULL;
   }
-  ns->server_versions = tok->args[0];
-  tok->args[0] = NULL;
 
   if (!(tok = find_first_by_keyword(tokens, K_PUBLISHED))) {
     log_fn(LOG_WARN, "Missing published time on directory.");
@@ -1381,14 +1178,6 @@ networkstatus_parse_from_string(const char *s)
      goto err;
   }
 
-  if ((tok = find_first_by_keyword(tokens, K_DIR_OPTIONS))) {
-    int i;
-    for (i=0; i < tok->n_args; ++i) {
-      if (!strcmp(tok->args[i], "Names"))
-        ns->binds_names = 1;
-    }
-  }
-
   ns->entries = smartlist_create();
   s = eos;
   SMARTLIST_FOREACH(tokens, directory_token_t *, t, token_free(t));
@@ -1400,6 +1189,18 @@ networkstatus_parse_from_string(const char *s)
   }
   smartlist_sort(ns->entries, _compare_routerstatus_entries);
 
+  /* Kill duplicate entries. */
+  for (i=0; i < smartlist_len(ns->entries)-1; ++i) {
+    routerstatus_t *rs1 = smartlist_get(ns->entries, i);
+    routerstatus_t *rs2 = smartlist_get(ns->entries, i+1);
+    if (!memcmp(rs1->identity_digest,
+                rs2->identity_digest, DIGEST_LEN)) {
+      log_fn(LOG_WARN, "Network-status has two entries for the same router. Dropping one.");
+      smartlist_del_keeporder(ns->entries, i--);
+      routerstatus_free(rs1);
+    }
+  }
+
   if (tokenize_string(s, NULL, tokens, NETSTATUS)) {
     log_fn(LOG_WARN, "Error tokenizing network-status footer.");
     goto err;
